# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: steammessages_econ.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import steam.protobufs.steammessages_unified_base_pb2 as steammessages__unified__base__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='steammessages_econ.proto',
  package='',
  syntax='proto2',
  serialized_pb=_b('\n\x18steammessages_econ.proto\x1a steammessages_unified_base.proto\"C\n-CEcon_ClientGetItemShopOverlayAuthURL_Request\x12\x12\n\nreturn_url\x18\x01 \x01(\t\"=\n.CEcon_ClientGetItemShopOverlayAuthURL_Response\x12\x0b\n\x03url\x18\x01 \x01(\t2\xa3\x02\n\x04\x45\x63on\x12\xd9\x01\n\x1f\x43lientGetItemShopOverlayAuthURL\x12..CEcon_ClientGetItemShopOverlayAuthURL_Request\x1a/.CEcon_ClientGetItemShopOverlayAuthURL_Response\"U\x82\xb5\x18QGenerates a URL which sets a secure cookie for in-game-browser itemshop purchases\x1a?\x82\xb5\x18;A service that provides communication with the econ serversB\x03\x90\x01\x01')
  ,
  dependencies=[steammessages__unified__base__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_CECON_CLIENTGETITEMSHOPOVERLAYAUTHURL_REQUEST = _descriptor.Descriptor(
  name='CEcon_ClientGetItemShopOverlayAuthURL_Request',
  full_name='CEcon_ClientGetItemShopOverlayAuthURL_Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_url', full_name='CEcon_ClientGetItemShopOverlayAuthURL_Request.return_url', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=62,
  serialized_end=129,
)


_CECON_CLIENTGETITEMSHOPOVERLAYAUTHURL_RESPONSE = _descriptor.Descriptor(
  name='CEcon_ClientGetItemShopOverlayAuthURL_Response',
  full_name='CEcon_ClientGetItemShopOverlayAuthURL_Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='url', full_name='CEcon_ClientGetItemShopOverlayAuthURL_Response.url', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=131,
  serialized_end=192,
)

DESCRIPTOR.message_types_by_name['CEcon_ClientGetItemShopOverlayAuthURL_Request'] = _CECON_CLIENTGETITEMSHOPOVERLAYAUTHURL_REQUEST
DESCRIPTOR.message_types_by_name['CEcon_ClientGetItemShopOverlayAuthURL_Response'] = _CECON_CLIENTGETITEMSHOPOVERLAYAUTHURL_RESPONSE

CEcon_ClientGetItemShopOverlayAuthURL_Request = _reflection.GeneratedProtocolMessageType('CEcon_ClientGetItemShopOverlayAuthURL_Request', (_message.Message,), dict(
  DESCRIPTOR = _CECON_CLIENTGETITEMSHOPOVERLAYAUTHURL_REQUEST,
  __module__ = 'steammessages_econ_pb2'
  # @@protoc_insertion_point(class_scope:CEcon_ClientGetItemShopOverlayAuthURL_Request)
  ))
_sym_db.RegisterMessage(CEcon_ClientGetItemShopOverlayAuthURL_Request)

CEcon_ClientGetItemShopOverlayAuthURL_Response = _reflection.GeneratedProtocolMessageType('CEcon_ClientGetItemShopOverlayAuthURL_Response', (_message.Message,), dict(
  DESCRIPTOR = _CECON_CLIENTGETITEMSHOPOVERLAYAUTHURL_RESPONSE,
  __module__ = 'steammessages_econ_pb2'
  # @@protoc_insertion_point(class_scope:CEcon_ClientGetItemShopOverlayAuthURL_Response)
  ))
_sym_db.RegisterMessage(CEcon_ClientGetItemShopOverlayAuthURL_Response)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\220\001\001'))

_ECON = _descriptor.ServiceDescriptor(
  name='Econ',
  full_name='Econ',
  file=DESCRIPTOR,
  index=0,
  options=_descriptor._ParseOptions(descriptor_pb2.ServiceOptions(), _b('\202\265\030;A service that provides communication with the econ servers')),
  serialized_start=195,
  serialized_end=486,
  methods=[
  _descriptor.MethodDescriptor(
    name='ClientGetItemShopOverlayAuthURL',
    full_name='Econ.ClientGetItemShopOverlayAuthURL',
    index=0,
    containing_service=None,
    input_type=_CECON_CLIENTGETITEMSHOPOVERLAYAUTHURL_REQUEST,
    output_type=_CECON_CLIENTGETITEMSHOPOVERLAYAUTHURL_RESPONSE,
    options=_descriptor._ParseOptions(descriptor_pb2.MethodOptions(), _b('\202\265\030QGenerates a URL which sets a secure cookie for in-game-browser itemshop purchases')),
  ),
])

Econ = service_reflection.GeneratedServiceType('Econ', (_service.Service,), dict(
  DESCRIPTOR = _ECON,
  __module__ = 'steammessages_econ_pb2'
  ))

Econ_Stub = service_reflection.GeneratedServiceStubType('Econ_Stub', (Econ,), dict(
  DESCRIPTOR = _ECON,
  __module__ = 'steammessages_econ_pb2'
  ))


# @@protoc_insertion_point(module_scope)
