# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: steammessages_clientserver_2.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import steam.protobufs.steammessages_base_pb2 as steammessages__base__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='steammessages_clientserver_2.proto',
  package='',
  syntax='proto2',
  serialized_pb=_b('\n\"steammessages_clientserver_2.proto\x1a\x18steammessages_base.proto\"\x81\x03\n\x1a\x43MsgClientUCMAddScreenshot\x12\r\n\x05\x61ppid\x18\x01 \x01(\r\x12\x10\n\x08\x66ilename\x18\x02 \x01(\t\x12\x11\n\tthumbname\x18\x03 \x01(\t\x12\x13\n\x0bvr_filename\x18\x0e \x01(\t\x12\x17\n\x0frtime32_created\x18\x04 \x01(\x07\x12\r\n\x05width\x18\x05 \x01(\r\x12\x0e\n\x06height\x18\x06 \x01(\r\x12\x13\n\x0bpermissions\x18\x07 \x01(\r\x12\x0f\n\x07\x63\x61ption\x18\x08 \x01(\t\x12\x15\n\rshortcut_name\x18\t \x01(\t\x12,\n\x03tag\x18\n \x03(\x0b\x32\x1f.CMsgClientUCMAddScreenshot.Tag\x12\x16\n\x0etagged_steamid\x18\x0b \x03(\x06\x12\x13\n\x0bspoiler_tag\x18\x0c \x01(\x08\x12\x1e\n\x16tagged_publishedfileid\x18\r \x03(\x04\x1a*\n\x03Tag\x12\x10\n\x08tag_name\x18\x01 \x01(\t\x12\x11\n\ttag_value\x18\x02 \x01(\t\"d\n\"CMsgClientUCMAddScreenshotResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12*\n\x0cscreenshotid\x18\x02 \x01(\x06:\x14\x31\x38\x34\x34\x36\x37\x34\x34\x30\x37\x33\x37\x30\x39\x35\x35\x31\x36\x31\x35\"K\n\x1d\x43MsgClientUCMDeleteScreenshot\x12*\n\x0cscreenshotid\x18\x01 \x01(\x06:\x14\x31\x38\x34\x34\x36\x37\x34\x34\x30\x37\x33\x37\x30\x39\x35\x35\x31\x36\x31\x35\";\n%CMsgClientUCMDeleteScreenshotResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\"\xd1\x02\n\x18\x43MsgClientUCMPublishFile\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x11\n\tfile_name\x18\x02 \x01(\t\x12\x19\n\x11preview_file_name\x18\x03 \x01(\t\x12\x17\n\x0f\x63onsumer_app_id\x18\x04 \x01(\r\x12\r\n\x05title\x18\x05 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x06 \x01(\t\x12\x0c\n\x04tags\x18\x08 \x03(\t\x12\x15\n\rworkshop_file\x18\t \x01(\x08\x12\x12\n\nvisibility\x18\n \x01(\x05\x12\x11\n\tfile_type\x18\x0b \x01(\r\x12\x0b\n\x03url\x18\x0c \x01(\t\x12\x16\n\x0evideo_provider\x18\r \x01(\r\x12\x1a\n\x12video_account_name\x18\x0e \x01(\t\x12\x18\n\x10video_identifier\x18\x0f \x01(\t\x12\x13\n\x0bin_progress\x18\x10 \x01(\x08\"\xa1\x01\n CMsgClientUCMPublishFileResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12/\n\x11published_file_id\x18\x02 \x01(\x06:\x14\x31\x38\x34\x34\x36\x37\x34\x34\x30\x37\x33\x37\x30\x39\x35\x35\x31\x36\x31\x35\x12\x38\n)needs_workshop_legal_agreement_acceptance\x18\x03 \x01(\x08:\x05\x66\x61lse\"\x9c\x07\n CMsgClientUCMUpdatePublishedFile\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x19\n\x11published_file_id\x18\x02 \x01(\x06\x12\x11\n\tfile_name\x18\x03 \x01(\t\x12\x19\n\x11preview_file_name\x18\x04 \x01(\t\x12\r\n\x05title\x18\x05 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x06 \x01(\t\x12\x0c\n\x04tags\x18\x07 \x03(\t\x12\x12\n\nvisibility\x18\x08 \x01(\x05\x12\x13\n\x0bupdate_file\x18\t \x01(\x08\x12\x1b\n\x13update_preview_file\x18\n \x01(\x08\x12\x14\n\x0cupdate_title\x18\x0b \x01(\x08\x12\x1a\n\x12update_description\x18\x0c \x01(\x08\x12\x13\n\x0bupdate_tags\x18\r \x01(\x08\x12\x19\n\x11update_visibility\x18\x0e \x01(\x08\x12\x1a\n\x12\x63hange_description\x18\x0f \x01(\t\x12\x12\n\nupdate_url\x18\x10 \x01(\x08\x12\x0b\n\x03url\x18\x11 \x01(\t\x12\x1f\n\x17update_content_manifest\x18\x12 \x01(\x08\x12\x18\n\x10\x63ontent_manifest\x18\x13 \x01(\x06\x12\x10\n\x08metadata\x18\x14 \x01(\t\x12\x17\n\x0fupdate_metadata\x18\x15 \x01(\x08\x12\x13\n\x08language\x18\x16 \x01(\x05:\x01\x30\x12\x16\n\x0eremoved_kvtags\x18\x17 \x03(\t\x12=\n\x06kvtags\x18\x18 \x03(\x0b\x32-.CMsgClientUCMUpdatePublishedFile.KeyValueTag\x12\x45\n\x08previews\x18\x19 \x03(\x0b\x32\x33.CMsgClientUCMUpdatePublishedFile.AdditionalPreview\x12\x1a\n\x12previews_to_remove\x18\x1a \x03(\x05\x12\x19\n\x11\x63lear_in_progress\x18\x1b \x01(\x08\x1a)\n\x0bKeyValueTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x1a\x8c\x01\n\x11\x41\x64\x64itionalPreview\x12\x1a\n\x12original_file_name\x18\x01 \x01(\t\x12\x1a\n\x12internal_file_name\x18\x02 \x01(\t\x12\x0f\n\x07videoid\x18\x03 \x01(\t\x12\x14\n\x0cpreview_type\x18\x04 \x01(\r\x12\x18\n\x0cupdate_index\x18\x05 \x01(\x05:\x02-1\"x\n(CMsgClientUCMUpdatePublishedFileResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x38\n)needs_workshop_legal_agreement_acceptance\x18\x02 \x01(\x08:\x05\x66\x61lse\"=\n CMsgClientUCMDeletePublishedFile\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\">\n(CMsgClientUCMDeletePublishedFileResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\"c\n(CMsgClientUCMEnumerateUserPublishedFiles\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x13\n\x0bstart_index\x18\x02 \x01(\r\x12\x12\n\nsort_order\x18\x03 \x01(\r\"\xe7\x01\n0CMsgClientUCMEnumerateUserPublishedFilesResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12Z\n\x0fpublished_files\x18\x02 \x03(\x0b\x32\x41.CMsgClientUCMEnumerateUserPublishedFilesResponse.PublishedFileId\x12\x15\n\rtotal_results\x18\x03 \x01(\r\x1a,\n\x0fPublishedFileId\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\"\x98\x01\n)CMsgClientUCMEnumerateUserSubscribedFiles\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x13\n\x0bstart_index\x18\x02 \x01(\r\x12\x14\n\tlist_type\x18\x03 \x01(\r:\x01\x31\x12\x1d\n\x12matching_file_type\x18\x04 \x01(\r:\x01\x30\x12\x11\n\x05\x63ount\x18\x05 \x01(\r:\x02\x35\x30\"\x89\x02\n1CMsgClientUCMEnumerateUserSubscribedFilesResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\\\n\x10subscribed_files\x18\x02 \x03(\x0b\x32\x42.CMsgClientUCMEnumerateUserSubscribedFilesResponse.PublishedFileId\x12\x15\n\rtotal_results\x18\x03 \x01(\r\x1aK\n\x0fPublishedFileId\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x1d\n\x12rtime32_subscribed\x18\x02 \x01(\x07:\x01\x30\"o\n4CMsgClientUCMEnumerateUserSubscribedFilesWithUpdates\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x13\n\x0bstart_index\x18\x02 \x01(\r\x12\x12\n\nstart_time\x18\x03 \x01(\x07\"\x91\x03\n<CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12g\n\x10subscribed_files\x18\x02 \x03(\x0b\x32M.CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.PublishedFileId\x12\x15\n\rtotal_results\x18\x03 \x01(\r\x1a\xbc\x01\n\x0fPublishedFileId\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x1d\n\x12rtime32_subscribed\x18\x02 \x01(\x07:\x01\x30\x12\r\n\x05\x61ppid\x18\x03 \x01(\r\x12\x15\n\rfile_hcontent\x18\x04 \x01(\x06\x12\x11\n\tfile_size\x18\x05 \x01(\r\x12\x1c\n\x14rtime32_last_updated\x18\x06 \x01(\x07\x12\x18\n\x10is_depot_content\x18\x07 \x01(\x08\"\xc6\x01\n$CMsgClientUCMPublishedFileSubscribed\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x0e\n\x06\x61pp_id\x18\x02 \x01(\r\x12\x15\n\rfile_hcontent\x18\x03 \x01(\x06\x12\x11\n\tfile_size\x18\x04 \x01(\r\x12\x18\n\x10rtime_subscribed\x18\x05 \x01(\r\x12\x18\n\x10is_depot_content\x18\x06 \x01(\x08\x12\x15\n\rrtime_updated\x18\x07 \x01(\r\"S\n&CMsgClientUCMPublishedFileUnsubscribed\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x0e\n\x06\x61pp_id\x18\x02 \x01(\r\"N\n!CMsgClientUCMPublishedFileDeleted\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x0e\n\x06\x61pp_id\x18\x02 \x01(\r\"\xa3\x01\n!CMsgClientUCMPublishedFileUpdated\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x0e\n\x06\x61pp_id\x18\x02 \x01(\r\x12\x14\n\x0ctime_updated\x18\x03 \x01(\r\x12\x10\n\x08hcontent\x18\x04 \x01(\x06\x12\x11\n\tfile_size\x18\x05 \x01(\x07\x12\x18\n\x10is_depot_content\x18\x06 \x01(\x08\"k\n$CMsgClientWorkshopItemChangesRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x19\n\x11last_time_updated\x18\x02 \x01(\r\x12\x18\n\x10num_items_needed\x18\x03 \x01(\r\"\xfb\x01\n%CMsgClientWorkshopItemChangesResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x13\n\x0bupdate_time\x18\x02 \x01(\r\x12O\n\x0eworkshop_items\x18\x05 \x03(\x0b\x32\x37.CMsgClientWorkshopItemChangesResponse.WorkshopItemInfo\x1aX\n\x10WorkshopItemInfo\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x14\n\x0ctime_updated\x18\x02 \x01(\r\x12\x13\n\x0bmanifest_id\x18\x03 \x01(\x06\"\xd8\x01\n!CMsgClientWorkshopItemInfoRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x19\n\x11last_time_updated\x18\x02 \x01(\r\x12G\n\x0eworkshop_items\x18\x03 \x03(\x0b\x32/.CMsgClientWorkshopItemInfoRequest.WorkshopItem\x1a?\n\x0cWorkshopItem\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x14\n\x0ctime_updated\x18\x02 \x01(\r\"\x9f\x02\n\"CMsgClientWorkshopItemInfoResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x13\n\x0bupdate_time\x18\x02 \x01(\r\x12L\n\x0eworkshop_items\x18\x03 \x03(\x0b\x32\x34.CMsgClientWorkshopItemInfoResponse.WorkshopItemInfo\x12\x15\n\rprivate_items\x18\x04 \x03(\x06\x1ak\n\x10WorkshopItemInfo\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x14\n\x0ctime_updated\x18\x02 \x01(\r\x12\x13\n\x0bmanifest_id\x18\x03 \x01(\x06\x12\x11\n\tis_legacy\x18\x04 \x01(\x08\"\x94\x01\n%CMsgClientUCMGetPublishedFilesForUser\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x18\n\x10\x63reator_steam_id\x18\x02 \x01(\x06\x12\x15\n\rrequired_tags\x18\x03 \x03(\t\x12\x15\n\rexcluded_tags\x18\x04 \x03(\t\x12\x13\n\x0bstart_index\x18\x05 \x01(\r\"\xe1\x01\n-CMsgClientUCMGetPublishedFilesForUserResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12W\n\x0fpublished_files\x18\x02 \x03(\x0b\x32>.CMsgClientUCMGetPublishedFilesForUserResponse.PublishedFileId\x12\x15\n\rtotal_results\x18\x03 \x01(\r\x1a,\n\x0fPublishedFileId\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\"d\n\'CMsgClientUCMSetUserPublishedFileAction\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x0e\n\x06\x61pp_id\x18\x02 \x01(\r\x12\x0e\n\x06\x61\x63tion\x18\x03 \x01(\x05\"E\n/CMsgClientUCMSetUserPublishedFileActionResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\"g\n0CMsgClientUCMEnumeratePublishedFilesByUserAction\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x13\n\x0bstart_index\x18\x02 \x01(\r\x12\x0e\n\x06\x61\x63tion\x18\x03 \x01(\x05\"\x94\x02\n8CMsgClientUCMEnumeratePublishedFilesByUserActionResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x62\n\x0fpublished_files\x18\x02 \x03(\x0b\x32I.CMsgClientUCMEnumeratePublishedFilesByUserActionResponse.PublishedFileId\x12\x15\n\rtotal_results\x18\x03 \x01(\r\x1aI\n\x0fPublishedFileId\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x1b\n\x10rtime_time_stamp\x18\x02 \x01(\x07:\x01\x30\"\x1e\n\x1c\x43MsgClientScreenshotsChanged\"w\n\x1c\x43MsgClientUpdateUserGameInfo\x12\x14\n\x0csteamid_idgs\x18\x01 \x01(\x06\x12\x0e\n\x06gameid\x18\x02 \x01(\x06\x12\x0f\n\x07game_ip\x18\x03 \x01(\r\x12\x11\n\tgame_port\x18\x04 \x01(\r\x12\r\n\x05token\x18\x05 \x01(\x0c\"S\n\x1c\x43MsgClientRichPresenceUpload\x12\x18\n\x10rich_presence_kv\x18\x01 \x01(\x0c\x12\x19\n\x11steamid_broadcast\x18\x02 \x03(\x06\"8\n\x1d\x43MsgClientRichPresenceRequest\x12\x17\n\x0fsteamid_request\x18\x01 \x03(\x06\"\x9d\x01\n\x1a\x43MsgClientRichPresenceInfo\x12?\n\rrich_presence\x18\x01 \x03(\x0b\x32(.CMsgClientRichPresenceInfo.RichPresence\x1a>\n\x0cRichPresence\x12\x14\n\x0csteamid_user\x18\x01 \x01(\x06\x12\x18\n\x10rich_presence_kv\x18\x02 \x01(\x0c\".\n\x1c\x43MsgClientCheckFileSignature\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\"\xf7\x01\n$CMsgClientCheckFileSignatureResponse\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x0b\n\x03pid\x18\x02 \x01(\r\x12\x0f\n\x07\x65result\x18\x03 \x01(\r\x12\x10\n\x08\x66ilename\x18\x04 \x01(\t\x12\x18\n\x10\x65signatureresult\x18\x05 \x01(\r\x12\x10\n\x08sha_file\x18\x06 \x01(\x0c\x12\x17\n\x0fsignatureheader\x18\x07 \x01(\x0c\x12\x10\n\x08\x66ilesize\x18\x08 \x01(\r\x12\x14\n\x0cgetlasterror\x18\t \x01(\r\x12\"\n\x1a\x65valvesignaturecheckdetail\x18\n \x01(\r\"P\n\x19\x43MsgClientReadMachineAuth\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\x12\x0e\n\x06offset\x18\x02 \x01(\r\x12\x11\n\tcubtoread\x18\x03 \x01(\r\"\xce\x01\n!CMsgClientReadMachineAuthResponse\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\x12\x0f\n\x07\x65result\x18\x02 \x01(\r\x12\x10\n\x08\x66ilesize\x18\x03 \x01(\r\x12\x10\n\x08sha_file\x18\x04 \x01(\x0c\x12\x14\n\x0cgetlasterror\x18\x05 \x01(\r\x12\x0e\n\x06offset\x18\x06 \x01(\r\x12\x0f\n\x07\x63ubread\x18\x07 \x01(\r\x12\x12\n\nbytes_read\x18\x08 \x01(\x0c\x12\x17\n\x0f\x66ilename_sentry\x18\t \x01(\t\"\xbd\x01\n\x1b\x43MsgClientUpdateMachineAuth\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\x12\x0e\n\x06offset\x18\x02 \x01(\r\x12\x12\n\ncubtowrite\x18\x03 \x01(\r\x12\r\n\x05\x62ytes\x18\x04 \x01(\x0c\x12\x10\n\x08otp_type\x18\x05 \x01(\r\x12\x16\n\x0eotp_identifier\x18\x06 \x01(\t\x12\x18\n\x10otp_sharedsecret\x18\x07 \x01(\x0c\x12\x15\n\rotp_timedrift\x18\x08 \x01(\r\"\xe1\x01\n#CMsgClientUpdateMachineAuthResponse\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\x12\x0f\n\x07\x65result\x18\x02 \x01(\r\x12\x10\n\x08\x66ilesize\x18\x03 \x01(\r\x12\x10\n\x08sha_file\x18\x04 \x01(\x0c\x12\x14\n\x0cgetlasterror\x18\x05 \x01(\r\x12\x0e\n\x06offset\x18\x06 \x01(\r\x12\x10\n\x08\x63ubwrote\x18\x07 \x01(\r\x12\x10\n\x08otp_type\x18\x08 \x01(\x05\x12\x11\n\totp_value\x18\t \x01(\r\x12\x16\n\x0eotp_identifier\x18\n \x01(\t\"\xa1\x02\n\x1c\x43MsgClientRequestMachineAuth\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\x12\x1a\n\x12\x65result_sentryfile\x18\x02 \x01(\r\x12\x10\n\x08\x66ilesize\x18\x03 \x01(\r\x12\x16\n\x0esha_sentryfile\x18\x04 \x01(\x0c\x12\x1b\n\x13lock_account_action\x18\x06 \x01(\x05\x12\x10\n\x08otp_type\x18\x07 \x01(\r\x12\x16\n\x0eotp_identifier\x18\x08 \x01(\t\x12\x18\n\x10otp_sharedsecret\x18\t \x01(\x0c\x12\x11\n\totp_value\x18\n \x01(\r\x12\x14\n\x0cmachine_name\x18\x0b \x01(\t\x12\x1f\n\x17machine_name_userchosen\x18\x0c \x01(\t\"7\n$CMsgClientRequestMachineAuthResponse\x12\x0f\n\x07\x65result\x18\x01 \x01(\r\"B\n\x1c\x43MsgClientCreateFriendsGroup\x12\x0f\n\x07steamid\x18\x01 \x01(\x06\x12\x11\n\tgroupname\x18\x02 \x01(\t\"H\n$CMsgClientCreateFriendsGroupResponse\x12\x0f\n\x07\x65result\x18\x01 \x01(\r\x12\x0f\n\x07groupid\x18\x02 \x01(\x05\"@\n\x1c\x43MsgClientDeleteFriendsGroup\x12\x0f\n\x07steamid\x18\x01 \x01(\x06\x12\x0f\n\x07groupid\x18\x02 \x01(\x05\"7\n$CMsgClientDeleteFriendsGroupResponse\x12\x0f\n\x07\x65result\x18\x01 \x01(\r\"B\n\x1c\x43MsgClientRenameFriendsGroup\x12\x0f\n\x07groupid\x18\x01 \x01(\x05\x12\x11\n\tgroupname\x18\x02 \x01(\t\"7\n$CMsgClientRenameFriendsGroupResponse\x12\x0f\n\x07\x65result\x18\x01 \x01(\r\"B\n\x1a\x43MsgClientAddFriendToGroup\x12\x0f\n\x07groupid\x18\x01 \x01(\x05\x12\x13\n\x0bsteamiduser\x18\x02 \x01(\x06\"5\n\"CMsgClientAddFriendToGroupResponse\x12\x0f\n\x07\x65result\x18\x01 \x01(\r\"G\n\x1f\x43MsgClientRemoveFriendFromGroup\x12\x0f\n\x07groupid\x18\x01 \x01(\x05\x12\x13\n\x0bsteamiduser\x18\x02 \x01(\x06\":\n\'CMsgClientRemoveFriendFromGroupResponse\x12\x0f\n\x07\x65result\x18\x01 \x01(\r\"$\n\x15\x43MsgClientRegisterKey\x12\x0b\n\x03key\x18\x01 \x01(\t\"p\n\x1a\x43MsgClientPurchaseResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x1f\n\x17purchase_result_details\x18\x02 \x01(\x05\x12\x1d\n\x15purchase_receipt_info\x18\x03 \x01(\x0c\"\xc5\x01\n\x1c\x43MsgClientActivateOEMLicense\x12\x19\n\x11\x62ios_manufacturer\x18\x01 \x01(\t\x12\x19\n\x11\x62ios_serialnumber\x18\x02 \x01(\t\x12\x14\n\x0clicense_file\x18\x03 \x01(\x0c\x12\x1e\n\x16mainboard_manufacturer\x18\x04 \x01(\t\x12\x19\n\x11mainboard_product\x18\x05 \x01(\t\x12\x1e\n\x16mainboard_serialnumber\x18\x06 \x01(\t\"9\n\x1c\x43MsgClientRegisterOEMMachine\x12\x19\n\x11oem_register_file\x18\x01 \x01(\x0c\"7\n$CMsgClientRegisterOEMMachineResponse\x12\x0f\n\x07\x65result\x18\x01 \x01(\r\"K\n\x1f\x43MsgClientPurchaseWithMachineID\x12\x12\n\npackage_id\x18\x01 \x01(\r\x12\x14\n\x0cmachine_info\x18\x02 \x01(\x0c\"g\n CMsgTrading_InitiateTradeRequest\x12\x18\n\x10trade_request_id\x18\x01 \x01(\r\x12\x15\n\rother_steamid\x18\x02 \x01(\x04\x12\x12\n\nother_name\x18\x03 \x01(\t\"\xd2\x02\n!CMsgTrading_InitiateTradeResponse\x12\x10\n\x08response\x18\x01 \x01(\r\x12\x18\n\x10trade_request_id\x18\x02 \x01(\r\x12\x15\n\rother_steamid\x18\x03 \x01(\x04\x12 \n\x18steamguard_required_days\x18\x04 \x01(\r\x12 \n\x18new_device_cooldown_days\x18\x05 \x01(\r\x12-\n%default_password_reset_probation_days\x18\x06 \x01(\r\x12%\n\x1dpassword_reset_probation_days\x18\x07 \x01(\r\x12+\n#default_email_change_probation_days\x18\x08 \x01(\r\x12#\n\x1b\x65mail_change_probation_days\x18\t \x01(\r\"7\n\x1e\x43MsgTrading_CancelTradeRequest\x12\x15\n\rother_steamid\x18\x01 \x01(\x04\"1\n\x18\x43MsgTrading_StartSession\x12\x15\n\rother_steamid\x18\x01 \x01(\x04\"\xaf\x01\n\x15\x43MsgClientEmailChange\x12\x10\n\x08password\x18\x01 \x01(\t\x12\r\n\x05\x65mail\x18\x02 \x01(\t\x12\x0c\n\x04\x63ode\x18\x03 \x01(\t\x12\r\n\x05\x66inal\x18\x04 \x01(\x08\x12\x11\n\tnewmethod\x18\x05 \x01(\x08\x12\x16\n\x0etwofactor_code\x18\x06 \x01(\t\x12\x10\n\x08sms_code\x18\x07 \x01(\t\x12\x1b\n\x13\x63lient_supports_sms\x18\x08 \x01(\x08\"N\n\x1d\x43MsgClientEmailChangeResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x19\n\x11requires_sms_code\x18\x02 \x01(\x08\"P\n\x19\x43MsgClientGetCDNAuthToken\x12\x10\n\x08\x64\x65pot_id\x18\x01 \x01(\r\x12\x11\n\thost_name\x18\x02 \x01(\t\x12\x0e\n\x06\x61pp_id\x18\x03 \x01(\r\"C\n\x1f\x43MsgClientGetDepotDecryptionKey\x12\x10\n\x08\x64\x65pot_id\x18\x01 \x01(\r\x12\x0e\n\x06\x61pp_id\x18\x02 \x01(\r\"m\n\'CMsgClientGetDepotDecryptionKeyResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x10\n\x08\x64\x65pot_id\x18\x02 \x01(\r\x12\x1c\n\x14\x64\x65pot_encryption_key\x18\x03 \x01(\x0c\"F\n\x1e\x43MsgClientCheckAppBetaPassword\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x14\n\x0c\x62\x65tapassword\x18\x02 \x01(\t\"\xc1\x01\n&CMsgClientCheckAppBetaPasswordResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12K\n\rbetapasswords\x18\x04 \x03(\x0b\x32\x34.CMsgClientCheckAppBetaPasswordResponse.BetaPassword\x1a\x36\n\x0c\x42\x65taPassword\x12\x10\n\x08\x62\x65taname\x18\x01 \x01(\t\x12\x14\n\x0c\x62\x65tapassword\x18\x02 \x01(\t\"\xd9\x03\n\x1c\x43MsgClientUpdateAppJobReport\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x11\n\tdepot_ids\x18\x02 \x03(\r\x12\x11\n\tapp_state\x18\x03 \x01(\r\x12\x15\n\rjob_app_error\x18\x04 \x01(\r\x12\x15\n\rerror_details\x18\x05 \x01(\t\x12\x14\n\x0cjob_duration\x18\x06 \x01(\r\x12\x1f\n\x17\x66iles_validation_failed\x18\x07 \x01(\r\x12\x1c\n\x14job_bytes_downloaded\x18\x08 \x01(\x04\x12\x18\n\x10job_bytes_staged\x18\t \x01(\x04\x12\x16\n\x0e\x62ytes_comitted\x18\n \x01(\x04\x12\x17\n\x0fstart_app_state\x18\x0b \x01(\r\x12\x18\n\x10stats_machine_id\x18\x0c \x01(\x06\x12\x13\n\x0b\x62ranch_name\x18\r \x01(\t\x12\x1e\n\x16total_bytes_downloaded\x18\x0e \x01(\x04\x12\x1a\n\x12total_bytes_staged\x18\x0f \x01(\x04\x12\x1c\n\x14total_bytes_restored\x18\x10 \x01(\x04\x12\x13\n\x0bis_borrowed\x18\x11 \x01(\x08\x12\x17\n\x0fis_free_weekend\x18\x12 \x01(\x08\"\xcb\x01\n\x1e\x43MsgClientDPContentStatsReport\x12\x18\n\x10stats_machine_id\x18\x01 \x01(\x06\x12\x14\n\x0c\x63ountry_code\x18\x02 \x01(\t\x12\x0f\n\x07os_type\x18\x03 \x01(\x05\x12\x10\n\x08language\x18\x04 \x01(\x05\x12\x1b\n\x13num_install_folders\x18\x05 \x01(\r\x12\x1b\n\x13num_installed_games\x18\x06 \x01(\r\x12\x1c\n\x14size_installed_games\x18\x07 \x01(\x04\"_\n!CMsgClientGetCDNAuthTokenResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\r:\x01\x32\x12\r\n\x05token\x18\x02 \x01(\t\x12\x17\n\x0f\x65xpiration_time\x18\x03 \x01(\r\"\xb8\x01\n\x1a\x43MsgDownloadRateStatistics\x12\x0f\n\x07\x63\x65ll_id\x18\x01 \x01(\r\x12\x34\n\x05stats\x18\x02 \x03(\x0b\x32%.CMsgDownloadRateStatistics.StatsInfo\x1aS\n\tStatsInfo\x12\x13\n\x0bsource_type\x18\x01 \x01(\r\x12\x11\n\tsource_id\x18\x02 \x01(\r\x12\x0f\n\x07seconds\x18\x03 \x01(\r\x12\r\n\x05\x62ytes\x18\x04 \x01(\x04\"H\n\x1c\x43MsgClientRequestAccountData\x12\x18\n\x10\x61\x63\x63ount_or_email\x18\x01 \x01(\t\x12\x0e\n\x06\x61\x63tion\x18\x02 \x01(\r\"\xd7\x01\n$CMsgClientRequestAccountDataResponse\x12\x0e\n\x06\x61\x63tion\x18\x01 \x01(\r\x12\x0f\n\x07\x65result\x18\x02 \x01(\r\x12\x14\n\x0c\x61\x63\x63ount_name\x18\x03 \x01(\t\x12\x12\n\nct_matches\x18\x04 \x01(\r\x12 \n\x18\x61\x63\x63ount_name_suggestion1\x18\x05 \x01(\t\x12 \n\x18\x61\x63\x63ount_name_suggestion2\x18\x06 \x01(\t\x12 \n\x18\x61\x63\x63ount_name_suggestion3\x18\x07 \x01(\t\"\x99\x01\n\x1b\x43MsgClientUGSGetGlobalStats\x12\x0e\n\x06gameid\x18\x01 \x01(\x04\x12\x1e\n\x16history_days_requested\x18\x02 \x01(\r\x12\x1b\n\x13time_last_requested\x18\x03 \x01(\x07\x12\x18\n\x10\x66irst_day_cached\x18\x04 \x01(\r\x12\x13\n\x0b\x64\x61ys_cached\x18\x05 \x01(\r\"\x95\x02\n#CMsgClientUGSGetGlobalStatsResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x11\n\ttimestamp\x18\x02 \x01(\x07\x12\x13\n\x0b\x64\x61y_current\x18\x03 \x01(\x05\x12\x36\n\x04\x64\x61ys\x18\x04 \x03(\x0b\x32(.CMsgClientUGSGetGlobalStatsResponse.Day\x1az\n\x03\x44\x61y\x12\x0e\n\x06\x64\x61y_id\x18\x01 \x01(\r\x12<\n\x05stats\x18\x02 \x03(\x0b\x32-.CMsgClientUGSGetGlobalStatsResponse.Day.Stat\x1a%\n\x04Stat\x12\x0f\n\x07stat_id\x18\x01 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x03\"\xd4\x03\n\x12\x43MsgGameServerData\x12\x13\n\x0bsteam_id_gs\x18\x01 \x01(\x06\x12\n\n\x02ip\x18\x02 \x01(\r\x12\x12\n\nquery_port\x18\x03 \x01(\r\x12\x11\n\tgame_port\x18\x04 \x01(\r\x12\x15\n\rsourcetv_port\x18\x05 \x01(\r\x12\x0c\n\x04name\x18\x16 \x01(\t\x12\x0e\n\x06\x61pp_id\x18\x06 \x01(\r\x12\x0f\n\x07gamedir\x18\x07 \x01(\t\x12\x0f\n\x07version\x18\x08 \x01(\t\x12\x0f\n\x07product\x18\t \x01(\t\x12\x0e\n\x06region\x18\n \x01(\t\x12+\n\x07players\x18\x0b \x03(\x0b\x32\x1a.CMsgGameServerData.Player\x12\x13\n\x0bmax_players\x18\x0c \x01(\r\x12\x11\n\tbot_count\x18\r \x01(\r\x12\x10\n\x08password\x18\x0e \x01(\x08\x12\x0e\n\x06secure\x18\x0f \x01(\x08\x12\x11\n\tdedicated\x18\x10 \x01(\x08\x12\n\n\x02os\x18\x11 \x01(\t\x12\x11\n\tgame_data\x18\x12 \x01(\t\x12\x19\n\x11game_data_version\x18\x13 \x01(\r\x12\x11\n\tgame_type\x18\x14 \x01(\t\x12\x0b\n\x03map\x18\x15 \x01(\t\x1a\x1a\n\x06Player\x12\x10\n\x08steam_id\x18\x01 \x01(\x06\"H\n\x14\x43MsgGameServerRemove\x12\x10\n\x08steam_id\x18\x01 \x01(\x06\x12\n\n\x02ip\x18\x02 \x01(\r\x12\x12\n\nquery_port\x18\x03 \x01(\r\"\x82\x01\n\x18\x43MsgClientGMSServerQuery\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x17\n\x0fgeo_location_ip\x18\x02 \x01(\r\x12\x13\n\x0bregion_code\x18\x03 \x01(\r\x12\x13\n\x0b\x66ilter_text\x18\x04 \x01(\t\x12\x13\n\x0bmax_servers\x18\x05 \x01(\r\"\xb4\x01\n CMsgGMSClientServerQueryResponse\x12\x39\n\x07servers\x18\x01 \x03(\x0b\x32(.CMsgGMSClientServerQueryResponse.Server\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x1a\x46\n\x06Server\x12\x11\n\tserver_ip\x18\x01 \x01(\r\x12\x13\n\x0bserver_port\x18\x02 \x01(\r\x12\x14\n\x0c\x61uth_players\x18\x03 \x01(\r\"O\n\x17\x43MsgGameServerOutOfDate\x12\x13\n\x0bsteam_id_gs\x18\x01 \x01(\x06\x12\x0e\n\x06reject\x18\x02 \x01(\x08\x12\x0f\n\x07message\x18\x03 \x01(\t\"2\n\x19\x43MsgClientRedeemGuestPass\x12\x15\n\rguest_pass_id\x18\x01 \x01(\x06\"c\n!CMsgClientRedeemGuestPassResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\r:\x01\x32\x12\x12\n\npackage_id\x18\x02 \x01(\r\x12\x16\n\x0emust_own_appid\x18\x03 \x01(\r\"8\n\x1f\x43MsgClientGetClanActivityCounts\x12\x15\n\rsteamid_clans\x18\x01 \x03(\x04\"=\n\'CMsgClientGetClanActivityCountsResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\r:\x01\x32\"y\n\x19\x43MsgClientOGSReportString\x12\x13\n\x0b\x61\x63\x63umulated\x18\x01 \x01(\x08\x12\x11\n\tsessionid\x18\x02 \x01(\x04\x12\x10\n\x08severity\x18\x03 \x01(\x05\x12\x11\n\tformatter\x18\x04 \x01(\t\x12\x0f\n\x07varargs\x18\x05 \x01(\x0c\"P\n\x16\x43MsgClientOGSReportBug\x12\x11\n\tsessionid\x18\x01 \x01(\x04\x12\x0f\n\x07\x62ugtext\x18\x02 \x01(\t\x12\x12\n\nscreenshot\x18\x03 \x01(\x0c\"0\n\x17\x43MsgGSAssociateWithClan\x12\x15\n\rsteam_id_clan\x18\x01 \x01(\x06\"L\n\x1f\x43MsgGSAssociateWithClanResponse\x12\x15\n\rsteam_id_clan\x18\x01 \x01(\x06\x12\x12\n\x07\x65result\x18\x02 \x01(\r:\x01\x32\"A\n#CMsgGSComputeNewPlayerCompatibility\x12\x1a\n\x12steam_id_candidate\x18\x01 \x01(\x06\"\xcf\x01\n+CMsgGSComputeNewPlayerCompatibilityResponse\x12\x1a\n\x12steam_id_candidate\x18\x01 \x01(\x06\x12\x12\n\x07\x65result\x18\x02 \x01(\r:\x01\x32\x12\x16\n\x0eis_clan_member\x18\x03 \x01(\x08\x12\x18\n\x10\x63t_dont_like_you\x18\x04 \x01(\x05\x12\x18\n\x10\x63t_you_dont_like\x18\x05 \x01(\x05\x12$\n\x1c\x63t_clanmembers_dont_like_you\x18\x06 \x01(\x05\"\x14\n\x12\x43MsgClientSentLogs\"`\n\x0c\x43MsgGCClient\x12\r\n\x05\x61ppid\x18\x01 \x01(\r\x12\x0f\n\x07msgtype\x18\x02 \x01(\r\x12\x0f\n\x07payload\x18\x03 \x01(\x0c\x12\x0f\n\x07steamid\x18\x04 \x01(\x06\x12\x0e\n\x06gcname\x18\x05 \x01(\t\".\n\x1c\x43MsgClientRequestFreeLicense\x12\x0e\n\x06\x61ppids\x18\x02 \x03(\r\"n\n$CMsgClientRequestFreeLicenseResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\r:\x01\x32\x12\x1a\n\x12granted_packageids\x18\x02 \x03(\r\x12\x16\n\x0egranted_appids\x18\x03 \x03(\r\"\xd3\x01\n#CMsgDRMDownloadRequestWithCrashData\x12\x16\n\x0e\x64ownload_flags\x18\x01 \x01(\r\x12\x1c\n\x14\x64ownload_types_known\x18\x02 \x01(\r\x12\x10\n\x08guid_drm\x18\x03 \x01(\x0c\x12\x12\n\nguid_split\x18\x04 \x01(\x0c\x12\x12\n\nguid_merge\x18\x05 \x01(\x0c\x12\x13\n\x0bmodule_name\x18\x06 \x01(\t\x12\x13\n\x0bmodule_path\x18\x07 \x01(\t\x12\x12\n\ncrash_data\x18\x08 \x01(\x0c\"\xdb\x01\n\x17\x43MsgDRMDownloadResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\r:\x01\x32\x12\x0e\n\x06\x61pp_id\x18\x02 \x01(\r\x12\x1a\n\x12\x62lob_download_type\x18\x03 \x01(\r\x12\x12\n\nmerge_guid\x18\x04 \x01(\x0c\x12\x1c\n\x14\x64ownload_file_dfs_ip\x18\x05 \x01(\r\x12\x1e\n\x16\x64ownload_file_dfs_port\x18\x06 \x01(\r\x12\x19\n\x11\x64ownload_file_url\x18\x07 \x01(\t\x12\x13\n\x0bmodule_path\x18\x08 \x01(\t\"\xd7\x01\n\x12\x43MsgDRMFinalResult\x12\x12\n\x07\x65Result\x18\x01 \x01(\r:\x01\x32\x12\x0e\n\x06\x61pp_id\x18\x02 \x01(\r\x12\x1a\n\x12\x62lob_download_type\x18\x03 \x01(\r\x12\x14\n\x0c\x65rror_detail\x18\x04 \x01(\r\x12\x12\n\nmerge_guid\x18\x05 \x01(\x0c\x12\x1c\n\x14\x64ownload_file_dfs_ip\x18\x06 \x01(\r\x12\x1e\n\x16\x64ownload_file_dfs_port\x18\x07 \x01(\r\x12\x19\n\x11\x64ownload_file_url\x18\x08 \x01(\t\"3\n\x1e\x43MsgClientDPCheckSpecialSurvey\x12\x11\n\tsurvey_id\x18\x01 \x01(\r\"\x96\x01\n&CMsgClientDPCheckSpecialSurveyResponse\x12\x12\n\x07\x65Result\x18\x01 \x01(\r:\x01\x32\x12\r\n\x05state\x18\x02 \x01(\r\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x12\n\ncustom_url\x18\x04 \x01(\t\x12\x18\n\x10include_software\x18\x05 \x01(\x08\x12\r\n\x05token\x18\x06 \x01(\x0c\"H\n%CMsgClientDPSendSpecialSurveyResponse\x12\x11\n\tsurvey_id\x18\x01 \x01(\r\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"O\n*CMsgClientDPSendSpecialSurveyResponseReply\x12\x12\n\x07\x65Result\x18\x01 \x01(\r:\x01\x32\x12\r\n\x05token\x18\x02 \x01(\x0c\"W\n\'CMsgClientRequestForgottenPasswordEmail\x12\x14\n\x0c\x61\x63\x63ount_name\x18\x01 \x01(\t\x12\x16\n\x0epassword_tried\x18\x02 \x01(\t\"_\n/CMsgClientRequestForgottenPasswordEmailResponse\x12\x0f\n\x07\x65Result\x18\x01 \x01(\r\x12\x1b\n\x13use_secret_question\x18\x02 \x01(\x08\"6\n\x1b\x43MsgClientItemAnnouncements\x12\x17\n\x0f\x63ount_new_items\x18\x01 \x01(\r\"$\n\"CMsgClientRequestItemAnnouncements\"\x9e\x01\n\x1b\x43MsgClientUserNotifications\x12@\n\rnotifications\x18\x01 \x03(\x0b\x32).CMsgClientUserNotifications.Notification\x1a=\n\x0cNotification\x12\x1e\n\x16user_notification_type\x18\x01 \x01(\r\x12\r\n\x05\x63ount\x18\x02 \x01(\r\"\x88\x01\n\x1e\x43MsgClientCommentNotifications\x12\x1a\n\x12\x63ount_new_comments\x18\x01 \x01(\r\x12 \n\x18\x63ount_new_comments_owner\x18\x02 \x01(\r\x12(\n count_new_comments_subscriptions\x18\x03 \x01(\r\"\'\n%CMsgClientRequestCommentNotifications\"g\n$CMsgClientOfflineMessageNotification\x12\x18\n\x10offline_messages\x18\x01 \x01(\r\x12%\n\x1d\x66riends_with_offline_messages\x18\x02 \x03(\r\"&\n$CMsgClientRequestOfflineMessageCount\"6\n#CMsgClientFSGetFriendMessageHistory\x12\x0f\n\x07steamid\x18\x01 \x01(\x06\"\xf5\x01\n+CMsgClientFSGetFriendMessageHistoryResponse\x12\x0f\n\x07steamid\x18\x01 \x01(\x06\x12\x0f\n\x07success\x18\x02 \x01(\r\x12L\n\x08messages\x18\x03 \x03(\x0b\x32:.CMsgClientFSGetFriendMessageHistoryResponse.FriendMessage\x1aV\n\rFriendMessage\x12\x11\n\taccountid\x18\x01 \x01(\r\x12\x11\n\ttimestamp\x18\x02 \x01(\r\x12\x0f\n\x07message\x18\x03 \x01(\t\x12\x0e\n\x06unread\x18\x04 \x01(\x08\"7\n5CMsgClientFSGetFriendMessageHistoryForOfflineMessages\"7\n!CMsgClientFSGetFriendsSteamLevels\x12\x12\n\naccountids\x18\x01 \x03(\r\"\x9b\x01\n)CMsgClientFSGetFriendsSteamLevelsResponse\x12\x42\n\x07\x66riends\x18\x01 \x03(\x0b\x32\x31.CMsgClientFSGetFriendsSteamLevelsResponse.Friend\x1a*\n\x06\x46riend\x12\x11\n\taccountid\x18\x01 \x01(\r\x12\r\n\x05level\x18\x02 \x01(\r\"\xeb\x01\n\x17\x43MsgClientEmailAddrInfo\x12\x15\n\remail_address\x18\x01 \x01(\t\x12\x1a\n\x12\x65mail_is_validated\x18\x02 \x01(\x08\x12 \n\x18\x65mail_validation_changed\x18\x03 \x01(\x08\x12\'\n\x1f\x63redential_change_requires_code\x18\x04 \x01(\x08\x12\x31\n)password_or_secretqa_change_requires_code\x18\x05 \x01(\x08\x12\x1f\n\x17remind_user_about_email\x18\x06 \x01(\x08\"\xb7\x01\n\x1e\x43MsgCREEnumeratePublishedFiles\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x12\n\nquery_type\x18\x02 \x01(\x05\x12\x13\n\x0bstart_index\x18\x03 \x01(\r\x12\x0c\n\x04\x64\x61ys\x18\x04 \x01(\r\x12\r\n\x05\x63ount\x18\x05 \x01(\r\x12\x0c\n\x04tags\x18\x06 \x03(\t\x12\x11\n\tuser_tags\x18\x07 \x03(\t\x12\x1e\n\x12matching_file_type\x18\x08 \x01(\r:\x02\x31\x33\"\x9d\x02\n&CMsgCREEnumeratePublishedFilesResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12P\n\x0fpublished_files\x18\x02 \x03(\x0b\x32\x37.CMsgCREEnumeratePublishedFilesResponse.PublishedFileId\x12\x15\n\rtotal_results\x18\x03 \x01(\r\x1av\n\x0fPublishedFileId\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x11\n\tvotes_for\x18\x02 \x01(\x05\x12\x15\n\rvotes_against\x18\x03 \x01(\x05\x12\x0f\n\x07reports\x18\x04 \x01(\x05\x12\r\n\x05score\x18\x05 \x01(\x02\"\x8b\x01\n\x16\x43MsgCREItemVoteSummary\x12\x43\n\x12published_file_ids\x18\x01 \x03(\x0b\x32\'.CMsgCREItemVoteSummary.PublishedFileId\x1a,\n\x0fPublishedFileId\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\"\xfa\x01\n\x1e\x43MsgCREItemVoteSummaryResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12L\n\x13item_vote_summaries\x18\x02 \x03(\x0b\x32/.CMsgCREItemVoteSummaryResponse.ItemVoteSummary\x1av\n\x0fItemVoteSummary\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x11\n\tvotes_for\x18\x02 \x01(\x05\x12\x15\n\rvotes_against\x18\x03 \x01(\x05\x12\x0f\n\x07reports\x18\x04 \x01(\x05\x12\r\n\x05score\x18\x05 \x01(\x02\"P\n\"CMsgCREUpdateUserPublishedItemVote\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x0f\n\x07vote_up\x18\x02 \x01(\x08\"@\n*CMsgCREUpdateUserPublishedItemVoteResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\"\xab\x01\n&CMsgCREGetUserPublishedItemVoteDetails\x12S\n\x12published_file_ids\x18\x01 \x03(\x0b\x32\x37.CMsgCREGetUserPublishedItemVoteDetails.PublishedFileId\x1a,\n\x0fPublishedFileId\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\"\xea\x01\n.CMsgCREGetUserPublishedItemVoteDetailsResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x62\n\x16user_item_vote_details\x18\x02 \x03(\x0b\x32\x42.CMsgCREGetUserPublishedItemVoteDetailsResponse.UserItemVoteDetail\x1a@\n\x12UserItemVoteDetail\x12\x19\n\x11published_file_id\x18\x01 \x01(\x06\x12\x0f\n\x04vote\x18\x02 \x01(\x05:\x01\x30\"\xb9\x01\n\x18\x43MsgGameServerPingSample\x12\r\n\x05my_ip\x18\x01 \x01(\x07\x12\x11\n\tgs_app_id\x18\x02 \x01(\x05\x12\x34\n\ngs_samples\x18\x03 \x03(\x0b\x32 .CMsgGameServerPingSample.Sample\x1a\x45\n\x06Sample\x12\n\n\x02ip\x18\x01 \x01(\x07\x12\x13\n\x0b\x61vg_ping_ms\x18\x02 \x01(\r\x12\x1a\n\x12stddev_ping_ms_x10\x18\x03 \x01(\r\"*\n\x16\x43MsgFSGetFollowerCount\x12\x10\n\x08steam_id\x18\x01 \x01(\x06\"F\n\x1e\x43MsgFSGetFollowerCountResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x10\n\x05\x63ount\x18\x02 \x01(\x05:\x01\x30\"(\n\x14\x43MsgFSGetIsFollowing\x12\x10\n\x08steam_id\x18\x01 \x01(\x06\"O\n\x1c\x43MsgFSGetIsFollowingResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x1b\n\x0cis_following\x18\x02 \x01(\x08:\x05\x66\x61lse\"3\n\x1c\x43MsgFSEnumerateFollowingList\x12\x13\n\x0bstart_index\x18\x01 \x01(\r\"d\n$CMsgFSEnumerateFollowingListResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x15\n\rtotal_results\x18\x02 \x01(\x05\x12\x11\n\tsteam_ids\x18\x03 \x03(\x06\"0\n\x1f\x43MsgDPGetNumberOfCurrentPlayers\x12\r\n\x05\x61ppid\x18\x01 \x01(\r\"S\n\'CMsgDPGetNumberOfCurrentPlayersResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x14\n\x0cplayer_count\x18\x02 \x01(\x05\"a\n#CMsgClientFriendUserStatusPublished\x12\x16\n\x0e\x66riend_steamid\x18\x01 \x01(\x06\x12\r\n\x05\x61ppid\x18\x02 \x01(\r\x12\x13\n\x0bstatus_text\x18\x03 \x01(\t\"b\n\x17\x43MsgClientServiceMethod\x12\x13\n\x0bmethod_name\x18\x01 \x01(\t\x12\x19\n\x11serialized_method\x18\x02 \x01(\x0c\x12\x17\n\x0fis_notification\x18\x03 \x01(\x08\"Z\n\x1f\x43MsgClientServiceMethodResponse\x12\x13\n\x0bmethod_name\x18\x01 \x01(\t\x12\"\n\x1aserialized_method_response\x18\x02 \x01(\x0c\"\"\n\x10\x43MsgClientUIMode\x12\x0e\n\x06uimode\x18\x01 \x01(\r\"<\n&CMsgClientVanityURLChangedNotification\x12\x12\n\nvanity_url\x18\x01 \x01(\t\"y\n%CMsgClientAuthorizeLocalDeviceRequest\x12\x1a\n\x12\x64\x65vice_description\x18\x01 \x01(\t\x12\x18\n\x10owner_account_id\x18\x02 \x01(\r\x12\x1a\n\x12local_device_token\x18\x03 \x01(\x04\"k\n\x1e\x43MsgClientAuthorizeLocalDevice\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x18\n\x10owner_account_id\x18\x02 \x01(\r\x12\x1b\n\x13\x61uthed_device_token\x18\x03 \x01(\x04\"v\n*CMsgClientAuthorizeLocalDeviceNotification\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x18\n\x10owner_account_id\x18\x02 \x01(\r\x12\x1a\n\x12local_device_token\x18\x03 \x01(\x04\"n\n\"CMsgClientDeauthorizeDeviceRequest\x12\"\n\x1a\x64\x65\x61uthorization_account_id\x18\x01 \x01(\r\x12$\n\x1c\x64\x65\x61uthorization_device_token\x18\x02 \x01(\x04\"U\n\x1b\x43MsgClientDeauthorizeDevice\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\"\n\x1a\x64\x65\x61uthorization_account_id\x18\x02 \x01(\r\"\xd1\x01\n&CMsgClientUseLocalDeviceAuthorizations\x12 \n\x18\x61uthorization_account_id\x18\x01 \x03(\r\x12J\n\rdevice_tokens\x18\x02 \x03(\x0b\x32\x33.CMsgClientUseLocalDeviceAuthorizations.DeviceToken\x1a\x39\n\x0b\x44\x65viceToken\x12\x18\n\x10owner_account_id\x18\x01 \x01(\r\x12\x10\n\x08token_id\x18\x02 \x01(\x04\" \n\x1e\x43MsgClientGetAuthorizedDevices\"\xad\x02\n&CMsgClientGetAuthorizedDevicesResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12S\n\x11\x61uthorized_device\x18\x02 \x03(\x0b\x32\x38.CMsgClientGetAuthorizedDevicesResponse.AuthorizedDevice\x1a\x99\x01\n\x10\x41uthorizedDevice\x12\x19\n\x11\x61uth_device_token\x18\x01 \x01(\x04\x12\x13\n\x0b\x64\x65vice_name\x18\x02 \x01(\t\x12\x18\n\x10last_access_time\x18\x03 \x01(\r\x12\x13\n\x0b\x62orrower_id\x18\x04 \x01(\r\x12\x12\n\nis_pending\x18\x05 \x01(\x08\x12\x12\n\napp_played\x18\x06 \x01(\r\"\x1b\n\x19\x43MsgClientGetEmoticonList\"v\n\x16\x43MsgClientEmoticonList\x12\x33\n\temoticons\x18\x01 \x03(\x0b\x32 .CMsgClientEmoticonList.Emoticon\x1a\'\n\x08\x45moticon\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05\x63ount\x18\x02 \x01(\x05\"\xc2\x01\n!CMsgClientSharedLibraryLockStatus\x12H\n\x0elocked_library\x18\x01 \x03(\x0b\x32\x30.CMsgClientSharedLibraryLockStatus.LockedLibrary\x12\x1d\n\x15own_library_locked_by\x18\x02 \x01(\r\x1a\x34\n\rLockedLibrary\x12\x10\n\x08owner_id\x18\x01 \x01(\r\x12\x11\n\tlocked_by\x18\x02 \x01(\r\"\xa7\x01\n\"CMsgClientSharedLibraryStopPlaying\x12\x14\n\x0cseconds_left\x18\x01 \x01(\x05\x12>\n\tstop_apps\x18\x02 \x03(\x0b\x32+.CMsgClientSharedLibraryStopPlaying.StopApp\x1a+\n\x07StopApp\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\r\x12\x10\n\x08owner_id\x18\x02 \x01(\r\"\xf1\x01\n\x15\x43MsgClientServiceCall\x12\x15\n\rsysid_routing\x18\x01 \x01(\x0c\x12\x13\n\x0b\x63\x61ll_handle\x18\x02 \x01(\r\x12\x12\n\nmodule_crc\x18\x03 \x01(\r\x12\x13\n\x0bmodule_hash\x18\x04 \x01(\x0c\x12\x13\n\x0b\x66unction_id\x18\x05 \x01(\r\x12\x16\n\x0e\x63ub_output_max\x18\x06 \x01(\r\x12\r\n\x05\x66lags\x18\x07 \x01(\r\x12\x15\n\rcallparameter\x18\x08 \x01(\x0c\x12\x11\n\tping_only\x18\t \x01(\x08\x12\x1d\n\x15max_outstanding_calls\x18\n \x01(\r\"Z\n\x17\x43MsgClientServiceModule\x12\x12\n\nmodule_crc\x18\x01 \x01(\r\x12\x13\n\x0bmodule_hash\x18\x02 \x01(\x0c\x12\x16\n\x0emodule_content\x18\x03 \x01(\x0c\"\xb8\x04\n\x1d\x43MsgClientServiceCallResponse\x12\x15\n\rsysid_routing\x18\x01 \x01(\x0c\x12\x13\n\x0b\x63\x61ll_handle\x18\x02 \x01(\r\x12\x12\n\nmodule_crc\x18\x03 \x01(\r\x12\x13\n\x0bmodule_hash\x18\x04 \x01(\x0c\x12\x13\n\x0b\x65\x63\x61llresult\x18\x05 \x01(\r\x12\x16\n\x0eresult_content\x18\x06 \x01(\x0c\x12\x17\n\x0fos_version_info\x18\x07 \x01(\x0c\x12\x13\n\x0bsystem_info\x18\x08 \x01(\x0c\x12\x14\n\x0cload_address\x18\t \x01(\x06\x12\x18\n\x10\x65xception_record\x18\n \x01(\x0c\x12 \n\x18portable_os_version_info\x18\x0b \x01(\x0c\x12\x1c\n\x14portable_system_info\x18\x0c \x01(\x0c\x12\x15\n\rwas_converted\x18\r \x01(\x08\x12\x17\n\x0finternal_result\x18\x0e \x01(\r\x12\x15\n\rcurrent_count\x18\x0f \x01(\r\x12\x18\n\x10last_call_handle\x18\x10 \x01(\r\x12\x1c\n\x14last_call_module_crc\x18\x11 \x01(\r\x12\x1f\n\x17last_call_sysid_routing\x18\x12 \x01(\x0c\x12\x18\n\x10last_ecallresult\x18\x13 \x01(\r\x12\x1c\n\x14last_callissue_delta\x18\x14 \x01(\r\x12\x1f\n\x17last_callcomplete_delta\x18\x15 \x01(\r\"\x17\n\x15\x43MsgAMUnlockStreaming\"K\n\x1d\x43MsgAMUnlockStreamingResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x16\n\x0e\x65ncryption_key\x18\x02 \x01(\x0c\"M\n\x1d\x43MsgClientPlayingSessionState\x12\x17\n\x0fplaying_blocked\x18\x02 \x01(\x08\x12\x13\n\x0bplaying_app\x18\x03 \x01(\r\"6\n\x1c\x43MsgClientKickPlayingSession\x12\x16\n\x0eonly_stop_game\x18\x01 \x01(\x08\"t\n\x17\x43MsgClientCreateAccount\x12\x14\n\x0c\x61\x63\x63ount_name\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\t\x12\r\n\x05\x65mail\x18\x03 \x01(\t\x12\x10\n\x08launcher\x18\x06 \x01(\r\x12\x10\n\x08language\x18\x07 \x01(\r\"F\n\x1f\x43MsgClientCreateAccountResponse\x12\x12\n\x07\x65result\x18\x01 \x01(\x05:\x01\x32\x12\x0f\n\x07steamid\x18\x02 \x01(\x06\"v\n\x1f\x43MsgClientVoiceCallPreAuthorize\x12\x16\n\x0e\x63\x61ller_steamid\x18\x01 \x01(\x06\x12\x18\n\x10receiver_steamid\x18\x02 \x01(\x06\x12\x11\n\tcaller_id\x18\x03 \x01(\x05\x12\x0e\n\x06hangup\x18\x04 \x01(\x08\"\x82\x01\n\'CMsgClientVoiceCallPreAuthorizeResponse\x12\x16\n\x0e\x63\x61ller_steamid\x18\x01 \x01(\x06\x12\x18\n\x10receiver_steamid\x18\x02 \x01(\x06\x12\x12\n\x07\x65result\x18\x03 \x01(\x05:\x01\x32\x12\x11\n\tcaller_id\x18\x04 \x01(\x05\x42\x05H\x01\x90\x01\x00')
  ,
  dependencies=[steammessages__base__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_CMSGCLIENTUCMADDSCREENSHOT_TAG = _descriptor.Descriptor(
  name='Tag',
  full_name='CMsgClientUCMAddScreenshot.Tag',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tag_name', full_name='CMsgClientUCMAddScreenshot.Tag.tag_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tag_value', full_name='CMsgClientUCMAddScreenshot.Tag.tag_value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=408,
  serialized_end=450,
)

_CMSGCLIENTUCMADDSCREENSHOT = _descriptor.Descriptor(
  name='CMsgClientUCMAddScreenshot',
  full_name='CMsgClientUCMAddScreenshot',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='appid', full_name='CMsgClientUCMAddScreenshot.appid', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='filename', full_name='CMsgClientUCMAddScreenshot.filename', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='thumbname', full_name='CMsgClientUCMAddScreenshot.thumbname', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='vr_filename', full_name='CMsgClientUCMAddScreenshot.vr_filename', index=3,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rtime32_created', full_name='CMsgClientUCMAddScreenshot.rtime32_created', index=4,
      number=4, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='width', full_name='CMsgClientUCMAddScreenshot.width', index=5,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='height', full_name='CMsgClientUCMAddScreenshot.height', index=6,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='permissions', full_name='CMsgClientUCMAddScreenshot.permissions', index=7,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='caption', full_name='CMsgClientUCMAddScreenshot.caption', index=8,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='shortcut_name', full_name='CMsgClientUCMAddScreenshot.shortcut_name', index=9,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tag', full_name='CMsgClientUCMAddScreenshot.tag', index=10,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tagged_steamid', full_name='CMsgClientUCMAddScreenshot.tagged_steamid', index=11,
      number=11, type=6, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='spoiler_tag', full_name='CMsgClientUCMAddScreenshot.spoiler_tag', index=12,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tagged_publishedfileid', full_name='CMsgClientUCMAddScreenshot.tagged_publishedfileid', index=13,
      number=13, type=4, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTUCMADDSCREENSHOT_TAG, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=65,
  serialized_end=450,
)


_CMSGCLIENTUCMADDSCREENSHOTRESPONSE = _descriptor.Descriptor(
  name='CMsgClientUCMAddScreenshotResponse',
  full_name='CMsgClientUCMAddScreenshotResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUCMAddScreenshotResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='screenshotid', full_name='CMsgClientUCMAddScreenshotResponse.screenshotid', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=True, default_value=18446744073709551615,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=452,
  serialized_end=552,
)


_CMSGCLIENTUCMDELETESCREENSHOT = _descriptor.Descriptor(
  name='CMsgClientUCMDeleteScreenshot',
  full_name='CMsgClientUCMDeleteScreenshot',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='screenshotid', full_name='CMsgClientUCMDeleteScreenshot.screenshotid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=True, default_value=18446744073709551615,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=554,
  serialized_end=629,
)


_CMSGCLIENTUCMDELETESCREENSHOTRESPONSE = _descriptor.Descriptor(
  name='CMsgClientUCMDeleteScreenshotResponse',
  full_name='CMsgClientUCMDeleteScreenshotResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUCMDeleteScreenshotResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=631,
  serialized_end=690,
)


_CMSGCLIENTUCMPUBLISHFILE = _descriptor.Descriptor(
  name='CMsgClientUCMPublishFile',
  full_name='CMsgClientUCMPublishFile',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUCMPublishFile.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='file_name', full_name='CMsgClientUCMPublishFile.file_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='preview_file_name', full_name='CMsgClientUCMPublishFile.preview_file_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='consumer_app_id', full_name='CMsgClientUCMPublishFile.consumer_app_id', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='title', full_name='CMsgClientUCMPublishFile.title', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='CMsgClientUCMPublishFile.description', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tags', full_name='CMsgClientUCMPublishFile.tags', index=6,
      number=8, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='workshop_file', full_name='CMsgClientUCMPublishFile.workshop_file', index=7,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='visibility', full_name='CMsgClientUCMPublishFile.visibility', index=8,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='file_type', full_name='CMsgClientUCMPublishFile.file_type', index=9,
      number=11, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='url', full_name='CMsgClientUCMPublishFile.url', index=10,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='video_provider', full_name='CMsgClientUCMPublishFile.video_provider', index=11,
      number=13, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='video_account_name', full_name='CMsgClientUCMPublishFile.video_account_name', index=12,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='video_identifier', full_name='CMsgClientUCMPublishFile.video_identifier', index=13,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='in_progress', full_name='CMsgClientUCMPublishFile.in_progress', index=14,
      number=16, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=693,
  serialized_end=1030,
)


_CMSGCLIENTUCMPUBLISHFILERESPONSE = _descriptor.Descriptor(
  name='CMsgClientUCMPublishFileResponse',
  full_name='CMsgClientUCMPublishFileResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUCMPublishFileResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMPublishFileResponse.published_file_id', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=True, default_value=18446744073709551615,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='needs_workshop_legal_agreement_acceptance', full_name='CMsgClientUCMPublishFileResponse.needs_workshop_legal_agreement_acceptance', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1033,
  serialized_end=1194,
)


_CMSGCLIENTUCMUPDATEPUBLISHEDFILE_KEYVALUETAG = _descriptor.Descriptor(
  name='KeyValueTag',
  full_name='CMsgClientUCMUpdatePublishedFile.KeyValueTag',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='CMsgClientUCMUpdatePublishedFile.KeyValueTag.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='CMsgClientUCMUpdatePublishedFile.KeyValueTag.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1937,
  serialized_end=1978,
)

_CMSGCLIENTUCMUPDATEPUBLISHEDFILE_ADDITIONALPREVIEW = _descriptor.Descriptor(
  name='AdditionalPreview',
  full_name='CMsgClientUCMUpdatePublishedFile.AdditionalPreview',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='original_file_name', full_name='CMsgClientUCMUpdatePublishedFile.AdditionalPreview.original_file_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='internal_file_name', full_name='CMsgClientUCMUpdatePublishedFile.AdditionalPreview.internal_file_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='videoid', full_name='CMsgClientUCMUpdatePublishedFile.AdditionalPreview.videoid', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='preview_type', full_name='CMsgClientUCMUpdatePublishedFile.AdditionalPreview.preview_type', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='update_index', full_name='CMsgClientUCMUpdatePublishedFile.AdditionalPreview.update_index', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=-1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1981,
  serialized_end=2121,
)

_CMSGCLIENTUCMUPDATEPUBLISHEDFILE = _descriptor.Descriptor(
  name='CMsgClientUCMUpdatePublishedFile',
  full_name='CMsgClientUCMUpdatePublishedFile',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUCMUpdatePublishedFile.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMUpdatePublishedFile.published_file_id', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='file_name', full_name='CMsgClientUCMUpdatePublishedFile.file_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='preview_file_name', full_name='CMsgClientUCMUpdatePublishedFile.preview_file_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='title', full_name='CMsgClientUCMUpdatePublishedFile.title', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='CMsgClientUCMUpdatePublishedFile.description', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tags', full_name='CMsgClientUCMUpdatePublishedFile.tags', index=6,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='visibility', full_name='CMsgClientUCMUpdatePublishedFile.visibility', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='update_file', full_name='CMsgClientUCMUpdatePublishedFile.update_file', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='update_preview_file', full_name='CMsgClientUCMUpdatePublishedFile.update_preview_file', index=9,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='update_title', full_name='CMsgClientUCMUpdatePublishedFile.update_title', index=10,
      number=11, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='update_description', full_name='CMsgClientUCMUpdatePublishedFile.update_description', index=11,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='update_tags', full_name='CMsgClientUCMUpdatePublishedFile.update_tags', index=12,
      number=13, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='update_visibility', full_name='CMsgClientUCMUpdatePublishedFile.update_visibility', index=13,
      number=14, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='change_description', full_name='CMsgClientUCMUpdatePublishedFile.change_description', index=14,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='update_url', full_name='CMsgClientUCMUpdatePublishedFile.update_url', index=15,
      number=16, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='url', full_name='CMsgClientUCMUpdatePublishedFile.url', index=16,
      number=17, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='update_content_manifest', full_name='CMsgClientUCMUpdatePublishedFile.update_content_manifest', index=17,
      number=18, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='content_manifest', full_name='CMsgClientUCMUpdatePublishedFile.content_manifest', index=18,
      number=19, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='CMsgClientUCMUpdatePublishedFile.metadata', index=19,
      number=20, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='update_metadata', full_name='CMsgClientUCMUpdatePublishedFile.update_metadata', index=20,
      number=21, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='language', full_name='CMsgClientUCMUpdatePublishedFile.language', index=21,
      number=22, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='removed_kvtags', full_name='CMsgClientUCMUpdatePublishedFile.removed_kvtags', index=22,
      number=23, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='kvtags', full_name='CMsgClientUCMUpdatePublishedFile.kvtags', index=23,
      number=24, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='previews', full_name='CMsgClientUCMUpdatePublishedFile.previews', index=24,
      number=25, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='previews_to_remove', full_name='CMsgClientUCMUpdatePublishedFile.previews_to_remove', index=25,
      number=26, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='clear_in_progress', full_name='CMsgClientUCMUpdatePublishedFile.clear_in_progress', index=26,
      number=27, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTUCMUPDATEPUBLISHEDFILE_KEYVALUETAG, _CMSGCLIENTUCMUPDATEPUBLISHEDFILE_ADDITIONALPREVIEW, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1197,
  serialized_end=2121,
)


_CMSGCLIENTUCMUPDATEPUBLISHEDFILERESPONSE = _descriptor.Descriptor(
  name='CMsgClientUCMUpdatePublishedFileResponse',
  full_name='CMsgClientUCMUpdatePublishedFileResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUCMUpdatePublishedFileResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='needs_workshop_legal_agreement_acceptance', full_name='CMsgClientUCMUpdatePublishedFileResponse.needs_workshop_legal_agreement_acceptance', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2123,
  serialized_end=2243,
)


_CMSGCLIENTUCMDELETEPUBLISHEDFILE = _descriptor.Descriptor(
  name='CMsgClientUCMDeletePublishedFile',
  full_name='CMsgClientUCMDeletePublishedFile',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMDeletePublishedFile.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2245,
  serialized_end=2306,
)


_CMSGCLIENTUCMDELETEPUBLISHEDFILERESPONSE = _descriptor.Descriptor(
  name='CMsgClientUCMDeletePublishedFileResponse',
  full_name='CMsgClientUCMDeletePublishedFileResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUCMDeletePublishedFileResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2308,
  serialized_end=2370,
)


_CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILES = _descriptor.Descriptor(
  name='CMsgClientUCMEnumerateUserPublishedFiles',
  full_name='CMsgClientUCMEnumerateUserPublishedFiles',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUCMEnumerateUserPublishedFiles.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_index', full_name='CMsgClientUCMEnumerateUserPublishedFiles.start_index', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sort_order', full_name='CMsgClientUCMEnumerateUserPublishedFiles.sort_order', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2372,
  serialized_end=2471,
)


_CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILESRESPONSE_PUBLISHEDFILEID = _descriptor.Descriptor(
  name='PublishedFileId',
  full_name='CMsgClientUCMEnumerateUserPublishedFilesResponse.PublishedFileId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMEnumerateUserPublishedFilesResponse.PublishedFileId.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2661,
  serialized_end=2705,
)

_CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILESRESPONSE = _descriptor.Descriptor(
  name='CMsgClientUCMEnumerateUserPublishedFilesResponse',
  full_name='CMsgClientUCMEnumerateUserPublishedFilesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUCMEnumerateUserPublishedFilesResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='published_files', full_name='CMsgClientUCMEnumerateUserPublishedFilesResponse.published_files', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_results', full_name='CMsgClientUCMEnumerateUserPublishedFilesResponse.total_results', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILESRESPONSE_PUBLISHEDFILEID, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2474,
  serialized_end=2705,
)


_CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILES = _descriptor.Descriptor(
  name='CMsgClientUCMEnumerateUserSubscribedFiles',
  full_name='CMsgClientUCMEnumerateUserSubscribedFiles',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUCMEnumerateUserSubscribedFiles.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_index', full_name='CMsgClientUCMEnumerateUserSubscribedFiles.start_index', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='list_type', full_name='CMsgClientUCMEnumerateUserSubscribedFiles.list_type', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='matching_file_type', full_name='CMsgClientUCMEnumerateUserSubscribedFiles.matching_file_type', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='count', full_name='CMsgClientUCMEnumerateUserSubscribedFiles.count', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=50,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2708,
  serialized_end=2860,
)


_CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESRESPONSE_PUBLISHEDFILEID = _descriptor.Descriptor(
  name='PublishedFileId',
  full_name='CMsgClientUCMEnumerateUserSubscribedFilesResponse.PublishedFileId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMEnumerateUserSubscribedFilesResponse.PublishedFileId.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rtime32_subscribed', full_name='CMsgClientUCMEnumerateUserSubscribedFilesResponse.PublishedFileId.rtime32_subscribed', index=1,
      number=2, type=7, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3053,
  serialized_end=3128,
)

_CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESRESPONSE = _descriptor.Descriptor(
  name='CMsgClientUCMEnumerateUserSubscribedFilesResponse',
  full_name='CMsgClientUCMEnumerateUserSubscribedFilesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUCMEnumerateUserSubscribedFilesResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='subscribed_files', full_name='CMsgClientUCMEnumerateUserSubscribedFilesResponse.subscribed_files', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_results', full_name='CMsgClientUCMEnumerateUserSubscribedFilesResponse.total_results', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESRESPONSE_PUBLISHEDFILEID, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2863,
  serialized_end=3128,
)


_CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATES = _descriptor.Descriptor(
  name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdates',
  full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdates',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdates.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_index', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdates.start_index', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdates.start_time', index=2,
      number=3, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3130,
  serialized_end=3241,
)


_CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATESRESPONSE_PUBLISHEDFILEID = _descriptor.Descriptor(
  name='PublishedFileId',
  full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.PublishedFileId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.PublishedFileId.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rtime32_subscribed', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.PublishedFileId.rtime32_subscribed', index=1,
      number=2, type=7, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='appid', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.PublishedFileId.appid', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='file_hcontent', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.PublishedFileId.file_hcontent', index=3,
      number=4, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='file_size', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.PublishedFileId.file_size', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rtime32_last_updated', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.PublishedFileId.rtime32_last_updated', index=5,
      number=6, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_depot_content', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.PublishedFileId.is_depot_content', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3457,
  serialized_end=3645,
)

_CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATESRESPONSE = _descriptor.Descriptor(
  name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse',
  full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='subscribed_files', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.subscribed_files', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_results', full_name='CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.total_results', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATESRESPONSE_PUBLISHEDFILEID, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3244,
  serialized_end=3645,
)


_CMSGCLIENTUCMPUBLISHEDFILESUBSCRIBED = _descriptor.Descriptor(
  name='CMsgClientUCMPublishedFileSubscribed',
  full_name='CMsgClientUCMPublishedFileSubscribed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMPublishedFileSubscribed.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUCMPublishedFileSubscribed.app_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='file_hcontent', full_name='CMsgClientUCMPublishedFileSubscribed.file_hcontent', index=2,
      number=3, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='file_size', full_name='CMsgClientUCMPublishedFileSubscribed.file_size', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rtime_subscribed', full_name='CMsgClientUCMPublishedFileSubscribed.rtime_subscribed', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_depot_content', full_name='CMsgClientUCMPublishedFileSubscribed.is_depot_content', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rtime_updated', full_name='CMsgClientUCMPublishedFileSubscribed.rtime_updated', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3648,
  serialized_end=3846,
)


_CMSGCLIENTUCMPUBLISHEDFILEUNSUBSCRIBED = _descriptor.Descriptor(
  name='CMsgClientUCMPublishedFileUnsubscribed',
  full_name='CMsgClientUCMPublishedFileUnsubscribed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMPublishedFileUnsubscribed.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUCMPublishedFileUnsubscribed.app_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3848,
  serialized_end=3931,
)


_CMSGCLIENTUCMPUBLISHEDFILEDELETED = _descriptor.Descriptor(
  name='CMsgClientUCMPublishedFileDeleted',
  full_name='CMsgClientUCMPublishedFileDeleted',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMPublishedFileDeleted.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUCMPublishedFileDeleted.app_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3933,
  serialized_end=4011,
)


_CMSGCLIENTUCMPUBLISHEDFILEUPDATED = _descriptor.Descriptor(
  name='CMsgClientUCMPublishedFileUpdated',
  full_name='CMsgClientUCMPublishedFileUpdated',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMPublishedFileUpdated.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUCMPublishedFileUpdated.app_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time_updated', full_name='CMsgClientUCMPublishedFileUpdated.time_updated', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hcontent', full_name='CMsgClientUCMPublishedFileUpdated.hcontent', index=3,
      number=4, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='file_size', full_name='CMsgClientUCMPublishedFileUpdated.file_size', index=4,
      number=5, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_depot_content', full_name='CMsgClientUCMPublishedFileUpdated.is_depot_content', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4014,
  serialized_end=4177,
)


_CMSGCLIENTWORKSHOPITEMCHANGESREQUEST = _descriptor.Descriptor(
  name='CMsgClientWorkshopItemChangesRequest',
  full_name='CMsgClientWorkshopItemChangesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientWorkshopItemChangesRequest.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='last_time_updated', full_name='CMsgClientWorkshopItemChangesRequest.last_time_updated', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='num_items_needed', full_name='CMsgClientWorkshopItemChangesRequest.num_items_needed', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4179,
  serialized_end=4286,
)


_CMSGCLIENTWORKSHOPITEMCHANGESRESPONSE_WORKSHOPITEMINFO = _descriptor.Descriptor(
  name='WorkshopItemInfo',
  full_name='CMsgClientWorkshopItemChangesResponse.WorkshopItemInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientWorkshopItemChangesResponse.WorkshopItemInfo.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time_updated', full_name='CMsgClientWorkshopItemChangesResponse.WorkshopItemInfo.time_updated', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='manifest_id', full_name='CMsgClientWorkshopItemChangesResponse.WorkshopItemInfo.manifest_id', index=2,
      number=3, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4452,
  serialized_end=4540,
)

_CMSGCLIENTWORKSHOPITEMCHANGESRESPONSE = _descriptor.Descriptor(
  name='CMsgClientWorkshopItemChangesResponse',
  full_name='CMsgClientWorkshopItemChangesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientWorkshopItemChangesResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='update_time', full_name='CMsgClientWorkshopItemChangesResponse.update_time', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='workshop_items', full_name='CMsgClientWorkshopItemChangesResponse.workshop_items', index=2,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTWORKSHOPITEMCHANGESRESPONSE_WORKSHOPITEMINFO, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4289,
  serialized_end=4540,
)


_CMSGCLIENTWORKSHOPITEMINFOREQUEST_WORKSHOPITEM = _descriptor.Descriptor(
  name='WorkshopItem',
  full_name='CMsgClientWorkshopItemInfoRequest.WorkshopItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientWorkshopItemInfoRequest.WorkshopItem.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time_updated', full_name='CMsgClientWorkshopItemInfoRequest.WorkshopItem.time_updated', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4696,
  serialized_end=4759,
)

_CMSGCLIENTWORKSHOPITEMINFOREQUEST = _descriptor.Descriptor(
  name='CMsgClientWorkshopItemInfoRequest',
  full_name='CMsgClientWorkshopItemInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientWorkshopItemInfoRequest.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='last_time_updated', full_name='CMsgClientWorkshopItemInfoRequest.last_time_updated', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='workshop_items', full_name='CMsgClientWorkshopItemInfoRequest.workshop_items', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTWORKSHOPITEMINFOREQUEST_WORKSHOPITEM, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4543,
  serialized_end=4759,
)


_CMSGCLIENTWORKSHOPITEMINFORESPONSE_WORKSHOPITEMINFO = _descriptor.Descriptor(
  name='WorkshopItemInfo',
  full_name='CMsgClientWorkshopItemInfoResponse.WorkshopItemInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientWorkshopItemInfoResponse.WorkshopItemInfo.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time_updated', full_name='CMsgClientWorkshopItemInfoResponse.WorkshopItemInfo.time_updated', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='manifest_id', full_name='CMsgClientWorkshopItemInfoResponse.WorkshopItemInfo.manifest_id', index=2,
      number=3, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_legacy', full_name='CMsgClientWorkshopItemInfoResponse.WorkshopItemInfo.is_legacy', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4942,
  serialized_end=5049,
)

_CMSGCLIENTWORKSHOPITEMINFORESPONSE = _descriptor.Descriptor(
  name='CMsgClientWorkshopItemInfoResponse',
  full_name='CMsgClientWorkshopItemInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientWorkshopItemInfoResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='update_time', full_name='CMsgClientWorkshopItemInfoResponse.update_time', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='workshop_items', full_name='CMsgClientWorkshopItemInfoResponse.workshop_items', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='private_items', full_name='CMsgClientWorkshopItemInfoResponse.private_items', index=3,
      number=4, type=6, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTWORKSHOPITEMINFORESPONSE_WORKSHOPITEMINFO, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4762,
  serialized_end=5049,
)


_CMSGCLIENTUCMGETPUBLISHEDFILESFORUSER = _descriptor.Descriptor(
  name='CMsgClientUCMGetPublishedFilesForUser',
  full_name='CMsgClientUCMGetPublishedFilesForUser',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUCMGetPublishedFilesForUser.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='creator_steam_id', full_name='CMsgClientUCMGetPublishedFilesForUser.creator_steam_id', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='required_tags', full_name='CMsgClientUCMGetPublishedFilesForUser.required_tags', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='excluded_tags', full_name='CMsgClientUCMGetPublishedFilesForUser.excluded_tags', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_index', full_name='CMsgClientUCMGetPublishedFilesForUser.start_index', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5052,
  serialized_end=5200,
)


_CMSGCLIENTUCMGETPUBLISHEDFILESFORUSERRESPONSE_PUBLISHEDFILEID = _descriptor.Descriptor(
  name='PublishedFileId',
  full_name='CMsgClientUCMGetPublishedFilesForUserResponse.PublishedFileId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMGetPublishedFilesForUserResponse.PublishedFileId.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2661,
  serialized_end=2705,
)

_CMSGCLIENTUCMGETPUBLISHEDFILESFORUSERRESPONSE = _descriptor.Descriptor(
  name='CMsgClientUCMGetPublishedFilesForUserResponse',
  full_name='CMsgClientUCMGetPublishedFilesForUserResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUCMGetPublishedFilesForUserResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='published_files', full_name='CMsgClientUCMGetPublishedFilesForUserResponse.published_files', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_results', full_name='CMsgClientUCMGetPublishedFilesForUserResponse.total_results', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTUCMGETPUBLISHEDFILESFORUSERRESPONSE_PUBLISHEDFILEID, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5203,
  serialized_end=5428,
)


_CMSGCLIENTUCMSETUSERPUBLISHEDFILEACTION = _descriptor.Descriptor(
  name='CMsgClientUCMSetUserPublishedFileAction',
  full_name='CMsgClientUCMSetUserPublishedFileAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMSetUserPublishedFileAction.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUCMSetUserPublishedFileAction.app_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='action', full_name='CMsgClientUCMSetUserPublishedFileAction.action', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5430,
  serialized_end=5530,
)


_CMSGCLIENTUCMSETUSERPUBLISHEDFILEACTIONRESPONSE = _descriptor.Descriptor(
  name='CMsgClientUCMSetUserPublishedFileActionResponse',
  full_name='CMsgClientUCMSetUserPublishedFileActionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUCMSetUserPublishedFileActionResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5532,
  serialized_end=5601,
)


_CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTION = _descriptor.Descriptor(
  name='CMsgClientUCMEnumeratePublishedFilesByUserAction',
  full_name='CMsgClientUCMEnumeratePublishedFilesByUserAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUCMEnumeratePublishedFilesByUserAction.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_index', full_name='CMsgClientUCMEnumeratePublishedFilesByUserAction.start_index', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='action', full_name='CMsgClientUCMEnumeratePublishedFilesByUserAction.action', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5603,
  serialized_end=5706,
)


_CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTIONRESPONSE_PUBLISHEDFILEID = _descriptor.Descriptor(
  name='PublishedFileId',
  full_name='CMsgClientUCMEnumeratePublishedFilesByUserActionResponse.PublishedFileId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgClientUCMEnumeratePublishedFilesByUserActionResponse.PublishedFileId.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rtime_time_stamp', full_name='CMsgClientUCMEnumeratePublishedFilesByUserActionResponse.PublishedFileId.rtime_time_stamp', index=1,
      number=2, type=7, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5912,
  serialized_end=5985,
)

_CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTIONRESPONSE = _descriptor.Descriptor(
  name='CMsgClientUCMEnumeratePublishedFilesByUserActionResponse',
  full_name='CMsgClientUCMEnumeratePublishedFilesByUserActionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUCMEnumeratePublishedFilesByUserActionResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='published_files', full_name='CMsgClientUCMEnumeratePublishedFilesByUserActionResponse.published_files', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_results', full_name='CMsgClientUCMEnumeratePublishedFilesByUserActionResponse.total_results', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTIONRESPONSE_PUBLISHEDFILEID, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5709,
  serialized_end=5985,
)


_CMSGCLIENTSCREENSHOTSCHANGED = _descriptor.Descriptor(
  name='CMsgClientScreenshotsChanged',
  full_name='CMsgClientScreenshotsChanged',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5987,
  serialized_end=6017,
)


_CMSGCLIENTUPDATEUSERGAMEINFO = _descriptor.Descriptor(
  name='CMsgClientUpdateUserGameInfo',
  full_name='CMsgClientUpdateUserGameInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid_idgs', full_name='CMsgClientUpdateUserGameInfo.steamid_idgs', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='gameid', full_name='CMsgClientUpdateUserGameInfo.gameid', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='game_ip', full_name='CMsgClientUpdateUserGameInfo.game_ip', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='game_port', full_name='CMsgClientUpdateUserGameInfo.game_port', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='token', full_name='CMsgClientUpdateUserGameInfo.token', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6019,
  serialized_end=6138,
)


_CMSGCLIENTRICHPRESENCEUPLOAD = _descriptor.Descriptor(
  name='CMsgClientRichPresenceUpload',
  full_name='CMsgClientRichPresenceUpload',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rich_presence_kv', full_name='CMsgClientRichPresenceUpload.rich_presence_kv', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='steamid_broadcast', full_name='CMsgClientRichPresenceUpload.steamid_broadcast', index=1,
      number=2, type=6, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6140,
  serialized_end=6223,
)


_CMSGCLIENTRICHPRESENCEREQUEST = _descriptor.Descriptor(
  name='CMsgClientRichPresenceRequest',
  full_name='CMsgClientRichPresenceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid_request', full_name='CMsgClientRichPresenceRequest.steamid_request', index=0,
      number=1, type=6, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6225,
  serialized_end=6281,
)


_CMSGCLIENTRICHPRESENCEINFO_RICHPRESENCE = _descriptor.Descriptor(
  name='RichPresence',
  full_name='CMsgClientRichPresenceInfo.RichPresence',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid_user', full_name='CMsgClientRichPresenceInfo.RichPresence.steamid_user', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rich_presence_kv', full_name='CMsgClientRichPresenceInfo.RichPresence.rich_presence_kv', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6379,
  serialized_end=6441,
)

_CMSGCLIENTRICHPRESENCEINFO = _descriptor.Descriptor(
  name='CMsgClientRichPresenceInfo',
  full_name='CMsgClientRichPresenceInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rich_presence', full_name='CMsgClientRichPresenceInfo.rich_presence', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTRICHPRESENCEINFO_RICHPRESENCE, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6284,
  serialized_end=6441,
)


_CMSGCLIENTCHECKFILESIGNATURE = _descriptor.Descriptor(
  name='CMsgClientCheckFileSignature',
  full_name='CMsgClientCheckFileSignature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientCheckFileSignature.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6443,
  serialized_end=6489,
)


_CMSGCLIENTCHECKFILESIGNATURERESPONSE = _descriptor.Descriptor(
  name='CMsgClientCheckFileSignatureResponse',
  full_name='CMsgClientCheckFileSignatureResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientCheckFileSignatureResponse.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='pid', full_name='CMsgClientCheckFileSignatureResponse.pid', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientCheckFileSignatureResponse.eresult', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='filename', full_name='CMsgClientCheckFileSignatureResponse.filename', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='esignatureresult', full_name='CMsgClientCheckFileSignatureResponse.esignatureresult', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sha_file', full_name='CMsgClientCheckFileSignatureResponse.sha_file', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='signatureheader', full_name='CMsgClientCheckFileSignatureResponse.signatureheader', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='filesize', full_name='CMsgClientCheckFileSignatureResponse.filesize', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='getlasterror', full_name='CMsgClientCheckFileSignatureResponse.getlasterror', index=8,
      number=9, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='evalvesignaturecheckdetail', full_name='CMsgClientCheckFileSignatureResponse.evalvesignaturecheckdetail', index=9,
      number=10, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6492,
  serialized_end=6739,
)


_CMSGCLIENTREADMACHINEAUTH = _descriptor.Descriptor(
  name='CMsgClientReadMachineAuth',
  full_name='CMsgClientReadMachineAuth',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filename', full_name='CMsgClientReadMachineAuth.filename', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='offset', full_name='CMsgClientReadMachineAuth.offset', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cubtoread', full_name='CMsgClientReadMachineAuth.cubtoread', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6741,
  serialized_end=6821,
)


_CMSGCLIENTREADMACHINEAUTHRESPONSE = _descriptor.Descriptor(
  name='CMsgClientReadMachineAuthResponse',
  full_name='CMsgClientReadMachineAuthResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filename', full_name='CMsgClientReadMachineAuthResponse.filename', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientReadMachineAuthResponse.eresult', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='filesize', full_name='CMsgClientReadMachineAuthResponse.filesize', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sha_file', full_name='CMsgClientReadMachineAuthResponse.sha_file', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='getlasterror', full_name='CMsgClientReadMachineAuthResponse.getlasterror', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='offset', full_name='CMsgClientReadMachineAuthResponse.offset', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cubread', full_name='CMsgClientReadMachineAuthResponse.cubread', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bytes_read', full_name='CMsgClientReadMachineAuthResponse.bytes_read', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='filename_sentry', full_name='CMsgClientReadMachineAuthResponse.filename_sentry', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6824,
  serialized_end=7030,
)


_CMSGCLIENTUPDATEMACHINEAUTH = _descriptor.Descriptor(
  name='CMsgClientUpdateMachineAuth',
  full_name='CMsgClientUpdateMachineAuth',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filename', full_name='CMsgClientUpdateMachineAuth.filename', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='offset', full_name='CMsgClientUpdateMachineAuth.offset', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cubtowrite', full_name='CMsgClientUpdateMachineAuth.cubtowrite', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bytes', full_name='CMsgClientUpdateMachineAuth.bytes', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='otp_type', full_name='CMsgClientUpdateMachineAuth.otp_type', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='otp_identifier', full_name='CMsgClientUpdateMachineAuth.otp_identifier', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='otp_sharedsecret', full_name='CMsgClientUpdateMachineAuth.otp_sharedsecret', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='otp_timedrift', full_name='CMsgClientUpdateMachineAuth.otp_timedrift', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7033,
  serialized_end=7222,
)


_CMSGCLIENTUPDATEMACHINEAUTHRESPONSE = _descriptor.Descriptor(
  name='CMsgClientUpdateMachineAuthResponse',
  full_name='CMsgClientUpdateMachineAuthResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filename', full_name='CMsgClientUpdateMachineAuthResponse.filename', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUpdateMachineAuthResponse.eresult', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='filesize', full_name='CMsgClientUpdateMachineAuthResponse.filesize', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sha_file', full_name='CMsgClientUpdateMachineAuthResponse.sha_file', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='getlasterror', full_name='CMsgClientUpdateMachineAuthResponse.getlasterror', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='offset', full_name='CMsgClientUpdateMachineAuthResponse.offset', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cubwrote', full_name='CMsgClientUpdateMachineAuthResponse.cubwrote', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='otp_type', full_name='CMsgClientUpdateMachineAuthResponse.otp_type', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='otp_value', full_name='CMsgClientUpdateMachineAuthResponse.otp_value', index=8,
      number=9, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='otp_identifier', full_name='CMsgClientUpdateMachineAuthResponse.otp_identifier', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7225,
  serialized_end=7450,
)


_CMSGCLIENTREQUESTMACHINEAUTH = _descriptor.Descriptor(
  name='CMsgClientRequestMachineAuth',
  full_name='CMsgClientRequestMachineAuth',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filename', full_name='CMsgClientRequestMachineAuth.filename', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='eresult_sentryfile', full_name='CMsgClientRequestMachineAuth.eresult_sentryfile', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='filesize', full_name='CMsgClientRequestMachineAuth.filesize', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sha_sentryfile', full_name='CMsgClientRequestMachineAuth.sha_sentryfile', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='lock_account_action', full_name='CMsgClientRequestMachineAuth.lock_account_action', index=4,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='otp_type', full_name='CMsgClientRequestMachineAuth.otp_type', index=5,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='otp_identifier', full_name='CMsgClientRequestMachineAuth.otp_identifier', index=6,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='otp_sharedsecret', full_name='CMsgClientRequestMachineAuth.otp_sharedsecret', index=7,
      number=9, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='otp_value', full_name='CMsgClientRequestMachineAuth.otp_value', index=8,
      number=10, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='machine_name', full_name='CMsgClientRequestMachineAuth.machine_name', index=9,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='machine_name_userchosen', full_name='CMsgClientRequestMachineAuth.machine_name_userchosen', index=10,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7453,
  serialized_end=7742,
)


_CMSGCLIENTREQUESTMACHINEAUTHRESPONSE = _descriptor.Descriptor(
  name='CMsgClientRequestMachineAuthResponse',
  full_name='CMsgClientRequestMachineAuthResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientRequestMachineAuthResponse.eresult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7744,
  serialized_end=7799,
)


_CMSGCLIENTCREATEFRIENDSGROUP = _descriptor.Descriptor(
  name='CMsgClientCreateFriendsGroup',
  full_name='CMsgClientCreateFriendsGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CMsgClientCreateFriendsGroup.steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='groupname', full_name='CMsgClientCreateFriendsGroup.groupname', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7801,
  serialized_end=7867,
)


_CMSGCLIENTCREATEFRIENDSGROUPRESPONSE = _descriptor.Descriptor(
  name='CMsgClientCreateFriendsGroupResponse',
  full_name='CMsgClientCreateFriendsGroupResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientCreateFriendsGroupResponse.eresult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='groupid', full_name='CMsgClientCreateFriendsGroupResponse.groupid', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7869,
  serialized_end=7941,
)


_CMSGCLIENTDELETEFRIENDSGROUP = _descriptor.Descriptor(
  name='CMsgClientDeleteFriendsGroup',
  full_name='CMsgClientDeleteFriendsGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CMsgClientDeleteFriendsGroup.steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='groupid', full_name='CMsgClientDeleteFriendsGroup.groupid', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7943,
  serialized_end=8007,
)


_CMSGCLIENTDELETEFRIENDSGROUPRESPONSE = _descriptor.Descriptor(
  name='CMsgClientDeleteFriendsGroupResponse',
  full_name='CMsgClientDeleteFriendsGroupResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientDeleteFriendsGroupResponse.eresult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8009,
  serialized_end=8064,
)


_CMSGCLIENTRENAMEFRIENDSGROUP = _descriptor.Descriptor(
  name='CMsgClientRenameFriendsGroup',
  full_name='CMsgClientRenameFriendsGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='groupid', full_name='CMsgClientRenameFriendsGroup.groupid', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='groupname', full_name='CMsgClientRenameFriendsGroup.groupname', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8066,
  serialized_end=8132,
)


_CMSGCLIENTRENAMEFRIENDSGROUPRESPONSE = _descriptor.Descriptor(
  name='CMsgClientRenameFriendsGroupResponse',
  full_name='CMsgClientRenameFriendsGroupResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientRenameFriendsGroupResponse.eresult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8134,
  serialized_end=8189,
)


_CMSGCLIENTADDFRIENDTOGROUP = _descriptor.Descriptor(
  name='CMsgClientAddFriendToGroup',
  full_name='CMsgClientAddFriendToGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='groupid', full_name='CMsgClientAddFriendToGroup.groupid', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='steamiduser', full_name='CMsgClientAddFriendToGroup.steamiduser', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8191,
  serialized_end=8257,
)


_CMSGCLIENTADDFRIENDTOGROUPRESPONSE = _descriptor.Descriptor(
  name='CMsgClientAddFriendToGroupResponse',
  full_name='CMsgClientAddFriendToGroupResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientAddFriendToGroupResponse.eresult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8259,
  serialized_end=8312,
)


_CMSGCLIENTREMOVEFRIENDFROMGROUP = _descriptor.Descriptor(
  name='CMsgClientRemoveFriendFromGroup',
  full_name='CMsgClientRemoveFriendFromGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='groupid', full_name='CMsgClientRemoveFriendFromGroup.groupid', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='steamiduser', full_name='CMsgClientRemoveFriendFromGroup.steamiduser', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8314,
  serialized_end=8385,
)


_CMSGCLIENTREMOVEFRIENDFROMGROUPRESPONSE = _descriptor.Descriptor(
  name='CMsgClientRemoveFriendFromGroupResponse',
  full_name='CMsgClientRemoveFriendFromGroupResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientRemoveFriendFromGroupResponse.eresult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8387,
  serialized_end=8445,
)


_CMSGCLIENTREGISTERKEY = _descriptor.Descriptor(
  name='CMsgClientRegisterKey',
  full_name='CMsgClientRegisterKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='CMsgClientRegisterKey.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8447,
  serialized_end=8483,
)


_CMSGCLIENTPURCHASERESPONSE = _descriptor.Descriptor(
  name='CMsgClientPurchaseResponse',
  full_name='CMsgClientPurchaseResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientPurchaseResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='purchase_result_details', full_name='CMsgClientPurchaseResponse.purchase_result_details', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='purchase_receipt_info', full_name='CMsgClientPurchaseResponse.purchase_receipt_info', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8485,
  serialized_end=8597,
)


_CMSGCLIENTACTIVATEOEMLICENSE = _descriptor.Descriptor(
  name='CMsgClientActivateOEMLicense',
  full_name='CMsgClientActivateOEMLicense',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bios_manufacturer', full_name='CMsgClientActivateOEMLicense.bios_manufacturer', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bios_serialnumber', full_name='CMsgClientActivateOEMLicense.bios_serialnumber', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='license_file', full_name='CMsgClientActivateOEMLicense.license_file', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mainboard_manufacturer', full_name='CMsgClientActivateOEMLicense.mainboard_manufacturer', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mainboard_product', full_name='CMsgClientActivateOEMLicense.mainboard_product', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mainboard_serialnumber', full_name='CMsgClientActivateOEMLicense.mainboard_serialnumber', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8600,
  serialized_end=8797,
)


_CMSGCLIENTREGISTEROEMMACHINE = _descriptor.Descriptor(
  name='CMsgClientRegisterOEMMachine',
  full_name='CMsgClientRegisterOEMMachine',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='oem_register_file', full_name='CMsgClientRegisterOEMMachine.oem_register_file', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8799,
  serialized_end=8856,
)


_CMSGCLIENTREGISTEROEMMACHINERESPONSE = _descriptor.Descriptor(
  name='CMsgClientRegisterOEMMachineResponse',
  full_name='CMsgClientRegisterOEMMachineResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientRegisterOEMMachineResponse.eresult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8858,
  serialized_end=8913,
)


_CMSGCLIENTPURCHASEWITHMACHINEID = _descriptor.Descriptor(
  name='CMsgClientPurchaseWithMachineID',
  full_name='CMsgClientPurchaseWithMachineID',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='package_id', full_name='CMsgClientPurchaseWithMachineID.package_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='machine_info', full_name='CMsgClientPurchaseWithMachineID.machine_info', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8915,
  serialized_end=8990,
)


_CMSGTRADING_INITIATETRADEREQUEST = _descriptor.Descriptor(
  name='CMsgTrading_InitiateTradeRequest',
  full_name='CMsgTrading_InitiateTradeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='trade_request_id', full_name='CMsgTrading_InitiateTradeRequest.trade_request_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='other_steamid', full_name='CMsgTrading_InitiateTradeRequest.other_steamid', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='other_name', full_name='CMsgTrading_InitiateTradeRequest.other_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8992,
  serialized_end=9095,
)


_CMSGTRADING_INITIATETRADERESPONSE = _descriptor.Descriptor(
  name='CMsgTrading_InitiateTradeResponse',
  full_name='CMsgTrading_InitiateTradeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='response', full_name='CMsgTrading_InitiateTradeResponse.response', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='trade_request_id', full_name='CMsgTrading_InitiateTradeResponse.trade_request_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='other_steamid', full_name='CMsgTrading_InitiateTradeResponse.other_steamid', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='steamguard_required_days', full_name='CMsgTrading_InitiateTradeResponse.steamguard_required_days', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='new_device_cooldown_days', full_name='CMsgTrading_InitiateTradeResponse.new_device_cooldown_days', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='default_password_reset_probation_days', full_name='CMsgTrading_InitiateTradeResponse.default_password_reset_probation_days', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='password_reset_probation_days', full_name='CMsgTrading_InitiateTradeResponse.password_reset_probation_days', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='default_email_change_probation_days', full_name='CMsgTrading_InitiateTradeResponse.default_email_change_probation_days', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='email_change_probation_days', full_name='CMsgTrading_InitiateTradeResponse.email_change_probation_days', index=8,
      number=9, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9098,
  serialized_end=9436,
)


_CMSGTRADING_CANCELTRADEREQUEST = _descriptor.Descriptor(
  name='CMsgTrading_CancelTradeRequest',
  full_name='CMsgTrading_CancelTradeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='other_steamid', full_name='CMsgTrading_CancelTradeRequest.other_steamid', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9438,
  serialized_end=9493,
)


_CMSGTRADING_STARTSESSION = _descriptor.Descriptor(
  name='CMsgTrading_StartSession',
  full_name='CMsgTrading_StartSession',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='other_steamid', full_name='CMsgTrading_StartSession.other_steamid', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9495,
  serialized_end=9544,
)


_CMSGCLIENTEMAILCHANGE = _descriptor.Descriptor(
  name='CMsgClientEmailChange',
  full_name='CMsgClientEmailChange',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='password', full_name='CMsgClientEmailChange.password', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='email', full_name='CMsgClientEmailChange.email', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='code', full_name='CMsgClientEmailChange.code', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='final', full_name='CMsgClientEmailChange.final', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='newmethod', full_name='CMsgClientEmailChange.newmethod', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='twofactor_code', full_name='CMsgClientEmailChange.twofactor_code', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sms_code', full_name='CMsgClientEmailChange.sms_code', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='client_supports_sms', full_name='CMsgClientEmailChange.client_supports_sms', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9547,
  serialized_end=9722,
)


_CMSGCLIENTEMAILCHANGERESPONSE = _descriptor.Descriptor(
  name='CMsgClientEmailChangeResponse',
  full_name='CMsgClientEmailChangeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientEmailChangeResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='requires_sms_code', full_name='CMsgClientEmailChangeResponse.requires_sms_code', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9724,
  serialized_end=9802,
)


_CMSGCLIENTGETCDNAUTHTOKEN = _descriptor.Descriptor(
  name='CMsgClientGetCDNAuthToken',
  full_name='CMsgClientGetCDNAuthToken',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='depot_id', full_name='CMsgClientGetCDNAuthToken.depot_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='host_name', full_name='CMsgClientGetCDNAuthToken.host_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientGetCDNAuthToken.app_id', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9804,
  serialized_end=9884,
)


_CMSGCLIENTGETDEPOTDECRYPTIONKEY = _descriptor.Descriptor(
  name='CMsgClientGetDepotDecryptionKey',
  full_name='CMsgClientGetDepotDecryptionKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='depot_id', full_name='CMsgClientGetDepotDecryptionKey.depot_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientGetDepotDecryptionKey.app_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9886,
  serialized_end=9953,
)


_CMSGCLIENTGETDEPOTDECRYPTIONKEYRESPONSE = _descriptor.Descriptor(
  name='CMsgClientGetDepotDecryptionKeyResponse',
  full_name='CMsgClientGetDepotDecryptionKeyResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientGetDepotDecryptionKeyResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='depot_id', full_name='CMsgClientGetDepotDecryptionKeyResponse.depot_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='depot_encryption_key', full_name='CMsgClientGetDepotDecryptionKeyResponse.depot_encryption_key', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9955,
  serialized_end=10064,
)


_CMSGCLIENTCHECKAPPBETAPASSWORD = _descriptor.Descriptor(
  name='CMsgClientCheckAppBetaPassword',
  full_name='CMsgClientCheckAppBetaPassword',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientCheckAppBetaPassword.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='betapassword', full_name='CMsgClientCheckAppBetaPassword.betapassword', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10066,
  serialized_end=10136,
)


_CMSGCLIENTCHECKAPPBETAPASSWORDRESPONSE_BETAPASSWORD = _descriptor.Descriptor(
  name='BetaPassword',
  full_name='CMsgClientCheckAppBetaPasswordResponse.BetaPassword',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='betaname', full_name='CMsgClientCheckAppBetaPasswordResponse.BetaPassword.betaname', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='betapassword', full_name='CMsgClientCheckAppBetaPasswordResponse.BetaPassword.betapassword', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10278,
  serialized_end=10332,
)

_CMSGCLIENTCHECKAPPBETAPASSWORDRESPONSE = _descriptor.Descriptor(
  name='CMsgClientCheckAppBetaPasswordResponse',
  full_name='CMsgClientCheckAppBetaPasswordResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientCheckAppBetaPasswordResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='betapasswords', full_name='CMsgClientCheckAppBetaPasswordResponse.betapasswords', index=1,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTCHECKAPPBETAPASSWORDRESPONSE_BETAPASSWORD, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10139,
  serialized_end=10332,
)


_CMSGCLIENTUPDATEAPPJOBREPORT = _descriptor.Descriptor(
  name='CMsgClientUpdateAppJobReport',
  full_name='CMsgClientUpdateAppJobReport',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientUpdateAppJobReport.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='depot_ids', full_name='CMsgClientUpdateAppJobReport.depot_ids', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='app_state', full_name='CMsgClientUpdateAppJobReport.app_state', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='job_app_error', full_name='CMsgClientUpdateAppJobReport.job_app_error', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='error_details', full_name='CMsgClientUpdateAppJobReport.error_details', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='job_duration', full_name='CMsgClientUpdateAppJobReport.job_duration', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='files_validation_failed', full_name='CMsgClientUpdateAppJobReport.files_validation_failed', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='job_bytes_downloaded', full_name='CMsgClientUpdateAppJobReport.job_bytes_downloaded', index=7,
      number=8, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='job_bytes_staged', full_name='CMsgClientUpdateAppJobReport.job_bytes_staged', index=8,
      number=9, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bytes_comitted', full_name='CMsgClientUpdateAppJobReport.bytes_comitted', index=9,
      number=10, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_app_state', full_name='CMsgClientUpdateAppJobReport.start_app_state', index=10,
      number=11, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stats_machine_id', full_name='CMsgClientUpdateAppJobReport.stats_machine_id', index=11,
      number=12, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='branch_name', full_name='CMsgClientUpdateAppJobReport.branch_name', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_bytes_downloaded', full_name='CMsgClientUpdateAppJobReport.total_bytes_downloaded', index=13,
      number=14, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_bytes_staged', full_name='CMsgClientUpdateAppJobReport.total_bytes_staged', index=14,
      number=15, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_bytes_restored', full_name='CMsgClientUpdateAppJobReport.total_bytes_restored', index=15,
      number=16, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_borrowed', full_name='CMsgClientUpdateAppJobReport.is_borrowed', index=16,
      number=17, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_free_weekend', full_name='CMsgClientUpdateAppJobReport.is_free_weekend', index=17,
      number=18, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10335,
  serialized_end=10808,
)


_CMSGCLIENTDPCONTENTSTATSREPORT = _descriptor.Descriptor(
  name='CMsgClientDPContentStatsReport',
  full_name='CMsgClientDPContentStatsReport',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stats_machine_id', full_name='CMsgClientDPContentStatsReport.stats_machine_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='country_code', full_name='CMsgClientDPContentStatsReport.country_code', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='os_type', full_name='CMsgClientDPContentStatsReport.os_type', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='language', full_name='CMsgClientDPContentStatsReport.language', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='num_install_folders', full_name='CMsgClientDPContentStatsReport.num_install_folders', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='num_installed_games', full_name='CMsgClientDPContentStatsReport.num_installed_games', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='size_installed_games', full_name='CMsgClientDPContentStatsReport.size_installed_games', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10811,
  serialized_end=11014,
)


_CMSGCLIENTGETCDNAUTHTOKENRESPONSE = _descriptor.Descriptor(
  name='CMsgClientGetCDNAuthTokenResponse',
  full_name='CMsgClientGetCDNAuthTokenResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientGetCDNAuthTokenResponse.eresult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='token', full_name='CMsgClientGetCDNAuthTokenResponse.token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='expiration_time', full_name='CMsgClientGetCDNAuthTokenResponse.expiration_time', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11016,
  serialized_end=11111,
)


_CMSGDOWNLOADRATESTATISTICS_STATSINFO = _descriptor.Descriptor(
  name='StatsInfo',
  full_name='CMsgDownloadRateStatistics.StatsInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='source_type', full_name='CMsgDownloadRateStatistics.StatsInfo.source_type', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='source_id', full_name='CMsgDownloadRateStatistics.StatsInfo.source_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='seconds', full_name='CMsgDownloadRateStatistics.StatsInfo.seconds', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bytes', full_name='CMsgDownloadRateStatistics.StatsInfo.bytes', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11215,
  serialized_end=11298,
)

_CMSGDOWNLOADRATESTATISTICS = _descriptor.Descriptor(
  name='CMsgDownloadRateStatistics',
  full_name='CMsgDownloadRateStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cell_id', full_name='CMsgDownloadRateStatistics.cell_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stats', full_name='CMsgDownloadRateStatistics.stats', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGDOWNLOADRATESTATISTICS_STATSINFO, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11114,
  serialized_end=11298,
)


_CMSGCLIENTREQUESTACCOUNTDATA = _descriptor.Descriptor(
  name='CMsgClientRequestAccountData',
  full_name='CMsgClientRequestAccountData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='account_or_email', full_name='CMsgClientRequestAccountData.account_or_email', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='action', full_name='CMsgClientRequestAccountData.action', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11300,
  serialized_end=11372,
)


_CMSGCLIENTREQUESTACCOUNTDATARESPONSE = _descriptor.Descriptor(
  name='CMsgClientRequestAccountDataResponse',
  full_name='CMsgClientRequestAccountDataResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='action', full_name='CMsgClientRequestAccountDataResponse.action', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientRequestAccountDataResponse.eresult', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='account_name', full_name='CMsgClientRequestAccountDataResponse.account_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ct_matches', full_name='CMsgClientRequestAccountDataResponse.ct_matches', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='account_name_suggestion1', full_name='CMsgClientRequestAccountDataResponse.account_name_suggestion1', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='account_name_suggestion2', full_name='CMsgClientRequestAccountDataResponse.account_name_suggestion2', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='account_name_suggestion3', full_name='CMsgClientRequestAccountDataResponse.account_name_suggestion3', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11375,
  serialized_end=11590,
)


_CMSGCLIENTUGSGETGLOBALSTATS = _descriptor.Descriptor(
  name='CMsgClientUGSGetGlobalStats',
  full_name='CMsgClientUGSGetGlobalStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='gameid', full_name='CMsgClientUGSGetGlobalStats.gameid', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='history_days_requested', full_name='CMsgClientUGSGetGlobalStats.history_days_requested', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time_last_requested', full_name='CMsgClientUGSGetGlobalStats.time_last_requested', index=2,
      number=3, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_day_cached', full_name='CMsgClientUGSGetGlobalStats.first_day_cached', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='days_cached', full_name='CMsgClientUGSGetGlobalStats.days_cached', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11593,
  serialized_end=11746,
)


_CMSGCLIENTUGSGETGLOBALSTATSRESPONSE_DAY_STAT = _descriptor.Descriptor(
  name='Stat',
  full_name='CMsgClientUGSGetGlobalStatsResponse.Day.Stat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stat_id', full_name='CMsgClientUGSGetGlobalStatsResponse.Day.Stat.stat_id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='data', full_name='CMsgClientUGSGetGlobalStatsResponse.Day.Stat.data', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11989,
  serialized_end=12026,
)

_CMSGCLIENTUGSGETGLOBALSTATSRESPONSE_DAY = _descriptor.Descriptor(
  name='Day',
  full_name='CMsgClientUGSGetGlobalStatsResponse.Day',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='day_id', full_name='CMsgClientUGSGetGlobalStatsResponse.Day.day_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stats', full_name='CMsgClientUGSGetGlobalStatsResponse.Day.stats', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTUGSGETGLOBALSTATSRESPONSE_DAY_STAT, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11904,
  serialized_end=12026,
)

_CMSGCLIENTUGSGETGLOBALSTATSRESPONSE = _descriptor.Descriptor(
  name='CMsgClientUGSGetGlobalStatsResponse',
  full_name='CMsgClientUGSGetGlobalStatsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientUGSGetGlobalStatsResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='CMsgClientUGSGetGlobalStatsResponse.timestamp', index=1,
      number=2, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='day_current', full_name='CMsgClientUGSGetGlobalStatsResponse.day_current', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='days', full_name='CMsgClientUGSGetGlobalStatsResponse.days', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTUGSGETGLOBALSTATSRESPONSE_DAY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11749,
  serialized_end=12026,
)


_CMSGGAMESERVERDATA_PLAYER = _descriptor.Descriptor(
  name='Player',
  full_name='CMsgGameServerData.Player',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steam_id', full_name='CMsgGameServerData.Player.steam_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12471,
  serialized_end=12497,
)

_CMSGGAMESERVERDATA = _descriptor.Descriptor(
  name='CMsgGameServerData',
  full_name='CMsgGameServerData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steam_id_gs', full_name='CMsgGameServerData.steam_id_gs', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ip', full_name='CMsgGameServerData.ip', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='query_port', full_name='CMsgGameServerData.query_port', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='game_port', full_name='CMsgGameServerData.game_port', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sourcetv_port', full_name='CMsgGameServerData.sourcetv_port', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='CMsgGameServerData.name', index=5,
      number=22, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgGameServerData.app_id', index=6,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='gamedir', full_name='CMsgGameServerData.gamedir', index=7,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='version', full_name='CMsgGameServerData.version', index=8,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='product', full_name='CMsgGameServerData.product', index=9,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='region', full_name='CMsgGameServerData.region', index=10,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='players', full_name='CMsgGameServerData.players', index=11,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_players', full_name='CMsgGameServerData.max_players', index=12,
      number=12, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bot_count', full_name='CMsgGameServerData.bot_count', index=13,
      number=13, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='password', full_name='CMsgGameServerData.password', index=14,
      number=14, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='secure', full_name='CMsgGameServerData.secure', index=15,
      number=15, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dedicated', full_name='CMsgGameServerData.dedicated', index=16,
      number=16, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='os', full_name='CMsgGameServerData.os', index=17,
      number=17, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='game_data', full_name='CMsgGameServerData.game_data', index=18,
      number=18, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='game_data_version', full_name='CMsgGameServerData.game_data_version', index=19,
      number=19, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='game_type', full_name='CMsgGameServerData.game_type', index=20,
      number=20, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='map', full_name='CMsgGameServerData.map', index=21,
      number=21, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGGAMESERVERDATA_PLAYER, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12029,
  serialized_end=12497,
)


_CMSGGAMESERVERREMOVE = _descriptor.Descriptor(
  name='CMsgGameServerRemove',
  full_name='CMsgGameServerRemove',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steam_id', full_name='CMsgGameServerRemove.steam_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ip', full_name='CMsgGameServerRemove.ip', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='query_port', full_name='CMsgGameServerRemove.query_port', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12499,
  serialized_end=12571,
)


_CMSGCLIENTGMSSERVERQUERY = _descriptor.Descriptor(
  name='CMsgClientGMSServerQuery',
  full_name='CMsgClientGMSServerQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientGMSServerQuery.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='geo_location_ip', full_name='CMsgClientGMSServerQuery.geo_location_ip', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='region_code', full_name='CMsgClientGMSServerQuery.region_code', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='filter_text', full_name='CMsgClientGMSServerQuery.filter_text', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_servers', full_name='CMsgClientGMSServerQuery.max_servers', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12574,
  serialized_end=12704,
)


_CMSGGMSCLIENTSERVERQUERYRESPONSE_SERVER = _descriptor.Descriptor(
  name='Server',
  full_name='CMsgGMSClientServerQueryResponse.Server',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='server_ip', full_name='CMsgGMSClientServerQueryResponse.Server.server_ip', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='server_port', full_name='CMsgGMSClientServerQueryResponse.Server.server_port', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='auth_players', full_name='CMsgGMSClientServerQueryResponse.Server.auth_players', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12817,
  serialized_end=12887,
)

_CMSGGMSCLIENTSERVERQUERYRESPONSE = _descriptor.Descriptor(
  name='CMsgGMSClientServerQueryResponse',
  full_name='CMsgGMSClientServerQueryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='servers', full_name='CMsgGMSClientServerQueryResponse.servers', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='error', full_name='CMsgGMSClientServerQueryResponse.error', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGGMSCLIENTSERVERQUERYRESPONSE_SERVER, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12707,
  serialized_end=12887,
)


_CMSGGAMESERVEROUTOFDATE = _descriptor.Descriptor(
  name='CMsgGameServerOutOfDate',
  full_name='CMsgGameServerOutOfDate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steam_id_gs', full_name='CMsgGameServerOutOfDate.steam_id_gs', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='reject', full_name='CMsgGameServerOutOfDate.reject', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='message', full_name='CMsgGameServerOutOfDate.message', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12889,
  serialized_end=12968,
)


_CMSGCLIENTREDEEMGUESTPASS = _descriptor.Descriptor(
  name='CMsgClientRedeemGuestPass',
  full_name='CMsgClientRedeemGuestPass',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='guest_pass_id', full_name='CMsgClientRedeemGuestPass.guest_pass_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12970,
  serialized_end=13020,
)


_CMSGCLIENTREDEEMGUESTPASSRESPONSE = _descriptor.Descriptor(
  name='CMsgClientRedeemGuestPassResponse',
  full_name='CMsgClientRedeemGuestPassResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientRedeemGuestPassResponse.eresult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='package_id', full_name='CMsgClientRedeemGuestPassResponse.package_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='must_own_appid', full_name='CMsgClientRedeemGuestPassResponse.must_own_appid', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13022,
  serialized_end=13121,
)


_CMSGCLIENTGETCLANACTIVITYCOUNTS = _descriptor.Descriptor(
  name='CMsgClientGetClanActivityCounts',
  full_name='CMsgClientGetClanActivityCounts',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid_clans', full_name='CMsgClientGetClanActivityCounts.steamid_clans', index=0,
      number=1, type=4, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13123,
  serialized_end=13179,
)


_CMSGCLIENTGETCLANACTIVITYCOUNTSRESPONSE = _descriptor.Descriptor(
  name='CMsgClientGetClanActivityCountsResponse',
  full_name='CMsgClientGetClanActivityCountsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientGetClanActivityCountsResponse.eresult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13181,
  serialized_end=13242,
)


_CMSGCLIENTOGSREPORTSTRING = _descriptor.Descriptor(
  name='CMsgClientOGSReportString',
  full_name='CMsgClientOGSReportString',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accumulated', full_name='CMsgClientOGSReportString.accumulated', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sessionid', full_name='CMsgClientOGSReportString.sessionid', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='severity', full_name='CMsgClientOGSReportString.severity', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='formatter', full_name='CMsgClientOGSReportString.formatter', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='varargs', full_name='CMsgClientOGSReportString.varargs', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13244,
  serialized_end=13365,
)


_CMSGCLIENTOGSREPORTBUG = _descriptor.Descriptor(
  name='CMsgClientOGSReportBug',
  full_name='CMsgClientOGSReportBug',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionid', full_name='CMsgClientOGSReportBug.sessionid', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bugtext', full_name='CMsgClientOGSReportBug.bugtext', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='screenshot', full_name='CMsgClientOGSReportBug.screenshot', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13367,
  serialized_end=13447,
)


_CMSGGSASSOCIATEWITHCLAN = _descriptor.Descriptor(
  name='CMsgGSAssociateWithClan',
  full_name='CMsgGSAssociateWithClan',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steam_id_clan', full_name='CMsgGSAssociateWithClan.steam_id_clan', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13449,
  serialized_end=13497,
)


_CMSGGSASSOCIATEWITHCLANRESPONSE = _descriptor.Descriptor(
  name='CMsgGSAssociateWithClanResponse',
  full_name='CMsgGSAssociateWithClanResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steam_id_clan', full_name='CMsgGSAssociateWithClanResponse.steam_id_clan', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgGSAssociateWithClanResponse.eresult', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13499,
  serialized_end=13575,
)


_CMSGGSCOMPUTENEWPLAYERCOMPATIBILITY = _descriptor.Descriptor(
  name='CMsgGSComputeNewPlayerCompatibility',
  full_name='CMsgGSComputeNewPlayerCompatibility',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steam_id_candidate', full_name='CMsgGSComputeNewPlayerCompatibility.steam_id_candidate', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13577,
  serialized_end=13642,
)


_CMSGGSCOMPUTENEWPLAYERCOMPATIBILITYRESPONSE = _descriptor.Descriptor(
  name='CMsgGSComputeNewPlayerCompatibilityResponse',
  full_name='CMsgGSComputeNewPlayerCompatibilityResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steam_id_candidate', full_name='CMsgGSComputeNewPlayerCompatibilityResponse.steam_id_candidate', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgGSComputeNewPlayerCompatibilityResponse.eresult', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_clan_member', full_name='CMsgGSComputeNewPlayerCompatibilityResponse.is_clan_member', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ct_dont_like_you', full_name='CMsgGSComputeNewPlayerCompatibilityResponse.ct_dont_like_you', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ct_you_dont_like', full_name='CMsgGSComputeNewPlayerCompatibilityResponse.ct_you_dont_like', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ct_clanmembers_dont_like_you', full_name='CMsgGSComputeNewPlayerCompatibilityResponse.ct_clanmembers_dont_like_you', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13645,
  serialized_end=13852,
)


_CMSGCLIENTSENTLOGS = _descriptor.Descriptor(
  name='CMsgClientSentLogs',
  full_name='CMsgClientSentLogs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13854,
  serialized_end=13874,
)


_CMSGGCCLIENT = _descriptor.Descriptor(
  name='CMsgGCClient',
  full_name='CMsgGCClient',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='appid', full_name='CMsgGCClient.appid', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='msgtype', full_name='CMsgGCClient.msgtype', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='payload', full_name='CMsgGCClient.payload', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CMsgGCClient.steamid', index=3,
      number=4, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='gcname', full_name='CMsgGCClient.gcname', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13876,
  serialized_end=13972,
)


_CMSGCLIENTREQUESTFREELICENSE = _descriptor.Descriptor(
  name='CMsgClientRequestFreeLicense',
  full_name='CMsgClientRequestFreeLicense',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='appids', full_name='CMsgClientRequestFreeLicense.appids', index=0,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13974,
  serialized_end=14020,
)


_CMSGCLIENTREQUESTFREELICENSERESPONSE = _descriptor.Descriptor(
  name='CMsgClientRequestFreeLicenseResponse',
  full_name='CMsgClientRequestFreeLicenseResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientRequestFreeLicenseResponse.eresult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='granted_packageids', full_name='CMsgClientRequestFreeLicenseResponse.granted_packageids', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='granted_appids', full_name='CMsgClientRequestFreeLicenseResponse.granted_appids', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14022,
  serialized_end=14132,
)


_CMSGDRMDOWNLOADREQUESTWITHCRASHDATA = _descriptor.Descriptor(
  name='CMsgDRMDownloadRequestWithCrashData',
  full_name='CMsgDRMDownloadRequestWithCrashData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='download_flags', full_name='CMsgDRMDownloadRequestWithCrashData.download_flags', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='download_types_known', full_name='CMsgDRMDownloadRequestWithCrashData.download_types_known', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='guid_drm', full_name='CMsgDRMDownloadRequestWithCrashData.guid_drm', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='guid_split', full_name='CMsgDRMDownloadRequestWithCrashData.guid_split', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='guid_merge', full_name='CMsgDRMDownloadRequestWithCrashData.guid_merge', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='module_name', full_name='CMsgDRMDownloadRequestWithCrashData.module_name', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='module_path', full_name='CMsgDRMDownloadRequestWithCrashData.module_path', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='crash_data', full_name='CMsgDRMDownloadRequestWithCrashData.crash_data', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14135,
  serialized_end=14346,
)


_CMSGDRMDOWNLOADRESPONSE = _descriptor.Descriptor(
  name='CMsgDRMDownloadResponse',
  full_name='CMsgDRMDownloadResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgDRMDownloadResponse.eresult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgDRMDownloadResponse.app_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='blob_download_type', full_name='CMsgDRMDownloadResponse.blob_download_type', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='merge_guid', full_name='CMsgDRMDownloadResponse.merge_guid', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='download_file_dfs_ip', full_name='CMsgDRMDownloadResponse.download_file_dfs_ip', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='download_file_dfs_port', full_name='CMsgDRMDownloadResponse.download_file_dfs_port', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='download_file_url', full_name='CMsgDRMDownloadResponse.download_file_url', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='module_path', full_name='CMsgDRMDownloadResponse.module_path', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14349,
  serialized_end=14568,
)


_CMSGDRMFINALRESULT = _descriptor.Descriptor(
  name='CMsgDRMFinalResult',
  full_name='CMsgDRMFinalResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eResult', full_name='CMsgDRMFinalResult.eResult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgDRMFinalResult.app_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='blob_download_type', full_name='CMsgDRMFinalResult.blob_download_type', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='error_detail', full_name='CMsgDRMFinalResult.error_detail', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='merge_guid', full_name='CMsgDRMFinalResult.merge_guid', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='download_file_dfs_ip', full_name='CMsgDRMFinalResult.download_file_dfs_ip', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='download_file_dfs_port', full_name='CMsgDRMFinalResult.download_file_dfs_port', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='download_file_url', full_name='CMsgDRMFinalResult.download_file_url', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14571,
  serialized_end=14786,
)


_CMSGCLIENTDPCHECKSPECIALSURVEY = _descriptor.Descriptor(
  name='CMsgClientDPCheckSpecialSurvey',
  full_name='CMsgClientDPCheckSpecialSurvey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='survey_id', full_name='CMsgClientDPCheckSpecialSurvey.survey_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14788,
  serialized_end=14839,
)


_CMSGCLIENTDPCHECKSPECIALSURVEYRESPONSE = _descriptor.Descriptor(
  name='CMsgClientDPCheckSpecialSurveyResponse',
  full_name='CMsgClientDPCheckSpecialSurveyResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eResult', full_name='CMsgClientDPCheckSpecialSurveyResponse.eResult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state', full_name='CMsgClientDPCheckSpecialSurveyResponse.state', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='CMsgClientDPCheckSpecialSurveyResponse.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='custom_url', full_name='CMsgClientDPCheckSpecialSurveyResponse.custom_url', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='include_software', full_name='CMsgClientDPCheckSpecialSurveyResponse.include_software', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='token', full_name='CMsgClientDPCheckSpecialSurveyResponse.token', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14842,
  serialized_end=14992,
)


_CMSGCLIENTDPSENDSPECIALSURVEYRESPONSE = _descriptor.Descriptor(
  name='CMsgClientDPSendSpecialSurveyResponse',
  full_name='CMsgClientDPSendSpecialSurveyResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='survey_id', full_name='CMsgClientDPSendSpecialSurveyResponse.survey_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='data', full_name='CMsgClientDPSendSpecialSurveyResponse.data', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14994,
  serialized_end=15066,
)


_CMSGCLIENTDPSENDSPECIALSURVEYRESPONSEREPLY = _descriptor.Descriptor(
  name='CMsgClientDPSendSpecialSurveyResponseReply',
  full_name='CMsgClientDPSendSpecialSurveyResponseReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eResult', full_name='CMsgClientDPSendSpecialSurveyResponseReply.eResult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='token', full_name='CMsgClientDPSendSpecialSurveyResponseReply.token', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15068,
  serialized_end=15147,
)


_CMSGCLIENTREQUESTFORGOTTENPASSWORDEMAIL = _descriptor.Descriptor(
  name='CMsgClientRequestForgottenPasswordEmail',
  full_name='CMsgClientRequestForgottenPasswordEmail',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='account_name', full_name='CMsgClientRequestForgottenPasswordEmail.account_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='password_tried', full_name='CMsgClientRequestForgottenPasswordEmail.password_tried', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15149,
  serialized_end=15236,
)


_CMSGCLIENTREQUESTFORGOTTENPASSWORDEMAILRESPONSE = _descriptor.Descriptor(
  name='CMsgClientRequestForgottenPasswordEmailResponse',
  full_name='CMsgClientRequestForgottenPasswordEmailResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eResult', full_name='CMsgClientRequestForgottenPasswordEmailResponse.eResult', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_secret_question', full_name='CMsgClientRequestForgottenPasswordEmailResponse.use_secret_question', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15238,
  serialized_end=15333,
)


_CMSGCLIENTITEMANNOUNCEMENTS = _descriptor.Descriptor(
  name='CMsgClientItemAnnouncements',
  full_name='CMsgClientItemAnnouncements',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='count_new_items', full_name='CMsgClientItemAnnouncements.count_new_items', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15335,
  serialized_end=15389,
)


_CMSGCLIENTREQUESTITEMANNOUNCEMENTS = _descriptor.Descriptor(
  name='CMsgClientRequestItemAnnouncements',
  full_name='CMsgClientRequestItemAnnouncements',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15391,
  serialized_end=15427,
)


_CMSGCLIENTUSERNOTIFICATIONS_NOTIFICATION = _descriptor.Descriptor(
  name='Notification',
  full_name='CMsgClientUserNotifications.Notification',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_notification_type', full_name='CMsgClientUserNotifications.Notification.user_notification_type', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='count', full_name='CMsgClientUserNotifications.Notification.count', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15527,
  serialized_end=15588,
)

_CMSGCLIENTUSERNOTIFICATIONS = _descriptor.Descriptor(
  name='CMsgClientUserNotifications',
  full_name='CMsgClientUserNotifications',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='notifications', full_name='CMsgClientUserNotifications.notifications', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTUSERNOTIFICATIONS_NOTIFICATION, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15430,
  serialized_end=15588,
)


_CMSGCLIENTCOMMENTNOTIFICATIONS = _descriptor.Descriptor(
  name='CMsgClientCommentNotifications',
  full_name='CMsgClientCommentNotifications',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='count_new_comments', full_name='CMsgClientCommentNotifications.count_new_comments', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='count_new_comments_owner', full_name='CMsgClientCommentNotifications.count_new_comments_owner', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='count_new_comments_subscriptions', full_name='CMsgClientCommentNotifications.count_new_comments_subscriptions', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15591,
  serialized_end=15727,
)


_CMSGCLIENTREQUESTCOMMENTNOTIFICATIONS = _descriptor.Descriptor(
  name='CMsgClientRequestCommentNotifications',
  full_name='CMsgClientRequestCommentNotifications',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15729,
  serialized_end=15768,
)


_CMSGCLIENTOFFLINEMESSAGENOTIFICATION = _descriptor.Descriptor(
  name='CMsgClientOfflineMessageNotification',
  full_name='CMsgClientOfflineMessageNotification',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='offline_messages', full_name='CMsgClientOfflineMessageNotification.offline_messages', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='friends_with_offline_messages', full_name='CMsgClientOfflineMessageNotification.friends_with_offline_messages', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15770,
  serialized_end=15873,
)


_CMSGCLIENTREQUESTOFFLINEMESSAGECOUNT = _descriptor.Descriptor(
  name='CMsgClientRequestOfflineMessageCount',
  full_name='CMsgClientRequestOfflineMessageCount',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15875,
  serialized_end=15913,
)


_CMSGCLIENTFSGETFRIENDMESSAGEHISTORY = _descriptor.Descriptor(
  name='CMsgClientFSGetFriendMessageHistory',
  full_name='CMsgClientFSGetFriendMessageHistory',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CMsgClientFSGetFriendMessageHistory.steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15915,
  serialized_end=15969,
)


_CMSGCLIENTFSGETFRIENDMESSAGEHISTORYRESPONSE_FRIENDMESSAGE = _descriptor.Descriptor(
  name='FriendMessage',
  full_name='CMsgClientFSGetFriendMessageHistoryResponse.FriendMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountid', full_name='CMsgClientFSGetFriendMessageHistoryResponse.FriendMessage.accountid', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='CMsgClientFSGetFriendMessageHistoryResponse.FriendMessage.timestamp', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='message', full_name='CMsgClientFSGetFriendMessageHistoryResponse.FriendMessage.message', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='unread', full_name='CMsgClientFSGetFriendMessageHistoryResponse.FriendMessage.unread', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16131,
  serialized_end=16217,
)

_CMSGCLIENTFSGETFRIENDMESSAGEHISTORYRESPONSE = _descriptor.Descriptor(
  name='CMsgClientFSGetFriendMessageHistoryResponse',
  full_name='CMsgClientFSGetFriendMessageHistoryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CMsgClientFSGetFriendMessageHistoryResponse.steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='success', full_name='CMsgClientFSGetFriendMessageHistoryResponse.success', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='messages', full_name='CMsgClientFSGetFriendMessageHistoryResponse.messages', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTFSGETFRIENDMESSAGEHISTORYRESPONSE_FRIENDMESSAGE, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15972,
  serialized_end=16217,
)


_CMSGCLIENTFSGETFRIENDMESSAGEHISTORYFOROFFLINEMESSAGES = _descriptor.Descriptor(
  name='CMsgClientFSGetFriendMessageHistoryForOfflineMessages',
  full_name='CMsgClientFSGetFriendMessageHistoryForOfflineMessages',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16219,
  serialized_end=16274,
)


_CMSGCLIENTFSGETFRIENDSSTEAMLEVELS = _descriptor.Descriptor(
  name='CMsgClientFSGetFriendsSteamLevels',
  full_name='CMsgClientFSGetFriendsSteamLevels',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountids', full_name='CMsgClientFSGetFriendsSteamLevels.accountids', index=0,
      number=1, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16276,
  serialized_end=16331,
)


_CMSGCLIENTFSGETFRIENDSSTEAMLEVELSRESPONSE_FRIEND = _descriptor.Descriptor(
  name='Friend',
  full_name='CMsgClientFSGetFriendsSteamLevelsResponse.Friend',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountid', full_name='CMsgClientFSGetFriendsSteamLevelsResponse.Friend.accountid', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='level', full_name='CMsgClientFSGetFriendsSteamLevelsResponse.Friend.level', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16447,
  serialized_end=16489,
)

_CMSGCLIENTFSGETFRIENDSSTEAMLEVELSRESPONSE = _descriptor.Descriptor(
  name='CMsgClientFSGetFriendsSteamLevelsResponse',
  full_name='CMsgClientFSGetFriendsSteamLevelsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='friends', full_name='CMsgClientFSGetFriendsSteamLevelsResponse.friends', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTFSGETFRIENDSSTEAMLEVELSRESPONSE_FRIEND, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16334,
  serialized_end=16489,
)


_CMSGCLIENTEMAILADDRINFO = _descriptor.Descriptor(
  name='CMsgClientEmailAddrInfo',
  full_name='CMsgClientEmailAddrInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='email_address', full_name='CMsgClientEmailAddrInfo.email_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='email_is_validated', full_name='CMsgClientEmailAddrInfo.email_is_validated', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='email_validation_changed', full_name='CMsgClientEmailAddrInfo.email_validation_changed', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='credential_change_requires_code', full_name='CMsgClientEmailAddrInfo.credential_change_requires_code', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='password_or_secretqa_change_requires_code', full_name='CMsgClientEmailAddrInfo.password_or_secretqa_change_requires_code', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='remind_user_about_email', full_name='CMsgClientEmailAddrInfo.remind_user_about_email', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16492,
  serialized_end=16727,
)


_CMSGCREENUMERATEPUBLISHEDFILES = _descriptor.Descriptor(
  name='CMsgCREEnumeratePublishedFiles',
  full_name='CMsgCREEnumeratePublishedFiles',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgCREEnumeratePublishedFiles.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='query_type', full_name='CMsgCREEnumeratePublishedFiles.query_type', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_index', full_name='CMsgCREEnumeratePublishedFiles.start_index', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='days', full_name='CMsgCREEnumeratePublishedFiles.days', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='count', full_name='CMsgCREEnumeratePublishedFiles.count', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tags', full_name='CMsgCREEnumeratePublishedFiles.tags', index=5,
      number=6, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_tags', full_name='CMsgCREEnumeratePublishedFiles.user_tags', index=6,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='matching_file_type', full_name='CMsgCREEnumeratePublishedFiles.matching_file_type', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=13,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16730,
  serialized_end=16913,
)


_CMSGCREENUMERATEPUBLISHEDFILESRESPONSE_PUBLISHEDFILEID = _descriptor.Descriptor(
  name='PublishedFileId',
  full_name='CMsgCREEnumeratePublishedFilesResponse.PublishedFileId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgCREEnumeratePublishedFilesResponse.PublishedFileId.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='votes_for', full_name='CMsgCREEnumeratePublishedFilesResponse.PublishedFileId.votes_for', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='votes_against', full_name='CMsgCREEnumeratePublishedFilesResponse.PublishedFileId.votes_against', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='reports', full_name='CMsgCREEnumeratePublishedFilesResponse.PublishedFileId.reports', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='score', full_name='CMsgCREEnumeratePublishedFilesResponse.PublishedFileId.score', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17083,
  serialized_end=17201,
)

_CMSGCREENUMERATEPUBLISHEDFILESRESPONSE = _descriptor.Descriptor(
  name='CMsgCREEnumeratePublishedFilesResponse',
  full_name='CMsgCREEnumeratePublishedFilesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgCREEnumeratePublishedFilesResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='published_files', full_name='CMsgCREEnumeratePublishedFilesResponse.published_files', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_results', full_name='CMsgCREEnumeratePublishedFilesResponse.total_results', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCREENUMERATEPUBLISHEDFILESRESPONSE_PUBLISHEDFILEID, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16916,
  serialized_end=17201,
)


_CMSGCREITEMVOTESUMMARY_PUBLISHEDFILEID = _descriptor.Descriptor(
  name='PublishedFileId',
  full_name='CMsgCREItemVoteSummary.PublishedFileId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgCREItemVoteSummary.PublishedFileId.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2661,
  serialized_end=2705,
)

_CMSGCREITEMVOTESUMMARY = _descriptor.Descriptor(
  name='CMsgCREItemVoteSummary',
  full_name='CMsgCREItemVoteSummary',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_ids', full_name='CMsgCREItemVoteSummary.published_file_ids', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCREITEMVOTESUMMARY_PUBLISHEDFILEID, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17204,
  serialized_end=17343,
)


_CMSGCREITEMVOTESUMMARYRESPONSE_ITEMVOTESUMMARY = _descriptor.Descriptor(
  name='ItemVoteSummary',
  full_name='CMsgCREItemVoteSummaryResponse.ItemVoteSummary',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgCREItemVoteSummaryResponse.ItemVoteSummary.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='votes_for', full_name='CMsgCREItemVoteSummaryResponse.ItemVoteSummary.votes_for', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='votes_against', full_name='CMsgCREItemVoteSummaryResponse.ItemVoteSummary.votes_against', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='reports', full_name='CMsgCREItemVoteSummaryResponse.ItemVoteSummary.reports', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='score', full_name='CMsgCREItemVoteSummaryResponse.ItemVoteSummary.score', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17478,
  serialized_end=17596,
)

_CMSGCREITEMVOTESUMMARYRESPONSE = _descriptor.Descriptor(
  name='CMsgCREItemVoteSummaryResponse',
  full_name='CMsgCREItemVoteSummaryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgCREItemVoteSummaryResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='item_vote_summaries', full_name='CMsgCREItemVoteSummaryResponse.item_vote_summaries', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCREITEMVOTESUMMARYRESPONSE_ITEMVOTESUMMARY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17346,
  serialized_end=17596,
)


_CMSGCREUPDATEUSERPUBLISHEDITEMVOTE = _descriptor.Descriptor(
  name='CMsgCREUpdateUserPublishedItemVote',
  full_name='CMsgCREUpdateUserPublishedItemVote',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgCREUpdateUserPublishedItemVote.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='vote_up', full_name='CMsgCREUpdateUserPublishedItemVote.vote_up', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17598,
  serialized_end=17678,
)


_CMSGCREUPDATEUSERPUBLISHEDITEMVOTERESPONSE = _descriptor.Descriptor(
  name='CMsgCREUpdateUserPublishedItemVoteResponse',
  full_name='CMsgCREUpdateUserPublishedItemVoteResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgCREUpdateUserPublishedItemVoteResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17680,
  serialized_end=17744,
)


_CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILS_PUBLISHEDFILEID = _descriptor.Descriptor(
  name='PublishedFileId',
  full_name='CMsgCREGetUserPublishedItemVoteDetails.PublishedFileId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgCREGetUserPublishedItemVoteDetails.PublishedFileId.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2661,
  serialized_end=2705,
)

_CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILS = _descriptor.Descriptor(
  name='CMsgCREGetUserPublishedItemVoteDetails',
  full_name='CMsgCREGetUserPublishedItemVoteDetails',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_ids', full_name='CMsgCREGetUserPublishedItemVoteDetails.published_file_ids', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILS_PUBLISHEDFILEID, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17747,
  serialized_end=17918,
)


_CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILSRESPONSE_USERITEMVOTEDETAIL = _descriptor.Descriptor(
  name='UserItemVoteDetail',
  full_name='CMsgCREGetUserPublishedItemVoteDetailsResponse.UserItemVoteDetail',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='published_file_id', full_name='CMsgCREGetUserPublishedItemVoteDetailsResponse.UserItemVoteDetail.published_file_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='vote', full_name='CMsgCREGetUserPublishedItemVoteDetailsResponse.UserItemVoteDetail.vote', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18091,
  serialized_end=18155,
)

_CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILSRESPONSE = _descriptor.Descriptor(
  name='CMsgCREGetUserPublishedItemVoteDetailsResponse',
  full_name='CMsgCREGetUserPublishedItemVoteDetailsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgCREGetUserPublishedItemVoteDetailsResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_item_vote_details', full_name='CMsgCREGetUserPublishedItemVoteDetailsResponse.user_item_vote_details', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILSRESPONSE_USERITEMVOTEDETAIL, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17921,
  serialized_end=18155,
)


_CMSGGAMESERVERPINGSAMPLE_SAMPLE = _descriptor.Descriptor(
  name='Sample',
  full_name='CMsgGameServerPingSample.Sample',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ip', full_name='CMsgGameServerPingSample.Sample.ip', index=0,
      number=1, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='avg_ping_ms', full_name='CMsgGameServerPingSample.Sample.avg_ping_ms', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stddev_ping_ms_x10', full_name='CMsgGameServerPingSample.Sample.stddev_ping_ms_x10', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18274,
  serialized_end=18343,
)

_CMSGGAMESERVERPINGSAMPLE = _descriptor.Descriptor(
  name='CMsgGameServerPingSample',
  full_name='CMsgGameServerPingSample',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='my_ip', full_name='CMsgGameServerPingSample.my_ip', index=0,
      number=1, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='gs_app_id', full_name='CMsgGameServerPingSample.gs_app_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='gs_samples', full_name='CMsgGameServerPingSample.gs_samples', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGGAMESERVERPINGSAMPLE_SAMPLE, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18158,
  serialized_end=18343,
)


_CMSGFSGETFOLLOWERCOUNT = _descriptor.Descriptor(
  name='CMsgFSGetFollowerCount',
  full_name='CMsgFSGetFollowerCount',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steam_id', full_name='CMsgFSGetFollowerCount.steam_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18345,
  serialized_end=18387,
)


_CMSGFSGETFOLLOWERCOUNTRESPONSE = _descriptor.Descriptor(
  name='CMsgFSGetFollowerCountResponse',
  full_name='CMsgFSGetFollowerCountResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgFSGetFollowerCountResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='count', full_name='CMsgFSGetFollowerCountResponse.count', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18389,
  serialized_end=18459,
)


_CMSGFSGETISFOLLOWING = _descriptor.Descriptor(
  name='CMsgFSGetIsFollowing',
  full_name='CMsgFSGetIsFollowing',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steam_id', full_name='CMsgFSGetIsFollowing.steam_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18461,
  serialized_end=18501,
)


_CMSGFSGETISFOLLOWINGRESPONSE = _descriptor.Descriptor(
  name='CMsgFSGetIsFollowingResponse',
  full_name='CMsgFSGetIsFollowingResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgFSGetIsFollowingResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_following', full_name='CMsgFSGetIsFollowingResponse.is_following', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18503,
  serialized_end=18582,
)


_CMSGFSENUMERATEFOLLOWINGLIST = _descriptor.Descriptor(
  name='CMsgFSEnumerateFollowingList',
  full_name='CMsgFSEnumerateFollowingList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='start_index', full_name='CMsgFSEnumerateFollowingList.start_index', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18584,
  serialized_end=18635,
)


_CMSGFSENUMERATEFOLLOWINGLISTRESPONSE = _descriptor.Descriptor(
  name='CMsgFSEnumerateFollowingListResponse',
  full_name='CMsgFSEnumerateFollowingListResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgFSEnumerateFollowingListResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_results', full_name='CMsgFSEnumerateFollowingListResponse.total_results', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='steam_ids', full_name='CMsgFSEnumerateFollowingListResponse.steam_ids', index=2,
      number=3, type=6, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18637,
  serialized_end=18737,
)


_CMSGDPGETNUMBEROFCURRENTPLAYERS = _descriptor.Descriptor(
  name='CMsgDPGetNumberOfCurrentPlayers',
  full_name='CMsgDPGetNumberOfCurrentPlayers',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='appid', full_name='CMsgDPGetNumberOfCurrentPlayers.appid', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18739,
  serialized_end=18787,
)


_CMSGDPGETNUMBEROFCURRENTPLAYERSRESPONSE = _descriptor.Descriptor(
  name='CMsgDPGetNumberOfCurrentPlayersResponse',
  full_name='CMsgDPGetNumberOfCurrentPlayersResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgDPGetNumberOfCurrentPlayersResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='player_count', full_name='CMsgDPGetNumberOfCurrentPlayersResponse.player_count', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18789,
  serialized_end=18872,
)


_CMSGCLIENTFRIENDUSERSTATUSPUBLISHED = _descriptor.Descriptor(
  name='CMsgClientFriendUserStatusPublished',
  full_name='CMsgClientFriendUserStatusPublished',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='friend_steamid', full_name='CMsgClientFriendUserStatusPublished.friend_steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='appid', full_name='CMsgClientFriendUserStatusPublished.appid', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='status_text', full_name='CMsgClientFriendUserStatusPublished.status_text', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18874,
  serialized_end=18971,
)


_CMSGCLIENTSERVICEMETHOD = _descriptor.Descriptor(
  name='CMsgClientServiceMethod',
  full_name='CMsgClientServiceMethod',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='method_name', full_name='CMsgClientServiceMethod.method_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='serialized_method', full_name='CMsgClientServiceMethod.serialized_method', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_notification', full_name='CMsgClientServiceMethod.is_notification', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18973,
  serialized_end=19071,
)


_CMSGCLIENTSERVICEMETHODRESPONSE = _descriptor.Descriptor(
  name='CMsgClientServiceMethodResponse',
  full_name='CMsgClientServiceMethodResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='method_name', full_name='CMsgClientServiceMethodResponse.method_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='serialized_method_response', full_name='CMsgClientServiceMethodResponse.serialized_method_response', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19073,
  serialized_end=19163,
)


_CMSGCLIENTUIMODE = _descriptor.Descriptor(
  name='CMsgClientUIMode',
  full_name='CMsgClientUIMode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uimode', full_name='CMsgClientUIMode.uimode', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19165,
  serialized_end=19199,
)


_CMSGCLIENTVANITYURLCHANGEDNOTIFICATION = _descriptor.Descriptor(
  name='CMsgClientVanityURLChangedNotification',
  full_name='CMsgClientVanityURLChangedNotification',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='vanity_url', full_name='CMsgClientVanityURLChangedNotification.vanity_url', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19201,
  serialized_end=19261,
)


_CMSGCLIENTAUTHORIZELOCALDEVICEREQUEST = _descriptor.Descriptor(
  name='CMsgClientAuthorizeLocalDeviceRequest',
  full_name='CMsgClientAuthorizeLocalDeviceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='device_description', full_name='CMsgClientAuthorizeLocalDeviceRequest.device_description', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='owner_account_id', full_name='CMsgClientAuthorizeLocalDeviceRequest.owner_account_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='local_device_token', full_name='CMsgClientAuthorizeLocalDeviceRequest.local_device_token', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19263,
  serialized_end=19384,
)


_CMSGCLIENTAUTHORIZELOCALDEVICE = _descriptor.Descriptor(
  name='CMsgClientAuthorizeLocalDevice',
  full_name='CMsgClientAuthorizeLocalDevice',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientAuthorizeLocalDevice.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='owner_account_id', full_name='CMsgClientAuthorizeLocalDevice.owner_account_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='authed_device_token', full_name='CMsgClientAuthorizeLocalDevice.authed_device_token', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19386,
  serialized_end=19493,
)


_CMSGCLIENTAUTHORIZELOCALDEVICENOTIFICATION = _descriptor.Descriptor(
  name='CMsgClientAuthorizeLocalDeviceNotification',
  full_name='CMsgClientAuthorizeLocalDeviceNotification',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientAuthorizeLocalDeviceNotification.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='owner_account_id', full_name='CMsgClientAuthorizeLocalDeviceNotification.owner_account_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='local_device_token', full_name='CMsgClientAuthorizeLocalDeviceNotification.local_device_token', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19495,
  serialized_end=19613,
)


_CMSGCLIENTDEAUTHORIZEDEVICEREQUEST = _descriptor.Descriptor(
  name='CMsgClientDeauthorizeDeviceRequest',
  full_name='CMsgClientDeauthorizeDeviceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deauthorization_account_id', full_name='CMsgClientDeauthorizeDeviceRequest.deauthorization_account_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='deauthorization_device_token', full_name='CMsgClientDeauthorizeDeviceRequest.deauthorization_device_token', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19615,
  serialized_end=19725,
)


_CMSGCLIENTDEAUTHORIZEDEVICE = _descriptor.Descriptor(
  name='CMsgClientDeauthorizeDevice',
  full_name='CMsgClientDeauthorizeDevice',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientDeauthorizeDevice.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='deauthorization_account_id', full_name='CMsgClientDeauthorizeDevice.deauthorization_account_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19727,
  serialized_end=19812,
)


_CMSGCLIENTUSELOCALDEVICEAUTHORIZATIONS_DEVICETOKEN = _descriptor.Descriptor(
  name='DeviceToken',
  full_name='CMsgClientUseLocalDeviceAuthorizations.DeviceToken',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='owner_account_id', full_name='CMsgClientUseLocalDeviceAuthorizations.DeviceToken.owner_account_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='token_id', full_name='CMsgClientUseLocalDeviceAuthorizations.DeviceToken.token_id', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19967,
  serialized_end=20024,
)

_CMSGCLIENTUSELOCALDEVICEAUTHORIZATIONS = _descriptor.Descriptor(
  name='CMsgClientUseLocalDeviceAuthorizations',
  full_name='CMsgClientUseLocalDeviceAuthorizations',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='authorization_account_id', full_name='CMsgClientUseLocalDeviceAuthorizations.authorization_account_id', index=0,
      number=1, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='device_tokens', full_name='CMsgClientUseLocalDeviceAuthorizations.device_tokens', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTUSELOCALDEVICEAUTHORIZATIONS_DEVICETOKEN, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19815,
  serialized_end=20024,
)


_CMSGCLIENTGETAUTHORIZEDDEVICES = _descriptor.Descriptor(
  name='CMsgClientGetAuthorizedDevices',
  full_name='CMsgClientGetAuthorizedDevices',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20026,
  serialized_end=20058,
)


_CMSGCLIENTGETAUTHORIZEDDEVICESRESPONSE_AUTHORIZEDDEVICE = _descriptor.Descriptor(
  name='AuthorizedDevice',
  full_name='CMsgClientGetAuthorizedDevicesResponse.AuthorizedDevice',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='auth_device_token', full_name='CMsgClientGetAuthorizedDevicesResponse.AuthorizedDevice.auth_device_token', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='device_name', full_name='CMsgClientGetAuthorizedDevicesResponse.AuthorizedDevice.device_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='last_access_time', full_name='CMsgClientGetAuthorizedDevicesResponse.AuthorizedDevice.last_access_time', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='borrower_id', full_name='CMsgClientGetAuthorizedDevicesResponse.AuthorizedDevice.borrower_id', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_pending', full_name='CMsgClientGetAuthorizedDevicesResponse.AuthorizedDevice.is_pending', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='app_played', full_name='CMsgClientGetAuthorizedDevicesResponse.AuthorizedDevice.app_played', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20209,
  serialized_end=20362,
)

_CMSGCLIENTGETAUTHORIZEDDEVICESRESPONSE = _descriptor.Descriptor(
  name='CMsgClientGetAuthorizedDevicesResponse',
  full_name='CMsgClientGetAuthorizedDevicesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientGetAuthorizedDevicesResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='authorized_device', full_name='CMsgClientGetAuthorizedDevicesResponse.authorized_device', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTGETAUTHORIZEDDEVICESRESPONSE_AUTHORIZEDDEVICE, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20061,
  serialized_end=20362,
)


_CMSGCLIENTGETEMOTICONLIST = _descriptor.Descriptor(
  name='CMsgClientGetEmoticonList',
  full_name='CMsgClientGetEmoticonList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20364,
  serialized_end=20391,
)


_CMSGCLIENTEMOTICONLIST_EMOTICON = _descriptor.Descriptor(
  name='Emoticon',
  full_name='CMsgClientEmoticonList.Emoticon',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='CMsgClientEmoticonList.Emoticon.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='count', full_name='CMsgClientEmoticonList.Emoticon.count', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20472,
  serialized_end=20511,
)

_CMSGCLIENTEMOTICONLIST = _descriptor.Descriptor(
  name='CMsgClientEmoticonList',
  full_name='CMsgClientEmoticonList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='emoticons', full_name='CMsgClientEmoticonList.emoticons', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTEMOTICONLIST_EMOTICON, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20393,
  serialized_end=20511,
)


_CMSGCLIENTSHAREDLIBRARYLOCKSTATUS_LOCKEDLIBRARY = _descriptor.Descriptor(
  name='LockedLibrary',
  full_name='CMsgClientSharedLibraryLockStatus.LockedLibrary',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='owner_id', full_name='CMsgClientSharedLibraryLockStatus.LockedLibrary.owner_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='locked_by', full_name='CMsgClientSharedLibraryLockStatus.LockedLibrary.locked_by', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20656,
  serialized_end=20708,
)

_CMSGCLIENTSHAREDLIBRARYLOCKSTATUS = _descriptor.Descriptor(
  name='CMsgClientSharedLibraryLockStatus',
  full_name='CMsgClientSharedLibraryLockStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='locked_library', full_name='CMsgClientSharedLibraryLockStatus.locked_library', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='own_library_locked_by', full_name='CMsgClientSharedLibraryLockStatus.own_library_locked_by', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTSHAREDLIBRARYLOCKSTATUS_LOCKEDLIBRARY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20514,
  serialized_end=20708,
)


_CMSGCLIENTSHAREDLIBRARYSTOPPLAYING_STOPAPP = _descriptor.Descriptor(
  name='StopApp',
  full_name='CMsgClientSharedLibraryStopPlaying.StopApp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app_id', full_name='CMsgClientSharedLibraryStopPlaying.StopApp.app_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='owner_id', full_name='CMsgClientSharedLibraryStopPlaying.StopApp.owner_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20835,
  serialized_end=20878,
)

_CMSGCLIENTSHAREDLIBRARYSTOPPLAYING = _descriptor.Descriptor(
  name='CMsgClientSharedLibraryStopPlaying',
  full_name='CMsgClientSharedLibraryStopPlaying',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='seconds_left', full_name='CMsgClientSharedLibraryStopPlaying.seconds_left', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stop_apps', full_name='CMsgClientSharedLibraryStopPlaying.stop_apps', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CMSGCLIENTSHAREDLIBRARYSTOPPLAYING_STOPAPP, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20711,
  serialized_end=20878,
)


_CMSGCLIENTSERVICECALL = _descriptor.Descriptor(
  name='CMsgClientServiceCall',
  full_name='CMsgClientServiceCall',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sysid_routing', full_name='CMsgClientServiceCall.sysid_routing', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='call_handle', full_name='CMsgClientServiceCall.call_handle', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='module_crc', full_name='CMsgClientServiceCall.module_crc', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='module_hash', full_name='CMsgClientServiceCall.module_hash', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='function_id', full_name='CMsgClientServiceCall.function_id', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cub_output_max', full_name='CMsgClientServiceCall.cub_output_max', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='flags', full_name='CMsgClientServiceCall.flags', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='callparameter', full_name='CMsgClientServiceCall.callparameter', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ping_only', full_name='CMsgClientServiceCall.ping_only', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_outstanding_calls', full_name='CMsgClientServiceCall.max_outstanding_calls', index=9,
      number=10, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20881,
  serialized_end=21122,
)


_CMSGCLIENTSERVICEMODULE = _descriptor.Descriptor(
  name='CMsgClientServiceModule',
  full_name='CMsgClientServiceModule',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='module_crc', full_name='CMsgClientServiceModule.module_crc', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='module_hash', full_name='CMsgClientServiceModule.module_hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='module_content', full_name='CMsgClientServiceModule.module_content', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21124,
  serialized_end=21214,
)


_CMSGCLIENTSERVICECALLRESPONSE = _descriptor.Descriptor(
  name='CMsgClientServiceCallResponse',
  full_name='CMsgClientServiceCallResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sysid_routing', full_name='CMsgClientServiceCallResponse.sysid_routing', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='call_handle', full_name='CMsgClientServiceCallResponse.call_handle', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='module_crc', full_name='CMsgClientServiceCallResponse.module_crc', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='module_hash', full_name='CMsgClientServiceCallResponse.module_hash', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ecallresult', full_name='CMsgClientServiceCallResponse.ecallresult', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='result_content', full_name='CMsgClientServiceCallResponse.result_content', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='os_version_info', full_name='CMsgClientServiceCallResponse.os_version_info', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='system_info', full_name='CMsgClientServiceCallResponse.system_info', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='load_address', full_name='CMsgClientServiceCallResponse.load_address', index=8,
      number=9, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='exception_record', full_name='CMsgClientServiceCallResponse.exception_record', index=9,
      number=10, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='portable_os_version_info', full_name='CMsgClientServiceCallResponse.portable_os_version_info', index=10,
      number=11, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='portable_system_info', full_name='CMsgClientServiceCallResponse.portable_system_info', index=11,
      number=12, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='was_converted', full_name='CMsgClientServiceCallResponse.was_converted', index=12,
      number=13, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='internal_result', full_name='CMsgClientServiceCallResponse.internal_result', index=13,
      number=14, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='current_count', full_name='CMsgClientServiceCallResponse.current_count', index=14,
      number=15, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='last_call_handle', full_name='CMsgClientServiceCallResponse.last_call_handle', index=15,
      number=16, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='last_call_module_crc', full_name='CMsgClientServiceCallResponse.last_call_module_crc', index=16,
      number=17, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='last_call_sysid_routing', full_name='CMsgClientServiceCallResponse.last_call_sysid_routing', index=17,
      number=18, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='last_ecallresult', full_name='CMsgClientServiceCallResponse.last_ecallresult', index=18,
      number=19, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='last_callissue_delta', full_name='CMsgClientServiceCallResponse.last_callissue_delta', index=19,
      number=20, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='last_callcomplete_delta', full_name='CMsgClientServiceCallResponse.last_callcomplete_delta', index=20,
      number=21, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21217,
  serialized_end=21785,
)


_CMSGAMUNLOCKSTREAMING = _descriptor.Descriptor(
  name='CMsgAMUnlockStreaming',
  full_name='CMsgAMUnlockStreaming',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21787,
  serialized_end=21810,
)


_CMSGAMUNLOCKSTREAMINGRESPONSE = _descriptor.Descriptor(
  name='CMsgAMUnlockStreamingResponse',
  full_name='CMsgAMUnlockStreamingResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgAMUnlockStreamingResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='encryption_key', full_name='CMsgAMUnlockStreamingResponse.encryption_key', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21812,
  serialized_end=21887,
)


_CMSGCLIENTPLAYINGSESSIONSTATE = _descriptor.Descriptor(
  name='CMsgClientPlayingSessionState',
  full_name='CMsgClientPlayingSessionState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='playing_blocked', full_name='CMsgClientPlayingSessionState.playing_blocked', index=0,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='playing_app', full_name='CMsgClientPlayingSessionState.playing_app', index=1,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21889,
  serialized_end=21966,
)


_CMSGCLIENTKICKPLAYINGSESSION = _descriptor.Descriptor(
  name='CMsgClientKickPlayingSession',
  full_name='CMsgClientKickPlayingSession',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='only_stop_game', full_name='CMsgClientKickPlayingSession.only_stop_game', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21968,
  serialized_end=22022,
)


_CMSGCLIENTCREATEACCOUNT = _descriptor.Descriptor(
  name='CMsgClientCreateAccount',
  full_name='CMsgClientCreateAccount',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='account_name', full_name='CMsgClientCreateAccount.account_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='password', full_name='CMsgClientCreateAccount.password', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='email', full_name='CMsgClientCreateAccount.email', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='launcher', full_name='CMsgClientCreateAccount.launcher', index=3,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='language', full_name='CMsgClientCreateAccount.language', index=4,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22024,
  serialized_end=22140,
)


_CMSGCLIENTCREATEACCOUNTRESPONSE = _descriptor.Descriptor(
  name='CMsgClientCreateAccountResponse',
  full_name='CMsgClientCreateAccountResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientCreateAccountResponse.eresult', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CMsgClientCreateAccountResponse.steamid', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22142,
  serialized_end=22212,
)


_CMSGCLIENTVOICECALLPREAUTHORIZE = _descriptor.Descriptor(
  name='CMsgClientVoiceCallPreAuthorize',
  full_name='CMsgClientVoiceCallPreAuthorize',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='caller_steamid', full_name='CMsgClientVoiceCallPreAuthorize.caller_steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='receiver_steamid', full_name='CMsgClientVoiceCallPreAuthorize.receiver_steamid', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='caller_id', full_name='CMsgClientVoiceCallPreAuthorize.caller_id', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hangup', full_name='CMsgClientVoiceCallPreAuthorize.hangup', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22214,
  serialized_end=22332,
)


_CMSGCLIENTVOICECALLPREAUTHORIZERESPONSE = _descriptor.Descriptor(
  name='CMsgClientVoiceCallPreAuthorizeResponse',
  full_name='CMsgClientVoiceCallPreAuthorizeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='caller_steamid', full_name='CMsgClientVoiceCallPreAuthorizeResponse.caller_steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='receiver_steamid', full_name='CMsgClientVoiceCallPreAuthorizeResponse.receiver_steamid', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CMsgClientVoiceCallPreAuthorizeResponse.eresult', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='caller_id', full_name='CMsgClientVoiceCallPreAuthorizeResponse.caller_id', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22335,
  serialized_end=22465,
)

_CMSGCLIENTUCMADDSCREENSHOT_TAG.containing_type = _CMSGCLIENTUCMADDSCREENSHOT
_CMSGCLIENTUCMADDSCREENSHOT.fields_by_name['tag'].message_type = _CMSGCLIENTUCMADDSCREENSHOT_TAG
_CMSGCLIENTUCMUPDATEPUBLISHEDFILE_KEYVALUETAG.containing_type = _CMSGCLIENTUCMUPDATEPUBLISHEDFILE
_CMSGCLIENTUCMUPDATEPUBLISHEDFILE_ADDITIONALPREVIEW.containing_type = _CMSGCLIENTUCMUPDATEPUBLISHEDFILE
_CMSGCLIENTUCMUPDATEPUBLISHEDFILE.fields_by_name['kvtags'].message_type = _CMSGCLIENTUCMUPDATEPUBLISHEDFILE_KEYVALUETAG
_CMSGCLIENTUCMUPDATEPUBLISHEDFILE.fields_by_name['previews'].message_type = _CMSGCLIENTUCMUPDATEPUBLISHEDFILE_ADDITIONALPREVIEW
_CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILESRESPONSE_PUBLISHEDFILEID.containing_type = _CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILESRESPONSE
_CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILESRESPONSE.fields_by_name['published_files'].message_type = _CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILESRESPONSE_PUBLISHEDFILEID
_CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESRESPONSE_PUBLISHEDFILEID.containing_type = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESRESPONSE
_CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESRESPONSE.fields_by_name['subscribed_files'].message_type = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESRESPONSE_PUBLISHEDFILEID
_CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATESRESPONSE_PUBLISHEDFILEID.containing_type = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATESRESPONSE
_CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATESRESPONSE.fields_by_name['subscribed_files'].message_type = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATESRESPONSE_PUBLISHEDFILEID
_CMSGCLIENTWORKSHOPITEMCHANGESRESPONSE_WORKSHOPITEMINFO.containing_type = _CMSGCLIENTWORKSHOPITEMCHANGESRESPONSE
_CMSGCLIENTWORKSHOPITEMCHANGESRESPONSE.fields_by_name['workshop_items'].message_type = _CMSGCLIENTWORKSHOPITEMCHANGESRESPONSE_WORKSHOPITEMINFO
_CMSGCLIENTWORKSHOPITEMINFOREQUEST_WORKSHOPITEM.containing_type = _CMSGCLIENTWORKSHOPITEMINFOREQUEST
_CMSGCLIENTWORKSHOPITEMINFOREQUEST.fields_by_name['workshop_items'].message_type = _CMSGCLIENTWORKSHOPITEMINFOREQUEST_WORKSHOPITEM
_CMSGCLIENTWORKSHOPITEMINFORESPONSE_WORKSHOPITEMINFO.containing_type = _CMSGCLIENTWORKSHOPITEMINFORESPONSE
_CMSGCLIENTWORKSHOPITEMINFORESPONSE.fields_by_name['workshop_items'].message_type = _CMSGCLIENTWORKSHOPITEMINFORESPONSE_WORKSHOPITEMINFO
_CMSGCLIENTUCMGETPUBLISHEDFILESFORUSERRESPONSE_PUBLISHEDFILEID.containing_type = _CMSGCLIENTUCMGETPUBLISHEDFILESFORUSERRESPONSE
_CMSGCLIENTUCMGETPUBLISHEDFILESFORUSERRESPONSE.fields_by_name['published_files'].message_type = _CMSGCLIENTUCMGETPUBLISHEDFILESFORUSERRESPONSE_PUBLISHEDFILEID
_CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTIONRESPONSE_PUBLISHEDFILEID.containing_type = _CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTIONRESPONSE
_CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTIONRESPONSE.fields_by_name['published_files'].message_type = _CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTIONRESPONSE_PUBLISHEDFILEID
_CMSGCLIENTRICHPRESENCEINFO_RICHPRESENCE.containing_type = _CMSGCLIENTRICHPRESENCEINFO
_CMSGCLIENTRICHPRESENCEINFO.fields_by_name['rich_presence'].message_type = _CMSGCLIENTRICHPRESENCEINFO_RICHPRESENCE
_CMSGCLIENTCHECKAPPBETAPASSWORDRESPONSE_BETAPASSWORD.containing_type = _CMSGCLIENTCHECKAPPBETAPASSWORDRESPONSE
_CMSGCLIENTCHECKAPPBETAPASSWORDRESPONSE.fields_by_name['betapasswords'].message_type = _CMSGCLIENTCHECKAPPBETAPASSWORDRESPONSE_BETAPASSWORD
_CMSGDOWNLOADRATESTATISTICS_STATSINFO.containing_type = _CMSGDOWNLOADRATESTATISTICS
_CMSGDOWNLOADRATESTATISTICS.fields_by_name['stats'].message_type = _CMSGDOWNLOADRATESTATISTICS_STATSINFO
_CMSGCLIENTUGSGETGLOBALSTATSRESPONSE_DAY_STAT.containing_type = _CMSGCLIENTUGSGETGLOBALSTATSRESPONSE_DAY
_CMSGCLIENTUGSGETGLOBALSTATSRESPONSE_DAY.fields_by_name['stats'].message_type = _CMSGCLIENTUGSGETGLOBALSTATSRESPONSE_DAY_STAT
_CMSGCLIENTUGSGETGLOBALSTATSRESPONSE_DAY.containing_type = _CMSGCLIENTUGSGETGLOBALSTATSRESPONSE
_CMSGCLIENTUGSGETGLOBALSTATSRESPONSE.fields_by_name['days'].message_type = _CMSGCLIENTUGSGETGLOBALSTATSRESPONSE_DAY
_CMSGGAMESERVERDATA_PLAYER.containing_type = _CMSGGAMESERVERDATA
_CMSGGAMESERVERDATA.fields_by_name['players'].message_type = _CMSGGAMESERVERDATA_PLAYER
_CMSGGMSCLIENTSERVERQUERYRESPONSE_SERVER.containing_type = _CMSGGMSCLIENTSERVERQUERYRESPONSE
_CMSGGMSCLIENTSERVERQUERYRESPONSE.fields_by_name['servers'].message_type = _CMSGGMSCLIENTSERVERQUERYRESPONSE_SERVER
_CMSGCLIENTUSERNOTIFICATIONS_NOTIFICATION.containing_type = _CMSGCLIENTUSERNOTIFICATIONS
_CMSGCLIENTUSERNOTIFICATIONS.fields_by_name['notifications'].message_type = _CMSGCLIENTUSERNOTIFICATIONS_NOTIFICATION
_CMSGCLIENTFSGETFRIENDMESSAGEHISTORYRESPONSE_FRIENDMESSAGE.containing_type = _CMSGCLIENTFSGETFRIENDMESSAGEHISTORYRESPONSE
_CMSGCLIENTFSGETFRIENDMESSAGEHISTORYRESPONSE.fields_by_name['messages'].message_type = _CMSGCLIENTFSGETFRIENDMESSAGEHISTORYRESPONSE_FRIENDMESSAGE
_CMSGCLIENTFSGETFRIENDSSTEAMLEVELSRESPONSE_FRIEND.containing_type = _CMSGCLIENTFSGETFRIENDSSTEAMLEVELSRESPONSE
_CMSGCLIENTFSGETFRIENDSSTEAMLEVELSRESPONSE.fields_by_name['friends'].message_type = _CMSGCLIENTFSGETFRIENDSSTEAMLEVELSRESPONSE_FRIEND
_CMSGCREENUMERATEPUBLISHEDFILESRESPONSE_PUBLISHEDFILEID.containing_type = _CMSGCREENUMERATEPUBLISHEDFILESRESPONSE
_CMSGCREENUMERATEPUBLISHEDFILESRESPONSE.fields_by_name['published_files'].message_type = _CMSGCREENUMERATEPUBLISHEDFILESRESPONSE_PUBLISHEDFILEID
_CMSGCREITEMVOTESUMMARY_PUBLISHEDFILEID.containing_type = _CMSGCREITEMVOTESUMMARY
_CMSGCREITEMVOTESUMMARY.fields_by_name['published_file_ids'].message_type = _CMSGCREITEMVOTESUMMARY_PUBLISHEDFILEID
_CMSGCREITEMVOTESUMMARYRESPONSE_ITEMVOTESUMMARY.containing_type = _CMSGCREITEMVOTESUMMARYRESPONSE
_CMSGCREITEMVOTESUMMARYRESPONSE.fields_by_name['item_vote_summaries'].message_type = _CMSGCREITEMVOTESUMMARYRESPONSE_ITEMVOTESUMMARY
_CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILS_PUBLISHEDFILEID.containing_type = _CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILS
_CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILS.fields_by_name['published_file_ids'].message_type = _CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILS_PUBLISHEDFILEID
_CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILSRESPONSE_USERITEMVOTEDETAIL.containing_type = _CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILSRESPONSE
_CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILSRESPONSE.fields_by_name['user_item_vote_details'].message_type = _CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILSRESPONSE_USERITEMVOTEDETAIL
_CMSGGAMESERVERPINGSAMPLE_SAMPLE.containing_type = _CMSGGAMESERVERPINGSAMPLE
_CMSGGAMESERVERPINGSAMPLE.fields_by_name['gs_samples'].message_type = _CMSGGAMESERVERPINGSAMPLE_SAMPLE
_CMSGCLIENTUSELOCALDEVICEAUTHORIZATIONS_DEVICETOKEN.containing_type = _CMSGCLIENTUSELOCALDEVICEAUTHORIZATIONS
_CMSGCLIENTUSELOCALDEVICEAUTHORIZATIONS.fields_by_name['device_tokens'].message_type = _CMSGCLIENTUSELOCALDEVICEAUTHORIZATIONS_DEVICETOKEN
_CMSGCLIENTGETAUTHORIZEDDEVICESRESPONSE_AUTHORIZEDDEVICE.containing_type = _CMSGCLIENTGETAUTHORIZEDDEVICESRESPONSE
_CMSGCLIENTGETAUTHORIZEDDEVICESRESPONSE.fields_by_name['authorized_device'].message_type = _CMSGCLIENTGETAUTHORIZEDDEVICESRESPONSE_AUTHORIZEDDEVICE
_CMSGCLIENTEMOTICONLIST_EMOTICON.containing_type = _CMSGCLIENTEMOTICONLIST
_CMSGCLIENTEMOTICONLIST.fields_by_name['emoticons'].message_type = _CMSGCLIENTEMOTICONLIST_EMOTICON
_CMSGCLIENTSHAREDLIBRARYLOCKSTATUS_LOCKEDLIBRARY.containing_type = _CMSGCLIENTSHAREDLIBRARYLOCKSTATUS
_CMSGCLIENTSHAREDLIBRARYLOCKSTATUS.fields_by_name['locked_library'].message_type = _CMSGCLIENTSHAREDLIBRARYLOCKSTATUS_LOCKEDLIBRARY
_CMSGCLIENTSHAREDLIBRARYSTOPPLAYING_STOPAPP.containing_type = _CMSGCLIENTSHAREDLIBRARYSTOPPLAYING
_CMSGCLIENTSHAREDLIBRARYSTOPPLAYING.fields_by_name['stop_apps'].message_type = _CMSGCLIENTSHAREDLIBRARYSTOPPLAYING_STOPAPP
DESCRIPTOR.message_types_by_name['CMsgClientUCMAddScreenshot'] = _CMSGCLIENTUCMADDSCREENSHOT
DESCRIPTOR.message_types_by_name['CMsgClientUCMAddScreenshotResponse'] = _CMSGCLIENTUCMADDSCREENSHOTRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUCMDeleteScreenshot'] = _CMSGCLIENTUCMDELETESCREENSHOT
DESCRIPTOR.message_types_by_name['CMsgClientUCMDeleteScreenshotResponse'] = _CMSGCLIENTUCMDELETESCREENSHOTRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUCMPublishFile'] = _CMSGCLIENTUCMPUBLISHFILE
DESCRIPTOR.message_types_by_name['CMsgClientUCMPublishFileResponse'] = _CMSGCLIENTUCMPUBLISHFILERESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUCMUpdatePublishedFile'] = _CMSGCLIENTUCMUPDATEPUBLISHEDFILE
DESCRIPTOR.message_types_by_name['CMsgClientUCMUpdatePublishedFileResponse'] = _CMSGCLIENTUCMUPDATEPUBLISHEDFILERESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUCMDeletePublishedFile'] = _CMSGCLIENTUCMDELETEPUBLISHEDFILE
DESCRIPTOR.message_types_by_name['CMsgClientUCMDeletePublishedFileResponse'] = _CMSGCLIENTUCMDELETEPUBLISHEDFILERESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUCMEnumerateUserPublishedFiles'] = _CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILES
DESCRIPTOR.message_types_by_name['CMsgClientUCMEnumerateUserPublishedFilesResponse'] = _CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILESRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUCMEnumerateUserSubscribedFiles'] = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILES
DESCRIPTOR.message_types_by_name['CMsgClientUCMEnumerateUserSubscribedFilesResponse'] = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUCMEnumerateUserSubscribedFilesWithUpdates'] = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATES
DESCRIPTOR.message_types_by_name['CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse'] = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATESRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUCMPublishedFileSubscribed'] = _CMSGCLIENTUCMPUBLISHEDFILESUBSCRIBED
DESCRIPTOR.message_types_by_name['CMsgClientUCMPublishedFileUnsubscribed'] = _CMSGCLIENTUCMPUBLISHEDFILEUNSUBSCRIBED
DESCRIPTOR.message_types_by_name['CMsgClientUCMPublishedFileDeleted'] = _CMSGCLIENTUCMPUBLISHEDFILEDELETED
DESCRIPTOR.message_types_by_name['CMsgClientUCMPublishedFileUpdated'] = _CMSGCLIENTUCMPUBLISHEDFILEUPDATED
DESCRIPTOR.message_types_by_name['CMsgClientWorkshopItemChangesRequest'] = _CMSGCLIENTWORKSHOPITEMCHANGESREQUEST
DESCRIPTOR.message_types_by_name['CMsgClientWorkshopItemChangesResponse'] = _CMSGCLIENTWORKSHOPITEMCHANGESRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientWorkshopItemInfoRequest'] = _CMSGCLIENTWORKSHOPITEMINFOREQUEST
DESCRIPTOR.message_types_by_name['CMsgClientWorkshopItemInfoResponse'] = _CMSGCLIENTWORKSHOPITEMINFORESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUCMGetPublishedFilesForUser'] = _CMSGCLIENTUCMGETPUBLISHEDFILESFORUSER
DESCRIPTOR.message_types_by_name['CMsgClientUCMGetPublishedFilesForUserResponse'] = _CMSGCLIENTUCMGETPUBLISHEDFILESFORUSERRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUCMSetUserPublishedFileAction'] = _CMSGCLIENTUCMSETUSERPUBLISHEDFILEACTION
DESCRIPTOR.message_types_by_name['CMsgClientUCMSetUserPublishedFileActionResponse'] = _CMSGCLIENTUCMSETUSERPUBLISHEDFILEACTIONRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUCMEnumeratePublishedFilesByUserAction'] = _CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTION
DESCRIPTOR.message_types_by_name['CMsgClientUCMEnumeratePublishedFilesByUserActionResponse'] = _CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTIONRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientScreenshotsChanged'] = _CMSGCLIENTSCREENSHOTSCHANGED
DESCRIPTOR.message_types_by_name['CMsgClientUpdateUserGameInfo'] = _CMSGCLIENTUPDATEUSERGAMEINFO
DESCRIPTOR.message_types_by_name['CMsgClientRichPresenceUpload'] = _CMSGCLIENTRICHPRESENCEUPLOAD
DESCRIPTOR.message_types_by_name['CMsgClientRichPresenceRequest'] = _CMSGCLIENTRICHPRESENCEREQUEST
DESCRIPTOR.message_types_by_name['CMsgClientRichPresenceInfo'] = _CMSGCLIENTRICHPRESENCEINFO
DESCRIPTOR.message_types_by_name['CMsgClientCheckFileSignature'] = _CMSGCLIENTCHECKFILESIGNATURE
DESCRIPTOR.message_types_by_name['CMsgClientCheckFileSignatureResponse'] = _CMSGCLIENTCHECKFILESIGNATURERESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientReadMachineAuth'] = _CMSGCLIENTREADMACHINEAUTH
DESCRIPTOR.message_types_by_name['CMsgClientReadMachineAuthResponse'] = _CMSGCLIENTREADMACHINEAUTHRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUpdateMachineAuth'] = _CMSGCLIENTUPDATEMACHINEAUTH
DESCRIPTOR.message_types_by_name['CMsgClientUpdateMachineAuthResponse'] = _CMSGCLIENTUPDATEMACHINEAUTHRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientRequestMachineAuth'] = _CMSGCLIENTREQUESTMACHINEAUTH
DESCRIPTOR.message_types_by_name['CMsgClientRequestMachineAuthResponse'] = _CMSGCLIENTREQUESTMACHINEAUTHRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientCreateFriendsGroup'] = _CMSGCLIENTCREATEFRIENDSGROUP
DESCRIPTOR.message_types_by_name['CMsgClientCreateFriendsGroupResponse'] = _CMSGCLIENTCREATEFRIENDSGROUPRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientDeleteFriendsGroup'] = _CMSGCLIENTDELETEFRIENDSGROUP
DESCRIPTOR.message_types_by_name['CMsgClientDeleteFriendsGroupResponse'] = _CMSGCLIENTDELETEFRIENDSGROUPRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientRenameFriendsGroup'] = _CMSGCLIENTRENAMEFRIENDSGROUP
DESCRIPTOR.message_types_by_name['CMsgClientRenameFriendsGroupResponse'] = _CMSGCLIENTRENAMEFRIENDSGROUPRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientAddFriendToGroup'] = _CMSGCLIENTADDFRIENDTOGROUP
DESCRIPTOR.message_types_by_name['CMsgClientAddFriendToGroupResponse'] = _CMSGCLIENTADDFRIENDTOGROUPRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientRemoveFriendFromGroup'] = _CMSGCLIENTREMOVEFRIENDFROMGROUP
DESCRIPTOR.message_types_by_name['CMsgClientRemoveFriendFromGroupResponse'] = _CMSGCLIENTREMOVEFRIENDFROMGROUPRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientRegisterKey'] = _CMSGCLIENTREGISTERKEY
DESCRIPTOR.message_types_by_name['CMsgClientPurchaseResponse'] = _CMSGCLIENTPURCHASERESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientActivateOEMLicense'] = _CMSGCLIENTACTIVATEOEMLICENSE
DESCRIPTOR.message_types_by_name['CMsgClientRegisterOEMMachine'] = _CMSGCLIENTREGISTEROEMMACHINE
DESCRIPTOR.message_types_by_name['CMsgClientRegisterOEMMachineResponse'] = _CMSGCLIENTREGISTEROEMMACHINERESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientPurchaseWithMachineID'] = _CMSGCLIENTPURCHASEWITHMACHINEID
DESCRIPTOR.message_types_by_name['CMsgTrading_InitiateTradeRequest'] = _CMSGTRADING_INITIATETRADEREQUEST
DESCRIPTOR.message_types_by_name['CMsgTrading_InitiateTradeResponse'] = _CMSGTRADING_INITIATETRADERESPONSE
DESCRIPTOR.message_types_by_name['CMsgTrading_CancelTradeRequest'] = _CMSGTRADING_CANCELTRADEREQUEST
DESCRIPTOR.message_types_by_name['CMsgTrading_StartSession'] = _CMSGTRADING_STARTSESSION
DESCRIPTOR.message_types_by_name['CMsgClientEmailChange'] = _CMSGCLIENTEMAILCHANGE
DESCRIPTOR.message_types_by_name['CMsgClientEmailChangeResponse'] = _CMSGCLIENTEMAILCHANGERESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientGetCDNAuthToken'] = _CMSGCLIENTGETCDNAUTHTOKEN
DESCRIPTOR.message_types_by_name['CMsgClientGetDepotDecryptionKey'] = _CMSGCLIENTGETDEPOTDECRYPTIONKEY
DESCRIPTOR.message_types_by_name['CMsgClientGetDepotDecryptionKeyResponse'] = _CMSGCLIENTGETDEPOTDECRYPTIONKEYRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientCheckAppBetaPassword'] = _CMSGCLIENTCHECKAPPBETAPASSWORD
DESCRIPTOR.message_types_by_name['CMsgClientCheckAppBetaPasswordResponse'] = _CMSGCLIENTCHECKAPPBETAPASSWORDRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUpdateAppJobReport'] = _CMSGCLIENTUPDATEAPPJOBREPORT
DESCRIPTOR.message_types_by_name['CMsgClientDPContentStatsReport'] = _CMSGCLIENTDPCONTENTSTATSREPORT
DESCRIPTOR.message_types_by_name['CMsgClientGetCDNAuthTokenResponse'] = _CMSGCLIENTGETCDNAUTHTOKENRESPONSE
DESCRIPTOR.message_types_by_name['CMsgDownloadRateStatistics'] = _CMSGDOWNLOADRATESTATISTICS
DESCRIPTOR.message_types_by_name['CMsgClientRequestAccountData'] = _CMSGCLIENTREQUESTACCOUNTDATA
DESCRIPTOR.message_types_by_name['CMsgClientRequestAccountDataResponse'] = _CMSGCLIENTREQUESTACCOUNTDATARESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUGSGetGlobalStats'] = _CMSGCLIENTUGSGETGLOBALSTATS
DESCRIPTOR.message_types_by_name['CMsgClientUGSGetGlobalStatsResponse'] = _CMSGCLIENTUGSGETGLOBALSTATSRESPONSE
DESCRIPTOR.message_types_by_name['CMsgGameServerData'] = _CMSGGAMESERVERDATA
DESCRIPTOR.message_types_by_name['CMsgGameServerRemove'] = _CMSGGAMESERVERREMOVE
DESCRIPTOR.message_types_by_name['CMsgClientGMSServerQuery'] = _CMSGCLIENTGMSSERVERQUERY
DESCRIPTOR.message_types_by_name['CMsgGMSClientServerQueryResponse'] = _CMSGGMSCLIENTSERVERQUERYRESPONSE
DESCRIPTOR.message_types_by_name['CMsgGameServerOutOfDate'] = _CMSGGAMESERVEROUTOFDATE
DESCRIPTOR.message_types_by_name['CMsgClientRedeemGuestPass'] = _CMSGCLIENTREDEEMGUESTPASS
DESCRIPTOR.message_types_by_name['CMsgClientRedeemGuestPassResponse'] = _CMSGCLIENTREDEEMGUESTPASSRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientGetClanActivityCounts'] = _CMSGCLIENTGETCLANACTIVITYCOUNTS
DESCRIPTOR.message_types_by_name['CMsgClientGetClanActivityCountsResponse'] = _CMSGCLIENTGETCLANACTIVITYCOUNTSRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientOGSReportString'] = _CMSGCLIENTOGSREPORTSTRING
DESCRIPTOR.message_types_by_name['CMsgClientOGSReportBug'] = _CMSGCLIENTOGSREPORTBUG
DESCRIPTOR.message_types_by_name['CMsgGSAssociateWithClan'] = _CMSGGSASSOCIATEWITHCLAN
DESCRIPTOR.message_types_by_name['CMsgGSAssociateWithClanResponse'] = _CMSGGSASSOCIATEWITHCLANRESPONSE
DESCRIPTOR.message_types_by_name['CMsgGSComputeNewPlayerCompatibility'] = _CMSGGSCOMPUTENEWPLAYERCOMPATIBILITY
DESCRIPTOR.message_types_by_name['CMsgGSComputeNewPlayerCompatibilityResponse'] = _CMSGGSCOMPUTENEWPLAYERCOMPATIBILITYRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientSentLogs'] = _CMSGCLIENTSENTLOGS
DESCRIPTOR.message_types_by_name['CMsgGCClient'] = _CMSGGCCLIENT
DESCRIPTOR.message_types_by_name['CMsgClientRequestFreeLicense'] = _CMSGCLIENTREQUESTFREELICENSE
DESCRIPTOR.message_types_by_name['CMsgClientRequestFreeLicenseResponse'] = _CMSGCLIENTREQUESTFREELICENSERESPONSE
DESCRIPTOR.message_types_by_name['CMsgDRMDownloadRequestWithCrashData'] = _CMSGDRMDOWNLOADREQUESTWITHCRASHDATA
DESCRIPTOR.message_types_by_name['CMsgDRMDownloadResponse'] = _CMSGDRMDOWNLOADRESPONSE
DESCRIPTOR.message_types_by_name['CMsgDRMFinalResult'] = _CMSGDRMFINALRESULT
DESCRIPTOR.message_types_by_name['CMsgClientDPCheckSpecialSurvey'] = _CMSGCLIENTDPCHECKSPECIALSURVEY
DESCRIPTOR.message_types_by_name['CMsgClientDPCheckSpecialSurveyResponse'] = _CMSGCLIENTDPCHECKSPECIALSURVEYRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientDPSendSpecialSurveyResponse'] = _CMSGCLIENTDPSENDSPECIALSURVEYRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientDPSendSpecialSurveyResponseReply'] = _CMSGCLIENTDPSENDSPECIALSURVEYRESPONSEREPLY
DESCRIPTOR.message_types_by_name['CMsgClientRequestForgottenPasswordEmail'] = _CMSGCLIENTREQUESTFORGOTTENPASSWORDEMAIL
DESCRIPTOR.message_types_by_name['CMsgClientRequestForgottenPasswordEmailResponse'] = _CMSGCLIENTREQUESTFORGOTTENPASSWORDEMAILRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientItemAnnouncements'] = _CMSGCLIENTITEMANNOUNCEMENTS
DESCRIPTOR.message_types_by_name['CMsgClientRequestItemAnnouncements'] = _CMSGCLIENTREQUESTITEMANNOUNCEMENTS
DESCRIPTOR.message_types_by_name['CMsgClientUserNotifications'] = _CMSGCLIENTUSERNOTIFICATIONS
DESCRIPTOR.message_types_by_name['CMsgClientCommentNotifications'] = _CMSGCLIENTCOMMENTNOTIFICATIONS
DESCRIPTOR.message_types_by_name['CMsgClientRequestCommentNotifications'] = _CMSGCLIENTREQUESTCOMMENTNOTIFICATIONS
DESCRIPTOR.message_types_by_name['CMsgClientOfflineMessageNotification'] = _CMSGCLIENTOFFLINEMESSAGENOTIFICATION
DESCRIPTOR.message_types_by_name['CMsgClientRequestOfflineMessageCount'] = _CMSGCLIENTREQUESTOFFLINEMESSAGECOUNT
DESCRIPTOR.message_types_by_name['CMsgClientFSGetFriendMessageHistory'] = _CMSGCLIENTFSGETFRIENDMESSAGEHISTORY
DESCRIPTOR.message_types_by_name['CMsgClientFSGetFriendMessageHistoryResponse'] = _CMSGCLIENTFSGETFRIENDMESSAGEHISTORYRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientFSGetFriendMessageHistoryForOfflineMessages'] = _CMSGCLIENTFSGETFRIENDMESSAGEHISTORYFOROFFLINEMESSAGES
DESCRIPTOR.message_types_by_name['CMsgClientFSGetFriendsSteamLevels'] = _CMSGCLIENTFSGETFRIENDSSTEAMLEVELS
DESCRIPTOR.message_types_by_name['CMsgClientFSGetFriendsSteamLevelsResponse'] = _CMSGCLIENTFSGETFRIENDSSTEAMLEVELSRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientEmailAddrInfo'] = _CMSGCLIENTEMAILADDRINFO
DESCRIPTOR.message_types_by_name['CMsgCREEnumeratePublishedFiles'] = _CMSGCREENUMERATEPUBLISHEDFILES
DESCRIPTOR.message_types_by_name['CMsgCREEnumeratePublishedFilesResponse'] = _CMSGCREENUMERATEPUBLISHEDFILESRESPONSE
DESCRIPTOR.message_types_by_name['CMsgCREItemVoteSummary'] = _CMSGCREITEMVOTESUMMARY
DESCRIPTOR.message_types_by_name['CMsgCREItemVoteSummaryResponse'] = _CMSGCREITEMVOTESUMMARYRESPONSE
DESCRIPTOR.message_types_by_name['CMsgCREUpdateUserPublishedItemVote'] = _CMSGCREUPDATEUSERPUBLISHEDITEMVOTE
DESCRIPTOR.message_types_by_name['CMsgCREUpdateUserPublishedItemVoteResponse'] = _CMSGCREUPDATEUSERPUBLISHEDITEMVOTERESPONSE
DESCRIPTOR.message_types_by_name['CMsgCREGetUserPublishedItemVoteDetails'] = _CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILS
DESCRIPTOR.message_types_by_name['CMsgCREGetUserPublishedItemVoteDetailsResponse'] = _CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILSRESPONSE
DESCRIPTOR.message_types_by_name['CMsgGameServerPingSample'] = _CMSGGAMESERVERPINGSAMPLE
DESCRIPTOR.message_types_by_name['CMsgFSGetFollowerCount'] = _CMSGFSGETFOLLOWERCOUNT
DESCRIPTOR.message_types_by_name['CMsgFSGetFollowerCountResponse'] = _CMSGFSGETFOLLOWERCOUNTRESPONSE
DESCRIPTOR.message_types_by_name['CMsgFSGetIsFollowing'] = _CMSGFSGETISFOLLOWING
DESCRIPTOR.message_types_by_name['CMsgFSGetIsFollowingResponse'] = _CMSGFSGETISFOLLOWINGRESPONSE
DESCRIPTOR.message_types_by_name['CMsgFSEnumerateFollowingList'] = _CMSGFSENUMERATEFOLLOWINGLIST
DESCRIPTOR.message_types_by_name['CMsgFSEnumerateFollowingListResponse'] = _CMSGFSENUMERATEFOLLOWINGLISTRESPONSE
DESCRIPTOR.message_types_by_name['CMsgDPGetNumberOfCurrentPlayers'] = _CMSGDPGETNUMBEROFCURRENTPLAYERS
DESCRIPTOR.message_types_by_name['CMsgDPGetNumberOfCurrentPlayersResponse'] = _CMSGDPGETNUMBEROFCURRENTPLAYERSRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientFriendUserStatusPublished'] = _CMSGCLIENTFRIENDUSERSTATUSPUBLISHED
DESCRIPTOR.message_types_by_name['CMsgClientServiceMethod'] = _CMSGCLIENTSERVICEMETHOD
DESCRIPTOR.message_types_by_name['CMsgClientServiceMethodResponse'] = _CMSGCLIENTSERVICEMETHODRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientUIMode'] = _CMSGCLIENTUIMODE
DESCRIPTOR.message_types_by_name['CMsgClientVanityURLChangedNotification'] = _CMSGCLIENTVANITYURLCHANGEDNOTIFICATION
DESCRIPTOR.message_types_by_name['CMsgClientAuthorizeLocalDeviceRequest'] = _CMSGCLIENTAUTHORIZELOCALDEVICEREQUEST
DESCRIPTOR.message_types_by_name['CMsgClientAuthorizeLocalDevice'] = _CMSGCLIENTAUTHORIZELOCALDEVICE
DESCRIPTOR.message_types_by_name['CMsgClientAuthorizeLocalDeviceNotification'] = _CMSGCLIENTAUTHORIZELOCALDEVICENOTIFICATION
DESCRIPTOR.message_types_by_name['CMsgClientDeauthorizeDeviceRequest'] = _CMSGCLIENTDEAUTHORIZEDEVICEREQUEST
DESCRIPTOR.message_types_by_name['CMsgClientDeauthorizeDevice'] = _CMSGCLIENTDEAUTHORIZEDEVICE
DESCRIPTOR.message_types_by_name['CMsgClientUseLocalDeviceAuthorizations'] = _CMSGCLIENTUSELOCALDEVICEAUTHORIZATIONS
DESCRIPTOR.message_types_by_name['CMsgClientGetAuthorizedDevices'] = _CMSGCLIENTGETAUTHORIZEDDEVICES
DESCRIPTOR.message_types_by_name['CMsgClientGetAuthorizedDevicesResponse'] = _CMSGCLIENTGETAUTHORIZEDDEVICESRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientGetEmoticonList'] = _CMSGCLIENTGETEMOTICONLIST
DESCRIPTOR.message_types_by_name['CMsgClientEmoticonList'] = _CMSGCLIENTEMOTICONLIST
DESCRIPTOR.message_types_by_name['CMsgClientSharedLibraryLockStatus'] = _CMSGCLIENTSHAREDLIBRARYLOCKSTATUS
DESCRIPTOR.message_types_by_name['CMsgClientSharedLibraryStopPlaying'] = _CMSGCLIENTSHAREDLIBRARYSTOPPLAYING
DESCRIPTOR.message_types_by_name['CMsgClientServiceCall'] = _CMSGCLIENTSERVICECALL
DESCRIPTOR.message_types_by_name['CMsgClientServiceModule'] = _CMSGCLIENTSERVICEMODULE
DESCRIPTOR.message_types_by_name['CMsgClientServiceCallResponse'] = _CMSGCLIENTSERVICECALLRESPONSE
DESCRIPTOR.message_types_by_name['CMsgAMUnlockStreaming'] = _CMSGAMUNLOCKSTREAMING
DESCRIPTOR.message_types_by_name['CMsgAMUnlockStreamingResponse'] = _CMSGAMUNLOCKSTREAMINGRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientPlayingSessionState'] = _CMSGCLIENTPLAYINGSESSIONSTATE
DESCRIPTOR.message_types_by_name['CMsgClientKickPlayingSession'] = _CMSGCLIENTKICKPLAYINGSESSION
DESCRIPTOR.message_types_by_name['CMsgClientCreateAccount'] = _CMSGCLIENTCREATEACCOUNT
DESCRIPTOR.message_types_by_name['CMsgClientCreateAccountResponse'] = _CMSGCLIENTCREATEACCOUNTRESPONSE
DESCRIPTOR.message_types_by_name['CMsgClientVoiceCallPreAuthorize'] = _CMSGCLIENTVOICECALLPREAUTHORIZE
DESCRIPTOR.message_types_by_name['CMsgClientVoiceCallPreAuthorizeResponse'] = _CMSGCLIENTVOICECALLPREAUTHORIZERESPONSE

CMsgClientUCMAddScreenshot = _reflection.GeneratedProtocolMessageType('CMsgClientUCMAddScreenshot', (_message.Message,), dict(

  Tag = _reflection.GeneratedProtocolMessageType('Tag', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTUCMADDSCREENSHOT_TAG,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientUCMAddScreenshot.Tag)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTUCMADDSCREENSHOT,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMAddScreenshot)
  ))
_sym_db.RegisterMessage(CMsgClientUCMAddScreenshot)
_sym_db.RegisterMessage(CMsgClientUCMAddScreenshot.Tag)

CMsgClientUCMAddScreenshotResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUCMAddScreenshotResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMADDSCREENSHOTRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMAddScreenshotResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUCMAddScreenshotResponse)

CMsgClientUCMDeleteScreenshot = _reflection.GeneratedProtocolMessageType('CMsgClientUCMDeleteScreenshot', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMDELETESCREENSHOT,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMDeleteScreenshot)
  ))
_sym_db.RegisterMessage(CMsgClientUCMDeleteScreenshot)

CMsgClientUCMDeleteScreenshotResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUCMDeleteScreenshotResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMDELETESCREENSHOTRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMDeleteScreenshotResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUCMDeleteScreenshotResponse)

CMsgClientUCMPublishFile = _reflection.GeneratedProtocolMessageType('CMsgClientUCMPublishFile', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMPUBLISHFILE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMPublishFile)
  ))
_sym_db.RegisterMessage(CMsgClientUCMPublishFile)

CMsgClientUCMPublishFileResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUCMPublishFileResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMPUBLISHFILERESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMPublishFileResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUCMPublishFileResponse)

CMsgClientUCMUpdatePublishedFile = _reflection.GeneratedProtocolMessageType('CMsgClientUCMUpdatePublishedFile', (_message.Message,), dict(

  KeyValueTag = _reflection.GeneratedProtocolMessageType('KeyValueTag', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTUCMUPDATEPUBLISHEDFILE_KEYVALUETAG,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientUCMUpdatePublishedFile.KeyValueTag)
    ))
  ,

  AdditionalPreview = _reflection.GeneratedProtocolMessageType('AdditionalPreview', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTUCMUPDATEPUBLISHEDFILE_ADDITIONALPREVIEW,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientUCMUpdatePublishedFile.AdditionalPreview)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTUCMUPDATEPUBLISHEDFILE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMUpdatePublishedFile)
  ))
_sym_db.RegisterMessage(CMsgClientUCMUpdatePublishedFile)
_sym_db.RegisterMessage(CMsgClientUCMUpdatePublishedFile.KeyValueTag)
_sym_db.RegisterMessage(CMsgClientUCMUpdatePublishedFile.AdditionalPreview)

CMsgClientUCMUpdatePublishedFileResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUCMUpdatePublishedFileResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMUPDATEPUBLISHEDFILERESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMUpdatePublishedFileResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUCMUpdatePublishedFileResponse)

CMsgClientUCMDeletePublishedFile = _reflection.GeneratedProtocolMessageType('CMsgClientUCMDeletePublishedFile', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMDELETEPUBLISHEDFILE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMDeletePublishedFile)
  ))
_sym_db.RegisterMessage(CMsgClientUCMDeletePublishedFile)

CMsgClientUCMDeletePublishedFileResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUCMDeletePublishedFileResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMDELETEPUBLISHEDFILERESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMDeletePublishedFileResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUCMDeletePublishedFileResponse)

CMsgClientUCMEnumerateUserPublishedFiles = _reflection.GeneratedProtocolMessageType('CMsgClientUCMEnumerateUserPublishedFiles', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILES,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMEnumerateUserPublishedFiles)
  ))
_sym_db.RegisterMessage(CMsgClientUCMEnumerateUserPublishedFiles)

CMsgClientUCMEnumerateUserPublishedFilesResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUCMEnumerateUserPublishedFilesResponse', (_message.Message,), dict(

  PublishedFileId = _reflection.GeneratedProtocolMessageType('PublishedFileId', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILESRESPONSE_PUBLISHEDFILEID,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientUCMEnumerateUserPublishedFilesResponse.PublishedFileId)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTUCMENUMERATEUSERPUBLISHEDFILESRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMEnumerateUserPublishedFilesResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUCMEnumerateUserPublishedFilesResponse)
_sym_db.RegisterMessage(CMsgClientUCMEnumerateUserPublishedFilesResponse.PublishedFileId)

CMsgClientUCMEnumerateUserSubscribedFiles = _reflection.GeneratedProtocolMessageType('CMsgClientUCMEnumerateUserSubscribedFiles', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILES,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMEnumerateUserSubscribedFiles)
  ))
_sym_db.RegisterMessage(CMsgClientUCMEnumerateUserSubscribedFiles)

CMsgClientUCMEnumerateUserSubscribedFilesResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUCMEnumerateUserSubscribedFilesResponse', (_message.Message,), dict(

  PublishedFileId = _reflection.GeneratedProtocolMessageType('PublishedFileId', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESRESPONSE_PUBLISHEDFILEID,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientUCMEnumerateUserSubscribedFilesResponse.PublishedFileId)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMEnumerateUserSubscribedFilesResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUCMEnumerateUserSubscribedFilesResponse)
_sym_db.RegisterMessage(CMsgClientUCMEnumerateUserSubscribedFilesResponse.PublishedFileId)

CMsgClientUCMEnumerateUserSubscribedFilesWithUpdates = _reflection.GeneratedProtocolMessageType('CMsgClientUCMEnumerateUserSubscribedFilesWithUpdates', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATES,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMEnumerateUserSubscribedFilesWithUpdates)
  ))
_sym_db.RegisterMessage(CMsgClientUCMEnumerateUserSubscribedFilesWithUpdates)

CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse', (_message.Message,), dict(

  PublishedFileId = _reflection.GeneratedProtocolMessageType('PublishedFileId', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATESRESPONSE_PUBLISHEDFILEID,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.PublishedFileId)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTUCMENUMERATEUSERSUBSCRIBEDFILESWITHUPDATESRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse)
_sym_db.RegisterMessage(CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse.PublishedFileId)

CMsgClientUCMPublishedFileSubscribed = _reflection.GeneratedProtocolMessageType('CMsgClientUCMPublishedFileSubscribed', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMPUBLISHEDFILESUBSCRIBED,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMPublishedFileSubscribed)
  ))
_sym_db.RegisterMessage(CMsgClientUCMPublishedFileSubscribed)

CMsgClientUCMPublishedFileUnsubscribed = _reflection.GeneratedProtocolMessageType('CMsgClientUCMPublishedFileUnsubscribed', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMPUBLISHEDFILEUNSUBSCRIBED,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMPublishedFileUnsubscribed)
  ))
_sym_db.RegisterMessage(CMsgClientUCMPublishedFileUnsubscribed)

CMsgClientUCMPublishedFileDeleted = _reflection.GeneratedProtocolMessageType('CMsgClientUCMPublishedFileDeleted', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMPUBLISHEDFILEDELETED,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMPublishedFileDeleted)
  ))
_sym_db.RegisterMessage(CMsgClientUCMPublishedFileDeleted)

CMsgClientUCMPublishedFileUpdated = _reflection.GeneratedProtocolMessageType('CMsgClientUCMPublishedFileUpdated', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMPUBLISHEDFILEUPDATED,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMPublishedFileUpdated)
  ))
_sym_db.RegisterMessage(CMsgClientUCMPublishedFileUpdated)

CMsgClientWorkshopItemChangesRequest = _reflection.GeneratedProtocolMessageType('CMsgClientWorkshopItemChangesRequest', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTWORKSHOPITEMCHANGESREQUEST,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientWorkshopItemChangesRequest)
  ))
_sym_db.RegisterMessage(CMsgClientWorkshopItemChangesRequest)

CMsgClientWorkshopItemChangesResponse = _reflection.GeneratedProtocolMessageType('CMsgClientWorkshopItemChangesResponse', (_message.Message,), dict(

  WorkshopItemInfo = _reflection.GeneratedProtocolMessageType('WorkshopItemInfo', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTWORKSHOPITEMCHANGESRESPONSE_WORKSHOPITEMINFO,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientWorkshopItemChangesResponse.WorkshopItemInfo)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTWORKSHOPITEMCHANGESRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientWorkshopItemChangesResponse)
  ))
_sym_db.RegisterMessage(CMsgClientWorkshopItemChangesResponse)
_sym_db.RegisterMessage(CMsgClientWorkshopItemChangesResponse.WorkshopItemInfo)

CMsgClientWorkshopItemInfoRequest = _reflection.GeneratedProtocolMessageType('CMsgClientWorkshopItemInfoRequest', (_message.Message,), dict(

  WorkshopItem = _reflection.GeneratedProtocolMessageType('WorkshopItem', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTWORKSHOPITEMINFOREQUEST_WORKSHOPITEM,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientWorkshopItemInfoRequest.WorkshopItem)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTWORKSHOPITEMINFOREQUEST,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientWorkshopItemInfoRequest)
  ))
_sym_db.RegisterMessage(CMsgClientWorkshopItemInfoRequest)
_sym_db.RegisterMessage(CMsgClientWorkshopItemInfoRequest.WorkshopItem)

CMsgClientWorkshopItemInfoResponse = _reflection.GeneratedProtocolMessageType('CMsgClientWorkshopItemInfoResponse', (_message.Message,), dict(

  WorkshopItemInfo = _reflection.GeneratedProtocolMessageType('WorkshopItemInfo', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTWORKSHOPITEMINFORESPONSE_WORKSHOPITEMINFO,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientWorkshopItemInfoResponse.WorkshopItemInfo)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTWORKSHOPITEMINFORESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientWorkshopItemInfoResponse)
  ))
_sym_db.RegisterMessage(CMsgClientWorkshopItemInfoResponse)
_sym_db.RegisterMessage(CMsgClientWorkshopItemInfoResponse.WorkshopItemInfo)

CMsgClientUCMGetPublishedFilesForUser = _reflection.GeneratedProtocolMessageType('CMsgClientUCMGetPublishedFilesForUser', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMGETPUBLISHEDFILESFORUSER,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMGetPublishedFilesForUser)
  ))
_sym_db.RegisterMessage(CMsgClientUCMGetPublishedFilesForUser)

CMsgClientUCMGetPublishedFilesForUserResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUCMGetPublishedFilesForUserResponse', (_message.Message,), dict(

  PublishedFileId = _reflection.GeneratedProtocolMessageType('PublishedFileId', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTUCMGETPUBLISHEDFILESFORUSERRESPONSE_PUBLISHEDFILEID,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientUCMGetPublishedFilesForUserResponse.PublishedFileId)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTUCMGETPUBLISHEDFILESFORUSERRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMGetPublishedFilesForUserResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUCMGetPublishedFilesForUserResponse)
_sym_db.RegisterMessage(CMsgClientUCMGetPublishedFilesForUserResponse.PublishedFileId)

CMsgClientUCMSetUserPublishedFileAction = _reflection.GeneratedProtocolMessageType('CMsgClientUCMSetUserPublishedFileAction', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMSETUSERPUBLISHEDFILEACTION,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMSetUserPublishedFileAction)
  ))
_sym_db.RegisterMessage(CMsgClientUCMSetUserPublishedFileAction)

CMsgClientUCMSetUserPublishedFileActionResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUCMSetUserPublishedFileActionResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMSETUSERPUBLISHEDFILEACTIONRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMSetUserPublishedFileActionResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUCMSetUserPublishedFileActionResponse)

CMsgClientUCMEnumeratePublishedFilesByUserAction = _reflection.GeneratedProtocolMessageType('CMsgClientUCMEnumeratePublishedFilesByUserAction', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTION,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMEnumeratePublishedFilesByUserAction)
  ))
_sym_db.RegisterMessage(CMsgClientUCMEnumeratePublishedFilesByUserAction)

CMsgClientUCMEnumeratePublishedFilesByUserActionResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUCMEnumeratePublishedFilesByUserActionResponse', (_message.Message,), dict(

  PublishedFileId = _reflection.GeneratedProtocolMessageType('PublishedFileId', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTIONRESPONSE_PUBLISHEDFILEID,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientUCMEnumeratePublishedFilesByUserActionResponse.PublishedFileId)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTUCMENUMERATEPUBLISHEDFILESBYUSERACTIONRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUCMEnumeratePublishedFilesByUserActionResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUCMEnumeratePublishedFilesByUserActionResponse)
_sym_db.RegisterMessage(CMsgClientUCMEnumeratePublishedFilesByUserActionResponse.PublishedFileId)

CMsgClientScreenshotsChanged = _reflection.GeneratedProtocolMessageType('CMsgClientScreenshotsChanged', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTSCREENSHOTSCHANGED,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientScreenshotsChanged)
  ))
_sym_db.RegisterMessage(CMsgClientScreenshotsChanged)

CMsgClientUpdateUserGameInfo = _reflection.GeneratedProtocolMessageType('CMsgClientUpdateUserGameInfo', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUPDATEUSERGAMEINFO,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUpdateUserGameInfo)
  ))
_sym_db.RegisterMessage(CMsgClientUpdateUserGameInfo)

CMsgClientRichPresenceUpload = _reflection.GeneratedProtocolMessageType('CMsgClientRichPresenceUpload', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTRICHPRESENCEUPLOAD,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRichPresenceUpload)
  ))
_sym_db.RegisterMessage(CMsgClientRichPresenceUpload)

CMsgClientRichPresenceRequest = _reflection.GeneratedProtocolMessageType('CMsgClientRichPresenceRequest', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTRICHPRESENCEREQUEST,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRichPresenceRequest)
  ))
_sym_db.RegisterMessage(CMsgClientRichPresenceRequest)

CMsgClientRichPresenceInfo = _reflection.GeneratedProtocolMessageType('CMsgClientRichPresenceInfo', (_message.Message,), dict(

  RichPresence = _reflection.GeneratedProtocolMessageType('RichPresence', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTRICHPRESENCEINFO_RICHPRESENCE,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientRichPresenceInfo.RichPresence)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTRICHPRESENCEINFO,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRichPresenceInfo)
  ))
_sym_db.RegisterMessage(CMsgClientRichPresenceInfo)
_sym_db.RegisterMessage(CMsgClientRichPresenceInfo.RichPresence)

CMsgClientCheckFileSignature = _reflection.GeneratedProtocolMessageType('CMsgClientCheckFileSignature', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTCHECKFILESIGNATURE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientCheckFileSignature)
  ))
_sym_db.RegisterMessage(CMsgClientCheckFileSignature)

CMsgClientCheckFileSignatureResponse = _reflection.GeneratedProtocolMessageType('CMsgClientCheckFileSignatureResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTCHECKFILESIGNATURERESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientCheckFileSignatureResponse)
  ))
_sym_db.RegisterMessage(CMsgClientCheckFileSignatureResponse)

CMsgClientReadMachineAuth = _reflection.GeneratedProtocolMessageType('CMsgClientReadMachineAuth', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREADMACHINEAUTH,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientReadMachineAuth)
  ))
_sym_db.RegisterMessage(CMsgClientReadMachineAuth)

CMsgClientReadMachineAuthResponse = _reflection.GeneratedProtocolMessageType('CMsgClientReadMachineAuthResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREADMACHINEAUTHRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientReadMachineAuthResponse)
  ))
_sym_db.RegisterMessage(CMsgClientReadMachineAuthResponse)

CMsgClientUpdateMachineAuth = _reflection.GeneratedProtocolMessageType('CMsgClientUpdateMachineAuth', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUPDATEMACHINEAUTH,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUpdateMachineAuth)
  ))
_sym_db.RegisterMessage(CMsgClientUpdateMachineAuth)

CMsgClientUpdateMachineAuthResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUpdateMachineAuthResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUPDATEMACHINEAUTHRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUpdateMachineAuthResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUpdateMachineAuthResponse)

CMsgClientRequestMachineAuth = _reflection.GeneratedProtocolMessageType('CMsgClientRequestMachineAuth', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREQUESTMACHINEAUTH,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRequestMachineAuth)
  ))
_sym_db.RegisterMessage(CMsgClientRequestMachineAuth)

CMsgClientRequestMachineAuthResponse = _reflection.GeneratedProtocolMessageType('CMsgClientRequestMachineAuthResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREQUESTMACHINEAUTHRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRequestMachineAuthResponse)
  ))
_sym_db.RegisterMessage(CMsgClientRequestMachineAuthResponse)

CMsgClientCreateFriendsGroup = _reflection.GeneratedProtocolMessageType('CMsgClientCreateFriendsGroup', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTCREATEFRIENDSGROUP,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientCreateFriendsGroup)
  ))
_sym_db.RegisterMessage(CMsgClientCreateFriendsGroup)

CMsgClientCreateFriendsGroupResponse = _reflection.GeneratedProtocolMessageType('CMsgClientCreateFriendsGroupResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTCREATEFRIENDSGROUPRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientCreateFriendsGroupResponse)
  ))
_sym_db.RegisterMessage(CMsgClientCreateFriendsGroupResponse)

CMsgClientDeleteFriendsGroup = _reflection.GeneratedProtocolMessageType('CMsgClientDeleteFriendsGroup', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTDELETEFRIENDSGROUP,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientDeleteFriendsGroup)
  ))
_sym_db.RegisterMessage(CMsgClientDeleteFriendsGroup)

CMsgClientDeleteFriendsGroupResponse = _reflection.GeneratedProtocolMessageType('CMsgClientDeleteFriendsGroupResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTDELETEFRIENDSGROUPRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientDeleteFriendsGroupResponse)
  ))
_sym_db.RegisterMessage(CMsgClientDeleteFriendsGroupResponse)

CMsgClientRenameFriendsGroup = _reflection.GeneratedProtocolMessageType('CMsgClientRenameFriendsGroup', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTRENAMEFRIENDSGROUP,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRenameFriendsGroup)
  ))
_sym_db.RegisterMessage(CMsgClientRenameFriendsGroup)

CMsgClientRenameFriendsGroupResponse = _reflection.GeneratedProtocolMessageType('CMsgClientRenameFriendsGroupResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTRENAMEFRIENDSGROUPRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRenameFriendsGroupResponse)
  ))
_sym_db.RegisterMessage(CMsgClientRenameFriendsGroupResponse)

CMsgClientAddFriendToGroup = _reflection.GeneratedProtocolMessageType('CMsgClientAddFriendToGroup', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTADDFRIENDTOGROUP,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientAddFriendToGroup)
  ))
_sym_db.RegisterMessage(CMsgClientAddFriendToGroup)

CMsgClientAddFriendToGroupResponse = _reflection.GeneratedProtocolMessageType('CMsgClientAddFriendToGroupResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTADDFRIENDTOGROUPRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientAddFriendToGroupResponse)
  ))
_sym_db.RegisterMessage(CMsgClientAddFriendToGroupResponse)

CMsgClientRemoveFriendFromGroup = _reflection.GeneratedProtocolMessageType('CMsgClientRemoveFriendFromGroup', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREMOVEFRIENDFROMGROUP,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRemoveFriendFromGroup)
  ))
_sym_db.RegisterMessage(CMsgClientRemoveFriendFromGroup)

CMsgClientRemoveFriendFromGroupResponse = _reflection.GeneratedProtocolMessageType('CMsgClientRemoveFriendFromGroupResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREMOVEFRIENDFROMGROUPRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRemoveFriendFromGroupResponse)
  ))
_sym_db.RegisterMessage(CMsgClientRemoveFriendFromGroupResponse)

CMsgClientRegisterKey = _reflection.GeneratedProtocolMessageType('CMsgClientRegisterKey', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREGISTERKEY,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRegisterKey)
  ))
_sym_db.RegisterMessage(CMsgClientRegisterKey)

CMsgClientPurchaseResponse = _reflection.GeneratedProtocolMessageType('CMsgClientPurchaseResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTPURCHASERESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientPurchaseResponse)
  ))
_sym_db.RegisterMessage(CMsgClientPurchaseResponse)

CMsgClientActivateOEMLicense = _reflection.GeneratedProtocolMessageType('CMsgClientActivateOEMLicense', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTACTIVATEOEMLICENSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientActivateOEMLicense)
  ))
_sym_db.RegisterMessage(CMsgClientActivateOEMLicense)

CMsgClientRegisterOEMMachine = _reflection.GeneratedProtocolMessageType('CMsgClientRegisterOEMMachine', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREGISTEROEMMACHINE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRegisterOEMMachine)
  ))
_sym_db.RegisterMessage(CMsgClientRegisterOEMMachine)

CMsgClientRegisterOEMMachineResponse = _reflection.GeneratedProtocolMessageType('CMsgClientRegisterOEMMachineResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREGISTEROEMMACHINERESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRegisterOEMMachineResponse)
  ))
_sym_db.RegisterMessage(CMsgClientRegisterOEMMachineResponse)

CMsgClientPurchaseWithMachineID = _reflection.GeneratedProtocolMessageType('CMsgClientPurchaseWithMachineID', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTPURCHASEWITHMACHINEID,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientPurchaseWithMachineID)
  ))
_sym_db.RegisterMessage(CMsgClientPurchaseWithMachineID)

CMsgTrading_InitiateTradeRequest = _reflection.GeneratedProtocolMessageType('CMsgTrading_InitiateTradeRequest', (_message.Message,), dict(
  DESCRIPTOR = _CMSGTRADING_INITIATETRADEREQUEST,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgTrading_InitiateTradeRequest)
  ))
_sym_db.RegisterMessage(CMsgTrading_InitiateTradeRequest)

CMsgTrading_InitiateTradeResponse = _reflection.GeneratedProtocolMessageType('CMsgTrading_InitiateTradeResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGTRADING_INITIATETRADERESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgTrading_InitiateTradeResponse)
  ))
_sym_db.RegisterMessage(CMsgTrading_InitiateTradeResponse)

CMsgTrading_CancelTradeRequest = _reflection.GeneratedProtocolMessageType('CMsgTrading_CancelTradeRequest', (_message.Message,), dict(
  DESCRIPTOR = _CMSGTRADING_CANCELTRADEREQUEST,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgTrading_CancelTradeRequest)
  ))
_sym_db.RegisterMessage(CMsgTrading_CancelTradeRequest)

CMsgTrading_StartSession = _reflection.GeneratedProtocolMessageType('CMsgTrading_StartSession', (_message.Message,), dict(
  DESCRIPTOR = _CMSGTRADING_STARTSESSION,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgTrading_StartSession)
  ))
_sym_db.RegisterMessage(CMsgTrading_StartSession)

CMsgClientEmailChange = _reflection.GeneratedProtocolMessageType('CMsgClientEmailChange', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTEMAILCHANGE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientEmailChange)
  ))
_sym_db.RegisterMessage(CMsgClientEmailChange)

CMsgClientEmailChangeResponse = _reflection.GeneratedProtocolMessageType('CMsgClientEmailChangeResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTEMAILCHANGERESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientEmailChangeResponse)
  ))
_sym_db.RegisterMessage(CMsgClientEmailChangeResponse)

CMsgClientGetCDNAuthToken = _reflection.GeneratedProtocolMessageType('CMsgClientGetCDNAuthToken', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTGETCDNAUTHTOKEN,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientGetCDNAuthToken)
  ))
_sym_db.RegisterMessage(CMsgClientGetCDNAuthToken)

CMsgClientGetDepotDecryptionKey = _reflection.GeneratedProtocolMessageType('CMsgClientGetDepotDecryptionKey', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTGETDEPOTDECRYPTIONKEY,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientGetDepotDecryptionKey)
  ))
_sym_db.RegisterMessage(CMsgClientGetDepotDecryptionKey)

CMsgClientGetDepotDecryptionKeyResponse = _reflection.GeneratedProtocolMessageType('CMsgClientGetDepotDecryptionKeyResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTGETDEPOTDECRYPTIONKEYRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientGetDepotDecryptionKeyResponse)
  ))
_sym_db.RegisterMessage(CMsgClientGetDepotDecryptionKeyResponse)

CMsgClientCheckAppBetaPassword = _reflection.GeneratedProtocolMessageType('CMsgClientCheckAppBetaPassword', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTCHECKAPPBETAPASSWORD,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientCheckAppBetaPassword)
  ))
_sym_db.RegisterMessage(CMsgClientCheckAppBetaPassword)

CMsgClientCheckAppBetaPasswordResponse = _reflection.GeneratedProtocolMessageType('CMsgClientCheckAppBetaPasswordResponse', (_message.Message,), dict(

  BetaPassword = _reflection.GeneratedProtocolMessageType('BetaPassword', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTCHECKAPPBETAPASSWORDRESPONSE_BETAPASSWORD,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientCheckAppBetaPasswordResponse.BetaPassword)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTCHECKAPPBETAPASSWORDRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientCheckAppBetaPasswordResponse)
  ))
_sym_db.RegisterMessage(CMsgClientCheckAppBetaPasswordResponse)
_sym_db.RegisterMessage(CMsgClientCheckAppBetaPasswordResponse.BetaPassword)

CMsgClientUpdateAppJobReport = _reflection.GeneratedProtocolMessageType('CMsgClientUpdateAppJobReport', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUPDATEAPPJOBREPORT,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUpdateAppJobReport)
  ))
_sym_db.RegisterMessage(CMsgClientUpdateAppJobReport)

CMsgClientDPContentStatsReport = _reflection.GeneratedProtocolMessageType('CMsgClientDPContentStatsReport', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTDPCONTENTSTATSREPORT,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientDPContentStatsReport)
  ))
_sym_db.RegisterMessage(CMsgClientDPContentStatsReport)

CMsgClientGetCDNAuthTokenResponse = _reflection.GeneratedProtocolMessageType('CMsgClientGetCDNAuthTokenResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTGETCDNAUTHTOKENRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientGetCDNAuthTokenResponse)
  ))
_sym_db.RegisterMessage(CMsgClientGetCDNAuthTokenResponse)

CMsgDownloadRateStatistics = _reflection.GeneratedProtocolMessageType('CMsgDownloadRateStatistics', (_message.Message,), dict(

  StatsInfo = _reflection.GeneratedProtocolMessageType('StatsInfo', (_message.Message,), dict(
    DESCRIPTOR = _CMSGDOWNLOADRATESTATISTICS_STATSINFO,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgDownloadRateStatistics.StatsInfo)
    ))
  ,
  DESCRIPTOR = _CMSGDOWNLOADRATESTATISTICS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgDownloadRateStatistics)
  ))
_sym_db.RegisterMessage(CMsgDownloadRateStatistics)
_sym_db.RegisterMessage(CMsgDownloadRateStatistics.StatsInfo)

CMsgClientRequestAccountData = _reflection.GeneratedProtocolMessageType('CMsgClientRequestAccountData', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREQUESTACCOUNTDATA,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRequestAccountData)
  ))
_sym_db.RegisterMessage(CMsgClientRequestAccountData)

CMsgClientRequestAccountDataResponse = _reflection.GeneratedProtocolMessageType('CMsgClientRequestAccountDataResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREQUESTACCOUNTDATARESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRequestAccountDataResponse)
  ))
_sym_db.RegisterMessage(CMsgClientRequestAccountDataResponse)

CMsgClientUGSGetGlobalStats = _reflection.GeneratedProtocolMessageType('CMsgClientUGSGetGlobalStats', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUGSGETGLOBALSTATS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUGSGetGlobalStats)
  ))
_sym_db.RegisterMessage(CMsgClientUGSGetGlobalStats)

CMsgClientUGSGetGlobalStatsResponse = _reflection.GeneratedProtocolMessageType('CMsgClientUGSGetGlobalStatsResponse', (_message.Message,), dict(

  Day = _reflection.GeneratedProtocolMessageType('Day', (_message.Message,), dict(

    Stat = _reflection.GeneratedProtocolMessageType('Stat', (_message.Message,), dict(
      DESCRIPTOR = _CMSGCLIENTUGSGETGLOBALSTATSRESPONSE_DAY_STAT,
      __module__ = 'steammessages_clientserver_2_pb2'
      # @@protoc_insertion_point(class_scope:CMsgClientUGSGetGlobalStatsResponse.Day.Stat)
      ))
    ,
    DESCRIPTOR = _CMSGCLIENTUGSGETGLOBALSTATSRESPONSE_DAY,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientUGSGetGlobalStatsResponse.Day)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTUGSGETGLOBALSTATSRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUGSGetGlobalStatsResponse)
  ))
_sym_db.RegisterMessage(CMsgClientUGSGetGlobalStatsResponse)
_sym_db.RegisterMessage(CMsgClientUGSGetGlobalStatsResponse.Day)
_sym_db.RegisterMessage(CMsgClientUGSGetGlobalStatsResponse.Day.Stat)

CMsgGameServerData = _reflection.GeneratedProtocolMessageType('CMsgGameServerData', (_message.Message,), dict(

  Player = _reflection.GeneratedProtocolMessageType('Player', (_message.Message,), dict(
    DESCRIPTOR = _CMSGGAMESERVERDATA_PLAYER,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgGameServerData.Player)
    ))
  ,
  DESCRIPTOR = _CMSGGAMESERVERDATA,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGameServerData)
  ))
_sym_db.RegisterMessage(CMsgGameServerData)
_sym_db.RegisterMessage(CMsgGameServerData.Player)

CMsgGameServerRemove = _reflection.GeneratedProtocolMessageType('CMsgGameServerRemove', (_message.Message,), dict(
  DESCRIPTOR = _CMSGGAMESERVERREMOVE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGameServerRemove)
  ))
_sym_db.RegisterMessage(CMsgGameServerRemove)

CMsgClientGMSServerQuery = _reflection.GeneratedProtocolMessageType('CMsgClientGMSServerQuery', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTGMSSERVERQUERY,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientGMSServerQuery)
  ))
_sym_db.RegisterMessage(CMsgClientGMSServerQuery)

CMsgGMSClientServerQueryResponse = _reflection.GeneratedProtocolMessageType('CMsgGMSClientServerQueryResponse', (_message.Message,), dict(

  Server = _reflection.GeneratedProtocolMessageType('Server', (_message.Message,), dict(
    DESCRIPTOR = _CMSGGMSCLIENTSERVERQUERYRESPONSE_SERVER,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgGMSClientServerQueryResponse.Server)
    ))
  ,
  DESCRIPTOR = _CMSGGMSCLIENTSERVERQUERYRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGMSClientServerQueryResponse)
  ))
_sym_db.RegisterMessage(CMsgGMSClientServerQueryResponse)
_sym_db.RegisterMessage(CMsgGMSClientServerQueryResponse.Server)

CMsgGameServerOutOfDate = _reflection.GeneratedProtocolMessageType('CMsgGameServerOutOfDate', (_message.Message,), dict(
  DESCRIPTOR = _CMSGGAMESERVEROUTOFDATE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGameServerOutOfDate)
  ))
_sym_db.RegisterMessage(CMsgGameServerOutOfDate)

CMsgClientRedeemGuestPass = _reflection.GeneratedProtocolMessageType('CMsgClientRedeemGuestPass', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREDEEMGUESTPASS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRedeemGuestPass)
  ))
_sym_db.RegisterMessage(CMsgClientRedeemGuestPass)

CMsgClientRedeemGuestPassResponse = _reflection.GeneratedProtocolMessageType('CMsgClientRedeemGuestPassResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREDEEMGUESTPASSRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRedeemGuestPassResponse)
  ))
_sym_db.RegisterMessage(CMsgClientRedeemGuestPassResponse)

CMsgClientGetClanActivityCounts = _reflection.GeneratedProtocolMessageType('CMsgClientGetClanActivityCounts', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTGETCLANACTIVITYCOUNTS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientGetClanActivityCounts)
  ))
_sym_db.RegisterMessage(CMsgClientGetClanActivityCounts)

CMsgClientGetClanActivityCountsResponse = _reflection.GeneratedProtocolMessageType('CMsgClientGetClanActivityCountsResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTGETCLANACTIVITYCOUNTSRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientGetClanActivityCountsResponse)
  ))
_sym_db.RegisterMessage(CMsgClientGetClanActivityCountsResponse)

CMsgClientOGSReportString = _reflection.GeneratedProtocolMessageType('CMsgClientOGSReportString', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTOGSREPORTSTRING,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientOGSReportString)
  ))
_sym_db.RegisterMessage(CMsgClientOGSReportString)

CMsgClientOGSReportBug = _reflection.GeneratedProtocolMessageType('CMsgClientOGSReportBug', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTOGSREPORTBUG,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientOGSReportBug)
  ))
_sym_db.RegisterMessage(CMsgClientOGSReportBug)

CMsgGSAssociateWithClan = _reflection.GeneratedProtocolMessageType('CMsgGSAssociateWithClan', (_message.Message,), dict(
  DESCRIPTOR = _CMSGGSASSOCIATEWITHCLAN,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGSAssociateWithClan)
  ))
_sym_db.RegisterMessage(CMsgGSAssociateWithClan)

CMsgGSAssociateWithClanResponse = _reflection.GeneratedProtocolMessageType('CMsgGSAssociateWithClanResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGGSASSOCIATEWITHCLANRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGSAssociateWithClanResponse)
  ))
_sym_db.RegisterMessage(CMsgGSAssociateWithClanResponse)

CMsgGSComputeNewPlayerCompatibility = _reflection.GeneratedProtocolMessageType('CMsgGSComputeNewPlayerCompatibility', (_message.Message,), dict(
  DESCRIPTOR = _CMSGGSCOMPUTENEWPLAYERCOMPATIBILITY,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGSComputeNewPlayerCompatibility)
  ))
_sym_db.RegisterMessage(CMsgGSComputeNewPlayerCompatibility)

CMsgGSComputeNewPlayerCompatibilityResponse = _reflection.GeneratedProtocolMessageType('CMsgGSComputeNewPlayerCompatibilityResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGGSCOMPUTENEWPLAYERCOMPATIBILITYRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGSComputeNewPlayerCompatibilityResponse)
  ))
_sym_db.RegisterMessage(CMsgGSComputeNewPlayerCompatibilityResponse)

CMsgClientSentLogs = _reflection.GeneratedProtocolMessageType('CMsgClientSentLogs', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTSENTLOGS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientSentLogs)
  ))
_sym_db.RegisterMessage(CMsgClientSentLogs)

CMsgGCClient = _reflection.GeneratedProtocolMessageType('CMsgGCClient', (_message.Message,), dict(
  DESCRIPTOR = _CMSGGCCLIENT,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGCClient)
  ))
_sym_db.RegisterMessage(CMsgGCClient)

CMsgClientRequestFreeLicense = _reflection.GeneratedProtocolMessageType('CMsgClientRequestFreeLicense', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREQUESTFREELICENSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRequestFreeLicense)
  ))
_sym_db.RegisterMessage(CMsgClientRequestFreeLicense)

CMsgClientRequestFreeLicenseResponse = _reflection.GeneratedProtocolMessageType('CMsgClientRequestFreeLicenseResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREQUESTFREELICENSERESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRequestFreeLicenseResponse)
  ))
_sym_db.RegisterMessage(CMsgClientRequestFreeLicenseResponse)

CMsgDRMDownloadRequestWithCrashData = _reflection.GeneratedProtocolMessageType('CMsgDRMDownloadRequestWithCrashData', (_message.Message,), dict(
  DESCRIPTOR = _CMSGDRMDOWNLOADREQUESTWITHCRASHDATA,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgDRMDownloadRequestWithCrashData)
  ))
_sym_db.RegisterMessage(CMsgDRMDownloadRequestWithCrashData)

CMsgDRMDownloadResponse = _reflection.GeneratedProtocolMessageType('CMsgDRMDownloadResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGDRMDOWNLOADRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgDRMDownloadResponse)
  ))
_sym_db.RegisterMessage(CMsgDRMDownloadResponse)

CMsgDRMFinalResult = _reflection.GeneratedProtocolMessageType('CMsgDRMFinalResult', (_message.Message,), dict(
  DESCRIPTOR = _CMSGDRMFINALRESULT,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgDRMFinalResult)
  ))
_sym_db.RegisterMessage(CMsgDRMFinalResult)

CMsgClientDPCheckSpecialSurvey = _reflection.GeneratedProtocolMessageType('CMsgClientDPCheckSpecialSurvey', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTDPCHECKSPECIALSURVEY,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientDPCheckSpecialSurvey)
  ))
_sym_db.RegisterMessage(CMsgClientDPCheckSpecialSurvey)

CMsgClientDPCheckSpecialSurveyResponse = _reflection.GeneratedProtocolMessageType('CMsgClientDPCheckSpecialSurveyResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTDPCHECKSPECIALSURVEYRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientDPCheckSpecialSurveyResponse)
  ))
_sym_db.RegisterMessage(CMsgClientDPCheckSpecialSurveyResponse)

CMsgClientDPSendSpecialSurveyResponse = _reflection.GeneratedProtocolMessageType('CMsgClientDPSendSpecialSurveyResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTDPSENDSPECIALSURVEYRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientDPSendSpecialSurveyResponse)
  ))
_sym_db.RegisterMessage(CMsgClientDPSendSpecialSurveyResponse)

CMsgClientDPSendSpecialSurveyResponseReply = _reflection.GeneratedProtocolMessageType('CMsgClientDPSendSpecialSurveyResponseReply', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTDPSENDSPECIALSURVEYRESPONSEREPLY,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientDPSendSpecialSurveyResponseReply)
  ))
_sym_db.RegisterMessage(CMsgClientDPSendSpecialSurveyResponseReply)

CMsgClientRequestForgottenPasswordEmail = _reflection.GeneratedProtocolMessageType('CMsgClientRequestForgottenPasswordEmail', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREQUESTFORGOTTENPASSWORDEMAIL,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRequestForgottenPasswordEmail)
  ))
_sym_db.RegisterMessage(CMsgClientRequestForgottenPasswordEmail)

CMsgClientRequestForgottenPasswordEmailResponse = _reflection.GeneratedProtocolMessageType('CMsgClientRequestForgottenPasswordEmailResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREQUESTFORGOTTENPASSWORDEMAILRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRequestForgottenPasswordEmailResponse)
  ))
_sym_db.RegisterMessage(CMsgClientRequestForgottenPasswordEmailResponse)

CMsgClientItemAnnouncements = _reflection.GeneratedProtocolMessageType('CMsgClientItemAnnouncements', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTITEMANNOUNCEMENTS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientItemAnnouncements)
  ))
_sym_db.RegisterMessage(CMsgClientItemAnnouncements)

CMsgClientRequestItemAnnouncements = _reflection.GeneratedProtocolMessageType('CMsgClientRequestItemAnnouncements', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREQUESTITEMANNOUNCEMENTS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRequestItemAnnouncements)
  ))
_sym_db.RegisterMessage(CMsgClientRequestItemAnnouncements)

CMsgClientUserNotifications = _reflection.GeneratedProtocolMessageType('CMsgClientUserNotifications', (_message.Message,), dict(

  Notification = _reflection.GeneratedProtocolMessageType('Notification', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTUSERNOTIFICATIONS_NOTIFICATION,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientUserNotifications.Notification)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTUSERNOTIFICATIONS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUserNotifications)
  ))
_sym_db.RegisterMessage(CMsgClientUserNotifications)
_sym_db.RegisterMessage(CMsgClientUserNotifications.Notification)

CMsgClientCommentNotifications = _reflection.GeneratedProtocolMessageType('CMsgClientCommentNotifications', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTCOMMENTNOTIFICATIONS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientCommentNotifications)
  ))
_sym_db.RegisterMessage(CMsgClientCommentNotifications)

CMsgClientRequestCommentNotifications = _reflection.GeneratedProtocolMessageType('CMsgClientRequestCommentNotifications', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREQUESTCOMMENTNOTIFICATIONS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRequestCommentNotifications)
  ))
_sym_db.RegisterMessage(CMsgClientRequestCommentNotifications)

CMsgClientOfflineMessageNotification = _reflection.GeneratedProtocolMessageType('CMsgClientOfflineMessageNotification', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTOFFLINEMESSAGENOTIFICATION,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientOfflineMessageNotification)
  ))
_sym_db.RegisterMessage(CMsgClientOfflineMessageNotification)

CMsgClientRequestOfflineMessageCount = _reflection.GeneratedProtocolMessageType('CMsgClientRequestOfflineMessageCount', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTREQUESTOFFLINEMESSAGECOUNT,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientRequestOfflineMessageCount)
  ))
_sym_db.RegisterMessage(CMsgClientRequestOfflineMessageCount)

CMsgClientFSGetFriendMessageHistory = _reflection.GeneratedProtocolMessageType('CMsgClientFSGetFriendMessageHistory', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTFSGETFRIENDMESSAGEHISTORY,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientFSGetFriendMessageHistory)
  ))
_sym_db.RegisterMessage(CMsgClientFSGetFriendMessageHistory)

CMsgClientFSGetFriendMessageHistoryResponse = _reflection.GeneratedProtocolMessageType('CMsgClientFSGetFriendMessageHistoryResponse', (_message.Message,), dict(

  FriendMessage = _reflection.GeneratedProtocolMessageType('FriendMessage', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTFSGETFRIENDMESSAGEHISTORYRESPONSE_FRIENDMESSAGE,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientFSGetFriendMessageHistoryResponse.FriendMessage)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTFSGETFRIENDMESSAGEHISTORYRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientFSGetFriendMessageHistoryResponse)
  ))
_sym_db.RegisterMessage(CMsgClientFSGetFriendMessageHistoryResponse)
_sym_db.RegisterMessage(CMsgClientFSGetFriendMessageHistoryResponse.FriendMessage)

CMsgClientFSGetFriendMessageHistoryForOfflineMessages = _reflection.GeneratedProtocolMessageType('CMsgClientFSGetFriendMessageHistoryForOfflineMessages', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTFSGETFRIENDMESSAGEHISTORYFOROFFLINEMESSAGES,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientFSGetFriendMessageHistoryForOfflineMessages)
  ))
_sym_db.RegisterMessage(CMsgClientFSGetFriendMessageHistoryForOfflineMessages)

CMsgClientFSGetFriendsSteamLevels = _reflection.GeneratedProtocolMessageType('CMsgClientFSGetFriendsSteamLevels', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTFSGETFRIENDSSTEAMLEVELS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientFSGetFriendsSteamLevels)
  ))
_sym_db.RegisterMessage(CMsgClientFSGetFriendsSteamLevels)

CMsgClientFSGetFriendsSteamLevelsResponse = _reflection.GeneratedProtocolMessageType('CMsgClientFSGetFriendsSteamLevelsResponse', (_message.Message,), dict(

  Friend = _reflection.GeneratedProtocolMessageType('Friend', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTFSGETFRIENDSSTEAMLEVELSRESPONSE_FRIEND,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientFSGetFriendsSteamLevelsResponse.Friend)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTFSGETFRIENDSSTEAMLEVELSRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientFSGetFriendsSteamLevelsResponse)
  ))
_sym_db.RegisterMessage(CMsgClientFSGetFriendsSteamLevelsResponse)
_sym_db.RegisterMessage(CMsgClientFSGetFriendsSteamLevelsResponse.Friend)

CMsgClientEmailAddrInfo = _reflection.GeneratedProtocolMessageType('CMsgClientEmailAddrInfo', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTEMAILADDRINFO,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientEmailAddrInfo)
  ))
_sym_db.RegisterMessage(CMsgClientEmailAddrInfo)

CMsgCREEnumeratePublishedFiles = _reflection.GeneratedProtocolMessageType('CMsgCREEnumeratePublishedFiles', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCREENUMERATEPUBLISHEDFILES,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgCREEnumeratePublishedFiles)
  ))
_sym_db.RegisterMessage(CMsgCREEnumeratePublishedFiles)

CMsgCREEnumeratePublishedFilesResponse = _reflection.GeneratedProtocolMessageType('CMsgCREEnumeratePublishedFilesResponse', (_message.Message,), dict(

  PublishedFileId = _reflection.GeneratedProtocolMessageType('PublishedFileId', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCREENUMERATEPUBLISHEDFILESRESPONSE_PUBLISHEDFILEID,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgCREEnumeratePublishedFilesResponse.PublishedFileId)
    ))
  ,
  DESCRIPTOR = _CMSGCREENUMERATEPUBLISHEDFILESRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgCREEnumeratePublishedFilesResponse)
  ))
_sym_db.RegisterMessage(CMsgCREEnumeratePublishedFilesResponse)
_sym_db.RegisterMessage(CMsgCREEnumeratePublishedFilesResponse.PublishedFileId)

CMsgCREItemVoteSummary = _reflection.GeneratedProtocolMessageType('CMsgCREItemVoteSummary', (_message.Message,), dict(

  PublishedFileId = _reflection.GeneratedProtocolMessageType('PublishedFileId', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCREITEMVOTESUMMARY_PUBLISHEDFILEID,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgCREItemVoteSummary.PublishedFileId)
    ))
  ,
  DESCRIPTOR = _CMSGCREITEMVOTESUMMARY,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgCREItemVoteSummary)
  ))
_sym_db.RegisterMessage(CMsgCREItemVoteSummary)
_sym_db.RegisterMessage(CMsgCREItemVoteSummary.PublishedFileId)

CMsgCREItemVoteSummaryResponse = _reflection.GeneratedProtocolMessageType('CMsgCREItemVoteSummaryResponse', (_message.Message,), dict(

  ItemVoteSummary = _reflection.GeneratedProtocolMessageType('ItemVoteSummary', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCREITEMVOTESUMMARYRESPONSE_ITEMVOTESUMMARY,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgCREItemVoteSummaryResponse.ItemVoteSummary)
    ))
  ,
  DESCRIPTOR = _CMSGCREITEMVOTESUMMARYRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgCREItemVoteSummaryResponse)
  ))
_sym_db.RegisterMessage(CMsgCREItemVoteSummaryResponse)
_sym_db.RegisterMessage(CMsgCREItemVoteSummaryResponse.ItemVoteSummary)

CMsgCREUpdateUserPublishedItemVote = _reflection.GeneratedProtocolMessageType('CMsgCREUpdateUserPublishedItemVote', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCREUPDATEUSERPUBLISHEDITEMVOTE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgCREUpdateUserPublishedItemVote)
  ))
_sym_db.RegisterMessage(CMsgCREUpdateUserPublishedItemVote)

CMsgCREUpdateUserPublishedItemVoteResponse = _reflection.GeneratedProtocolMessageType('CMsgCREUpdateUserPublishedItemVoteResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCREUPDATEUSERPUBLISHEDITEMVOTERESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgCREUpdateUserPublishedItemVoteResponse)
  ))
_sym_db.RegisterMessage(CMsgCREUpdateUserPublishedItemVoteResponse)

CMsgCREGetUserPublishedItemVoteDetails = _reflection.GeneratedProtocolMessageType('CMsgCREGetUserPublishedItemVoteDetails', (_message.Message,), dict(

  PublishedFileId = _reflection.GeneratedProtocolMessageType('PublishedFileId', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILS_PUBLISHEDFILEID,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgCREGetUserPublishedItemVoteDetails.PublishedFileId)
    ))
  ,
  DESCRIPTOR = _CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgCREGetUserPublishedItemVoteDetails)
  ))
_sym_db.RegisterMessage(CMsgCREGetUserPublishedItemVoteDetails)
_sym_db.RegisterMessage(CMsgCREGetUserPublishedItemVoteDetails.PublishedFileId)

CMsgCREGetUserPublishedItemVoteDetailsResponse = _reflection.GeneratedProtocolMessageType('CMsgCREGetUserPublishedItemVoteDetailsResponse', (_message.Message,), dict(

  UserItemVoteDetail = _reflection.GeneratedProtocolMessageType('UserItemVoteDetail', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILSRESPONSE_USERITEMVOTEDETAIL,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgCREGetUserPublishedItemVoteDetailsResponse.UserItemVoteDetail)
    ))
  ,
  DESCRIPTOR = _CMSGCREGETUSERPUBLISHEDITEMVOTEDETAILSRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgCREGetUserPublishedItemVoteDetailsResponse)
  ))
_sym_db.RegisterMessage(CMsgCREGetUserPublishedItemVoteDetailsResponse)
_sym_db.RegisterMessage(CMsgCREGetUserPublishedItemVoteDetailsResponse.UserItemVoteDetail)

CMsgGameServerPingSample = _reflection.GeneratedProtocolMessageType('CMsgGameServerPingSample', (_message.Message,), dict(

  Sample = _reflection.GeneratedProtocolMessageType('Sample', (_message.Message,), dict(
    DESCRIPTOR = _CMSGGAMESERVERPINGSAMPLE_SAMPLE,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgGameServerPingSample.Sample)
    ))
  ,
  DESCRIPTOR = _CMSGGAMESERVERPINGSAMPLE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGameServerPingSample)
  ))
_sym_db.RegisterMessage(CMsgGameServerPingSample)
_sym_db.RegisterMessage(CMsgGameServerPingSample.Sample)

CMsgFSGetFollowerCount = _reflection.GeneratedProtocolMessageType('CMsgFSGetFollowerCount', (_message.Message,), dict(
  DESCRIPTOR = _CMSGFSGETFOLLOWERCOUNT,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgFSGetFollowerCount)
  ))
_sym_db.RegisterMessage(CMsgFSGetFollowerCount)

CMsgFSGetFollowerCountResponse = _reflection.GeneratedProtocolMessageType('CMsgFSGetFollowerCountResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGFSGETFOLLOWERCOUNTRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgFSGetFollowerCountResponse)
  ))
_sym_db.RegisterMessage(CMsgFSGetFollowerCountResponse)

CMsgFSGetIsFollowing = _reflection.GeneratedProtocolMessageType('CMsgFSGetIsFollowing', (_message.Message,), dict(
  DESCRIPTOR = _CMSGFSGETISFOLLOWING,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgFSGetIsFollowing)
  ))
_sym_db.RegisterMessage(CMsgFSGetIsFollowing)

CMsgFSGetIsFollowingResponse = _reflection.GeneratedProtocolMessageType('CMsgFSGetIsFollowingResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGFSGETISFOLLOWINGRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgFSGetIsFollowingResponse)
  ))
_sym_db.RegisterMessage(CMsgFSGetIsFollowingResponse)

CMsgFSEnumerateFollowingList = _reflection.GeneratedProtocolMessageType('CMsgFSEnumerateFollowingList', (_message.Message,), dict(
  DESCRIPTOR = _CMSGFSENUMERATEFOLLOWINGLIST,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgFSEnumerateFollowingList)
  ))
_sym_db.RegisterMessage(CMsgFSEnumerateFollowingList)

CMsgFSEnumerateFollowingListResponse = _reflection.GeneratedProtocolMessageType('CMsgFSEnumerateFollowingListResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGFSENUMERATEFOLLOWINGLISTRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgFSEnumerateFollowingListResponse)
  ))
_sym_db.RegisterMessage(CMsgFSEnumerateFollowingListResponse)

CMsgDPGetNumberOfCurrentPlayers = _reflection.GeneratedProtocolMessageType('CMsgDPGetNumberOfCurrentPlayers', (_message.Message,), dict(
  DESCRIPTOR = _CMSGDPGETNUMBEROFCURRENTPLAYERS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgDPGetNumberOfCurrentPlayers)
  ))
_sym_db.RegisterMessage(CMsgDPGetNumberOfCurrentPlayers)

CMsgDPGetNumberOfCurrentPlayersResponse = _reflection.GeneratedProtocolMessageType('CMsgDPGetNumberOfCurrentPlayersResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGDPGETNUMBEROFCURRENTPLAYERSRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgDPGetNumberOfCurrentPlayersResponse)
  ))
_sym_db.RegisterMessage(CMsgDPGetNumberOfCurrentPlayersResponse)

CMsgClientFriendUserStatusPublished = _reflection.GeneratedProtocolMessageType('CMsgClientFriendUserStatusPublished', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTFRIENDUSERSTATUSPUBLISHED,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientFriendUserStatusPublished)
  ))
_sym_db.RegisterMessage(CMsgClientFriendUserStatusPublished)

CMsgClientServiceMethod = _reflection.GeneratedProtocolMessageType('CMsgClientServiceMethod', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTSERVICEMETHOD,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientServiceMethod)
  ))
_sym_db.RegisterMessage(CMsgClientServiceMethod)

CMsgClientServiceMethodResponse = _reflection.GeneratedProtocolMessageType('CMsgClientServiceMethodResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTSERVICEMETHODRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientServiceMethodResponse)
  ))
_sym_db.RegisterMessage(CMsgClientServiceMethodResponse)

CMsgClientUIMode = _reflection.GeneratedProtocolMessageType('CMsgClientUIMode', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTUIMODE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUIMode)
  ))
_sym_db.RegisterMessage(CMsgClientUIMode)

CMsgClientVanityURLChangedNotification = _reflection.GeneratedProtocolMessageType('CMsgClientVanityURLChangedNotification', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTVANITYURLCHANGEDNOTIFICATION,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientVanityURLChangedNotification)
  ))
_sym_db.RegisterMessage(CMsgClientVanityURLChangedNotification)

CMsgClientAuthorizeLocalDeviceRequest = _reflection.GeneratedProtocolMessageType('CMsgClientAuthorizeLocalDeviceRequest', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTAUTHORIZELOCALDEVICEREQUEST,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientAuthorizeLocalDeviceRequest)
  ))
_sym_db.RegisterMessage(CMsgClientAuthorizeLocalDeviceRequest)

CMsgClientAuthorizeLocalDevice = _reflection.GeneratedProtocolMessageType('CMsgClientAuthorizeLocalDevice', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTAUTHORIZELOCALDEVICE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientAuthorizeLocalDevice)
  ))
_sym_db.RegisterMessage(CMsgClientAuthorizeLocalDevice)

CMsgClientAuthorizeLocalDeviceNotification = _reflection.GeneratedProtocolMessageType('CMsgClientAuthorizeLocalDeviceNotification', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTAUTHORIZELOCALDEVICENOTIFICATION,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientAuthorizeLocalDeviceNotification)
  ))
_sym_db.RegisterMessage(CMsgClientAuthorizeLocalDeviceNotification)

CMsgClientDeauthorizeDeviceRequest = _reflection.GeneratedProtocolMessageType('CMsgClientDeauthorizeDeviceRequest', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTDEAUTHORIZEDEVICEREQUEST,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientDeauthorizeDeviceRequest)
  ))
_sym_db.RegisterMessage(CMsgClientDeauthorizeDeviceRequest)

CMsgClientDeauthorizeDevice = _reflection.GeneratedProtocolMessageType('CMsgClientDeauthorizeDevice', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTDEAUTHORIZEDEVICE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientDeauthorizeDevice)
  ))
_sym_db.RegisterMessage(CMsgClientDeauthorizeDevice)

CMsgClientUseLocalDeviceAuthorizations = _reflection.GeneratedProtocolMessageType('CMsgClientUseLocalDeviceAuthorizations', (_message.Message,), dict(

  DeviceToken = _reflection.GeneratedProtocolMessageType('DeviceToken', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTUSELOCALDEVICEAUTHORIZATIONS_DEVICETOKEN,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientUseLocalDeviceAuthorizations.DeviceToken)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTUSELOCALDEVICEAUTHORIZATIONS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientUseLocalDeviceAuthorizations)
  ))
_sym_db.RegisterMessage(CMsgClientUseLocalDeviceAuthorizations)
_sym_db.RegisterMessage(CMsgClientUseLocalDeviceAuthorizations.DeviceToken)

CMsgClientGetAuthorizedDevices = _reflection.GeneratedProtocolMessageType('CMsgClientGetAuthorizedDevices', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTGETAUTHORIZEDDEVICES,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientGetAuthorizedDevices)
  ))
_sym_db.RegisterMessage(CMsgClientGetAuthorizedDevices)

CMsgClientGetAuthorizedDevicesResponse = _reflection.GeneratedProtocolMessageType('CMsgClientGetAuthorizedDevicesResponse', (_message.Message,), dict(

  AuthorizedDevice = _reflection.GeneratedProtocolMessageType('AuthorizedDevice', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTGETAUTHORIZEDDEVICESRESPONSE_AUTHORIZEDDEVICE,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientGetAuthorizedDevicesResponse.AuthorizedDevice)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTGETAUTHORIZEDDEVICESRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientGetAuthorizedDevicesResponse)
  ))
_sym_db.RegisterMessage(CMsgClientGetAuthorizedDevicesResponse)
_sym_db.RegisterMessage(CMsgClientGetAuthorizedDevicesResponse.AuthorizedDevice)

CMsgClientGetEmoticonList = _reflection.GeneratedProtocolMessageType('CMsgClientGetEmoticonList', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTGETEMOTICONLIST,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientGetEmoticonList)
  ))
_sym_db.RegisterMessage(CMsgClientGetEmoticonList)

CMsgClientEmoticonList = _reflection.GeneratedProtocolMessageType('CMsgClientEmoticonList', (_message.Message,), dict(

  Emoticon = _reflection.GeneratedProtocolMessageType('Emoticon', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTEMOTICONLIST_EMOTICON,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientEmoticonList.Emoticon)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTEMOTICONLIST,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientEmoticonList)
  ))
_sym_db.RegisterMessage(CMsgClientEmoticonList)
_sym_db.RegisterMessage(CMsgClientEmoticonList.Emoticon)

CMsgClientSharedLibraryLockStatus = _reflection.GeneratedProtocolMessageType('CMsgClientSharedLibraryLockStatus', (_message.Message,), dict(

  LockedLibrary = _reflection.GeneratedProtocolMessageType('LockedLibrary', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTSHAREDLIBRARYLOCKSTATUS_LOCKEDLIBRARY,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientSharedLibraryLockStatus.LockedLibrary)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTSHAREDLIBRARYLOCKSTATUS,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientSharedLibraryLockStatus)
  ))
_sym_db.RegisterMessage(CMsgClientSharedLibraryLockStatus)
_sym_db.RegisterMessage(CMsgClientSharedLibraryLockStatus.LockedLibrary)

CMsgClientSharedLibraryStopPlaying = _reflection.GeneratedProtocolMessageType('CMsgClientSharedLibraryStopPlaying', (_message.Message,), dict(

  StopApp = _reflection.GeneratedProtocolMessageType('StopApp', (_message.Message,), dict(
    DESCRIPTOR = _CMSGCLIENTSHAREDLIBRARYSTOPPLAYING_STOPAPP,
    __module__ = 'steammessages_clientserver_2_pb2'
    # @@protoc_insertion_point(class_scope:CMsgClientSharedLibraryStopPlaying.StopApp)
    ))
  ,
  DESCRIPTOR = _CMSGCLIENTSHAREDLIBRARYSTOPPLAYING,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientSharedLibraryStopPlaying)
  ))
_sym_db.RegisterMessage(CMsgClientSharedLibraryStopPlaying)
_sym_db.RegisterMessage(CMsgClientSharedLibraryStopPlaying.StopApp)

CMsgClientServiceCall = _reflection.GeneratedProtocolMessageType('CMsgClientServiceCall', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTSERVICECALL,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientServiceCall)
  ))
_sym_db.RegisterMessage(CMsgClientServiceCall)

CMsgClientServiceModule = _reflection.GeneratedProtocolMessageType('CMsgClientServiceModule', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTSERVICEMODULE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientServiceModule)
  ))
_sym_db.RegisterMessage(CMsgClientServiceModule)

CMsgClientServiceCallResponse = _reflection.GeneratedProtocolMessageType('CMsgClientServiceCallResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTSERVICECALLRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientServiceCallResponse)
  ))
_sym_db.RegisterMessage(CMsgClientServiceCallResponse)

CMsgAMUnlockStreaming = _reflection.GeneratedProtocolMessageType('CMsgAMUnlockStreaming', (_message.Message,), dict(
  DESCRIPTOR = _CMSGAMUNLOCKSTREAMING,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgAMUnlockStreaming)
  ))
_sym_db.RegisterMessage(CMsgAMUnlockStreaming)

CMsgAMUnlockStreamingResponse = _reflection.GeneratedProtocolMessageType('CMsgAMUnlockStreamingResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGAMUNLOCKSTREAMINGRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgAMUnlockStreamingResponse)
  ))
_sym_db.RegisterMessage(CMsgAMUnlockStreamingResponse)

CMsgClientPlayingSessionState = _reflection.GeneratedProtocolMessageType('CMsgClientPlayingSessionState', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTPLAYINGSESSIONSTATE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientPlayingSessionState)
  ))
_sym_db.RegisterMessage(CMsgClientPlayingSessionState)

CMsgClientKickPlayingSession = _reflection.GeneratedProtocolMessageType('CMsgClientKickPlayingSession', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTKICKPLAYINGSESSION,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientKickPlayingSession)
  ))
_sym_db.RegisterMessage(CMsgClientKickPlayingSession)

CMsgClientCreateAccount = _reflection.GeneratedProtocolMessageType('CMsgClientCreateAccount', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTCREATEACCOUNT,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientCreateAccount)
  ))
_sym_db.RegisterMessage(CMsgClientCreateAccount)

CMsgClientCreateAccountResponse = _reflection.GeneratedProtocolMessageType('CMsgClientCreateAccountResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTCREATEACCOUNTRESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientCreateAccountResponse)
  ))
_sym_db.RegisterMessage(CMsgClientCreateAccountResponse)

CMsgClientVoiceCallPreAuthorize = _reflection.GeneratedProtocolMessageType('CMsgClientVoiceCallPreAuthorize', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTVOICECALLPREAUTHORIZE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientVoiceCallPreAuthorize)
  ))
_sym_db.RegisterMessage(CMsgClientVoiceCallPreAuthorize)

CMsgClientVoiceCallPreAuthorizeResponse = _reflection.GeneratedProtocolMessageType('CMsgClientVoiceCallPreAuthorizeResponse', (_message.Message,), dict(
  DESCRIPTOR = _CMSGCLIENTVOICECALLPREAUTHORIZERESPONSE,
  __module__ = 'steammessages_clientserver_2_pb2'
  # @@protoc_insertion_point(class_scope:CMsgClientVoiceCallPreAuthorizeResponse)
  ))
_sym_db.RegisterMessage(CMsgClientVoiceCallPreAuthorizeResponse)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('H\001\220\001\000'))
# @@protoc_insertion_point(module_scope)
